/**
 * filename:analysisTool.js
 * desc:分析及量算
 * deps:jQuery
 * authors:zc
 * time:2014-09-04 14:31:43
 * version:0.0.1
 * updatelog:
 * 0.0.1:the first version
 **/


(function(ns) {
    var SeAnalysisTool = null,

        VERSION = '0.0.1',

        _analysisServerResult = null,
        _analysisLoaclResult = null,
        globalScope = ns;




    /**
     * 检查earth是否可用
     * @param {bool} earth是否必须,若为false,则空球时仍判定成功并打印一个警告(debugMode == true时)
     * @param {string} [position] 位置,一般是函数名
     * @return {bool} earth是否可用
     */
    var _checkEarth = function(necessary, position) {
        if (position == undefined) {
            position = ''
        }

        if (earth == null) {
            if (necessary) {
                if (globalScope.debugMode) {
                    console.error('err3005:earth initialize fail,position:' + position);
                }
                return false;
            } else {
                if (globalScope.debugMode) {
                    console.warn('warn3006:earth initialize fail,position:' + position);
                }
                return true;
            }
        } else {
            return true;
        }
    }

    var _valueOrDefault = function(obj, property, defaultValue) {
        try {
            if (!obj || obj[property] == undefined || obj[property] == null) {
                return defaultValue;
            }

            return obj[property];
        } catch (e) {
            return defaultValue;
        }
    }

    var _geoPoint2vector3 = function(geoPoint) {
        if (!geoPoint) {
            return null;
        }

        try {
            var v3 = earth.Factory.CreateVector3s();
            v3.X = geoPoint.Longitude;
            v3.Y = geoPoint.Latitude;
            v3.Z = geoPoint.Altitude;

            return v3;
        } catch (e) {
            return null;
        }
    }

    var _vector32point = function(v3) {
        if (!v3) {
            return null;
        }

        var pt = earth.Factory.CreateGeoPoints();
        pt.Longitude = v3.x;
        pt.Latitude = v3.y;
        pt.Altitude = v3.z;
        return pt;
    }

    /**
     * 测量结果格式化，保留三位有效数字
     */
    var _parseMeasureResult = function(mtype, result, type) {
        var unit = "",
            resultHeader = "";

        if (mtype == "HorizontalDistance") {
            resultHeader = "水平距离";
        } else if (mtype == "Height") {
            resultHeader = "垂直距离";
        } else if (mtype == "Area") {
            resultHeader = "水平面积";
        } else if (mtype == "SpatialArea") {
            resultHeader = "空间面积";
        } else if (mtype == "VerticalArea") {
            resultHeader = "立面面积";
        } else if (mtype == "PathLength") {
            resultHeader = "地表距离";
        } else if (mtype == "PlaneAngle") {
            resultHeader = "平面角度";
        } else if (mtype == "LineLength") {
            resultHeader = "空间距离";
        }

        if (type === 9) { // 角度
            unit = "度";
        } else if (type === 4 || type === 7 || type === 8) { //投影面积测量、空间面积、立面面积
            unit = "平方千米";
            if (result < 1) {
                result = result * 1000000;
                unit = "平方米";
            }
        } else { //其它测量
            unit = "千米";
            if (result < 1) {
                result = result * 1000;
                unit = "米";
            }
        }


        var resultObj = {
            measureType: resultHeader,
            measureResult: result,
            measureUnit: unit
        };

        return resultObj;
    };

    var _parseDate = function(date) {
        if (!date) {
            return null;
        }

        if (date instanceof Date) {
            return date;
        }

        try {
            return new Date(date);
        } catch (e) {
            return null;
        }
    };



    SeAnalysisTool = function(options) {
        if (!(this instanceof SeAnalysisTool)) {
            return new SeAnalysisTool(options);
        }

        this.initialize.apply(this, arguments);
    }


    $.extend(SeAnalysisTool.prototype, {
        /**
         * 初始化
         * @param  {object} options
         * options:{
         * 		earth:{object} 三维球
         * }
         * @return {void}
         */
        initialize: function(options) {
            if (options.earth == null) {
                if (globalScope.debugMode) {
                    console.warn('warn3001:null earth,position:STAMP.SeAnimationManager.initialize');
                }
            }

            earth = options.earth;
        },

        /**
         * 清除辅助图元
         * @return {void}
         */
        clearShape: function() {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.clearShape')) {
                return;
            }

            earth.ShapeCreator.Clear();
        },



        /*==========================
        =       measure 测量       =
        ==========================*/
        /**
         * 清理测量结果
         * @return {void}
         */
        clearMeasure: function() {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.clearMeasure')) {
                return;
            }

            earth.Measure.Clear();
        },

        /**
         * 退出测量状态
         * @return {void}
         */
        exitMeasure: function() {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.exitMeasure')) {
                return;
            }

            earth.Event.OnMeasureFinish = function() {};
            earth.Environment.SetCursorStyle(209);
        },

        /**
         * 测量
         * @param  {object} options
         * options:{
         * 		[measureType]:{seEnums.seMeasureType} 测量类型 default:seEnums.seMeasureType.horizontalDistance,即水平距离
         * 		[clearShape]:{bool} 测量完成后是否清除绘制的shape default:true,
         * 		[onMeasureFinish]:{function(options)} 测量完成回调
         * 		options:{
         * 			measureType:{string} 测量类型,
         * 			measureResult:{double} 测量结果,
         * 			measureUnit:{string} 单位
         * 		}
         * }
         * @return {void}
         */
        measure: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.measure')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.measure');
                }
                return;
            }

            var measureType = _valueOrDefault(options, 'measureType', 'HorizontalDistance');

            earth.Event.OnMeasureFinish = function(result, type) {
                earth.Event.OnMeasureFinish = function() {};

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                if (options.onMeasureFinish && typeof options.onMeasureFinish === 'function') {
                    options.onMeasureFinish(_parseMeasureResult(measureType, result, type));
                }
            };

            switch (measureType) {
                case "HorizontalDistance":
                    //水平距离
                    earth.Measure.MeasureHorizontalDistance();
                    break;
                case "Height":
                    //垂直距离
                    earth.Measure.MeasureHeight();
                    break;
                case "Area":
                    //水平面积
                    earth.Measure.MeasureArea();
                    break;
                case "SpatialArea":
                    //空间面积
                    earth.Measure.MeasureSpatialArea();
                    break;
                case "VerticalArea":
                    //立面面积
                    earth.Measure.MeasureVerticalArea();
                    break;
                case "PathLength":
                    //球面距离
                    earth.Measure.MeasurePathLength();
                    break;
                case "LineLength":
                    //直线距离
                    earth.Measure.MeasureLineLength();
                    break;
                case "PlaneAngle":
                    //平面角度
                    earth.Measure.MeasurePlaneAngle();
                    break;
                default:
                    break;
            }
        },
        /*-----  End of measure  ------*/




        /*==============================================
        =          GeometryAlgorithm 几何计算          =
        ==============================================*/
        /**
         * 计算点到折线的距离
         * @param  {object} options
         * options:{
         * 		linePoints:{SEVector3s} 一个表示折线的点序列的三维矢量集合,
         * 		point:{SEGeoPoint} 一个用经纬度高程表示的坐标点
         * }
         * @return {SEVector3s}     计算结果:result.Length,单位:米
         */
        point2polyline: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.point2polyline')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.point2polyline');
                }
                return;
            }

            var v3s = options.linePoints;
            if (!v3s || v3s.Count < 2) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的折线点列,position:SeAnalysisTool.STAMP.point2polyline');
                }
                return;
            }
            var pt = options.point;
            if (!pt) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不明确的坐标点,position:SeAnalysisTool.STAMP.point2polyline');
                }
                return;
            }

            return earth.GeometryAlgorithm.CalculatePointPolylineDistance(v3s, pt);
        },

        /**
         * 计算点到直线的距离
         * @param  {options} options
         * options:{
         * 		lineStartPoint:{SEVector3s} 一个表示线段起点的三维矢量,
         * 		lineEndPoint:{SEVector3s} 一个表示线段终点的三维矢量,
         * 		point:{SEVector3s} 一个表示待测量距离的点的三维矢量
         * }
         * @return {SEVector3s}     测量结果:result.Length,单位:米
         */
        point2line: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.point2line')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.point2line');
                }
                return;
            }
            var spt = options.lineStartPoint;
            var ept = options.lineEndPoint;
            var pt = options.point;

            if (!spt || !ept || !pt) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数,position:SeAnalysisTool.STAMP.point2line');
                }
                return;
            }

            return earth.GeometryAlgorithm.DistancePointLine(spt, ept, pt);
        },

        /**
         * 计算两条折线间的距离
         * @param  {options} options
         * options:{
         * 		line1Points:{SEVector3s} 一个表示折线的点序列的三维矢量,
         * 		line2Points:{SEVector3s} 一个表示折线的点序列的三维矢量
         * }
         * @return {SEVector3s}     测量结果:result.Length,单位:米
         */
        polyline2polyline: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.polyline2polyline')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.polyline2polyline');
                }
                return;
            }

            var v3s1 = options.line1Points;
            var v3s2 = options.line2Points;
            if (!v3s1 || !v3s2) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数,position:SeAnalysisTool.STAMP.polyline2polyline');
                }
                return;
            }

            if (v3s1.Count < 2 || v3s2.Count < 2) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的折线点列,position:SeAnalysisTool.STAMP.polyline2polyline');
                }
                return;
            }

            return earth.GeometryAlgorithm.CalculateLineLineDistance(v3s1, v3s2);
        },

        /**
         * 计算两个多边形间的距离
         * @param  {options} options
         * options:{
         * 		polygon1Points:{SEVector3s} 一个表示多边形的点序列的三维矢量,
         * 		polygon2Points:{SEVector3s} 一个表示多边形的点序列的三维矢量
         * }
         * @return {SEVector3s}     测量结果:result.Length,单位:米
         */
        polygon2polygon: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.polygon2polygon')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.polygon2polygon');
                }
                return;
            }

            var v3s1 = options.polygon1Points;
            var v3s2 = options.polygon2Points;
            if (!v3s1 || !v3s2) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数,position:SeAnalysisTool.STAMP.polygon2polygon');
                }
                return;
            }

            if (v3s1.Count < 3 || v3s2.Count < 3) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的点列,position:SeAnalysisTool.STAMP.polygon2polygon');
                }
                return;
            }

            return earth.GeometryAlgorithm.CalculatePolygonDistance(v3s1, v3s2);
        },

        /**
         * 计算点到多边形的距离
         * @param  {object} options
         * options:{
         * 		polygonPoints:{SEVector3s} 一个表示多边形的点序列的三维矢量集合,
         * 		point:{SEGeoPoint} 一个用经纬度高程表示的坐标点
         * }
         * @return {SEVector3s}     计算结果:result.Length,单位:米
         */
        point2polygon: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.point2polygon')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }

            var v3s = options.polygonPoints;
            var pt = options.point;

            if (!v3s || !pt) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }
            if (v3s.Count < 3) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的点列,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }

            return earth.GeometryAlgorithm.CalculatePointPolygonDistance(v3s, pt);
        },

        /**
         * 计算折线到多边形的距离
         * @param  {options} options
         * options:{
         * 		polylinePoints:{SEVector3s} 一个表示折线的点序列的三维矢量,
         * 		polygonPoints:{SEVector3s} 一个表示多边形的点序列的三维矢量
         * }
         * @return {SEVector3s}     计算结果:result.Length,单位:米
         */
        polyline2polygon: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.polyline2polygon')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.polyline2polygon');
                }
                return;
            }

            var v3s1 = options.polylinePoints;
            var v3s2 = options.polygonPoints;

            if (!v3s1 || !v3s2) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数,position:SeAnalysisTool.STAMP.polyline2polygon');
                }
                return;
            }

            if (v3s1.Count < 2) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的折线点列,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }
            if (v3s2.Count < 3) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的多边形点列,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }

            return earth.GeometryAlgorithm.CalculateLinePolygonDistance(v3s1, v3s2);
        },

        /**
         * 计算两个建筑间的距离(按投影轮廓的最小矩形计算)
         *  @param  {options} options
         * options:{
         * 		buildingIds:{array<string>} 建筑ID的数组
         * }
         * @return {SEVector3s}     计算结果:result.Length,单位:米
         */
        building2building: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.building2building')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.building2building');
                }
                return;
            }

            var ids = options.buildingIds;
            if (!ids || !$.isArray(ids) || ids.length < 2) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的建筑ID序列,position:SeAnalysisTool.STAMP.point2polygon');
                }
                return;
            }

            var ptCol = null;
            var pts = null;
            var v3sList = [];
            for (var i = 0; i < 2; i++) {
                ptCol = earth.GeometryAlgorithm.CalculateModelContour(ids[i]);
                if (!ptCol) {
                    return;
                }
                pts = ptCol.Items(0);
                var v3s = earth.Factory.CreateVector3s();
                for (var j = 0; j < pts.Count; j++) {
                    v3s.AddVector(_geoPoint2vector3(pts.GetPointAt(j)));
                }
                v3sList.push(v3s);
            }

            return earth.GeometryAlgorithm.CalculatePolygonDistance(v3sList[0], v3sList[1]);
        },
        /*-----  End of GeometryAlgorithm  ------*/




        /*=====================================
        =            analysis 分析            =
        =====================================*/
        /**
         * 清理分析结果
         * @return {void}
         */
        clearAnalysis: function() {
            if (_analysisServerResult) {
                _analysisServerResult.ClearRes();
                _analysisServerResult = null;
            }
            if (_analysisLoaclResult) {
                _analysisLoaclResult.ClearRes();
                _analysisLoaclResult = null;
            }
        },

        /**
         * 分析地表面积
         * @param  {object} options
         * options:{
         * 		polygon:{SEVector3s} 一个表示测量范围的三维矢量集合,
         * 		[onSurfaceArea]:{function(option)} 分析完成回调
         * 		options:{
         * 			result:{SEAnalysisResult} 测量结果
         * 			area:{double} 地表面积,
         * 			unit:{string} 单位
         * 		}
         * }
         * @return {void}
         */
        surfaceArea: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.surfaceArea')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.surfaceArea');
                }
                return;
            }

            var v3s = options.polygon;
            if (!v3s || v3s.Count < 3) {
                if (globalScope.debugMode) {
                    console.warn('warn5001:不合法的测量范围');
                }
                return;
            }

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onSurfaceArea && typeof options.onSurfaceArea === 'function') {
                    options.onSurfaceArea({
                        result: result,
                        area: result.TerrainSurfaceArea,
                        unit: '平方米'
                    });
                }
            }
            earth.Analysis.SurfaceArea(v3s);
        },

        /**
         * 分析地表面积
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[onSurfaceArea]:{function(option)} 分析完成回调
         * 		options:{
         * 			result:{SEAnalysisResult} 测量结果
         * 			area:{double} 地表面积,
         * 			unit:{string} 单位
         * 		}
         * }
         * @return {void}
         */
        analysisSurfaceArea: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisSurfaceArea')) {
                return;
            }

            var me = this;
            earth.Event.OnCreateGeometry = function(polygon) {
                earth.Event.OnCreateGeometry = function() {};

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    polygon: polygon
                });
                me.surfaceArea(op);
            };
            earth.ShapeCreator.CreatePolygon();
        },

        /**
         * 通视分析
         * @param  {object} options
         * options:{
         * 		linePoints:{SEGeoPoints} 一个表示视线的坐标点集合,
         * 		[desType]:{seEnums.seAnalysisDestination} 分析目标 default:seEnums.seAnalysisDestination.both,即分析地形和模型,
         * 		[startHeight]:{float} 起点高度 default:2(m)，
         * 		[endHeight]:{float} 终点高度 default:2(m),
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onLineOfSight]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {null|SEAnalysisResult} 分析结果,同步(客户端)分析时可用,异步(服务端)分析时为null
         */
        lineOfSight: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.lineOfSight')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.lineOfSight');
                }
                return;
            }
            var pts = options.linePoints;
            if (!pts) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数linePoints,position:SeAnalysisTool.STAMP.lineOfSight');
                }
                return;
            }
            if (pts.Count != 2) {
                if (globalScope.debugMode) {
                    console.error('err5001:不合法的参数linePoints,position:SeAnalysisTool.STAMP.lineOfSight');
                }
                return;
            }

            var desType = _valueOrDefault(options, 'desType', 3);
            var sh = _valueOrDefault(options, 'startHeight', 2);
            var eh = _valueOrDefault(options, 'endHeight', 2);
            var srcType = _valueOrDefault(options, 'srcType', 3);

            if (isNaN(desType) || desType < 1 || desType > 3) {
                desType = 3;
            }
            if (isNaN(srcType) || srcType < 1 || srcType > 3) {
                srcType = 3;
            }


            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onLineOfSight && typeof options.onLineOfSight === 'function') {
                    options.onLineOfSight({
                        result: result
                    });
                }
            }
            var res = earth.Analysis.LineSight(desType, sh, eh, pts, srcType);
            _analysisLoaclResult = res;

            return res;
        },

        /**
         * 通视分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[desType]:{seEnums.seAnalysisDestination} 分析目标 default:seEnums.seAnalysisDestination.both,即分析地形和模型,
         * 		[startHeight]:{float} 起点高度 default:2(m)，
         * 		[endHeight]:{float} 终点高度 default:2(m),
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onLineOfSight]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * 		[onAnalysisLineOfSight]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {void}
         */
        analysisLineOfSight: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisLineOfSight')) {
                return;
            }

            var me = this;
            earth.Event.OnCreateGeometry = function(points, type) {
                earth.Event.OnCreateGeometry = function() {};

                //points instanceof SEGeoPoints
                //type == 3

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    linePoints: points
                });
                var res = me.lineOfSight(op);

                if (options.onAnalysisLineOfSight && typeof options.onAnalysisLineOfSight) {
                    options.onAnalysisLineOfSight({
                        result: res
                    });
                }
            };
            earth.ShapeCreator.CreateLine();
        },

        /**
         * 通视高度
         * @param  {ojbect} options
         * options:{
         * 		points:{SEGeoPoints} 需要显示高度的坐标点集
         * }
         * @return {void}
         */
        showHeightLine: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.showHeightLine')) {
                return;
            }

            if (!options || !options.points) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数points,position:SeAnalysisTool.STAMP.showHeightLine');
                }
                return;
            }

            earth.ShapeCreator.HeightLimit(options.points);
        },

        /**
         * 视域分析
         * @param  {object} options
         * options:{
         * 		sector:{SESector} 一个表示视野的扇面,
         * 		[desType]:{seEnums.seAnalysisDestination} 分析目标 default:seEnums.seAnalysisDestination.both,即分析地形和模型,
         * 		[angle]:{float} 扇面角 default:53(度)，
         * 		[height]:{float} 高度 default:2(m),
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onViewShed]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {null|SEAnalysisResult} 分析结果,同步(客户端)分析时可用,异步(服务端)分析时为null
         */
        viewShed: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.viewShed')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.viewShed');
                }
                return;
            }
            var sec = options.sector;
            if (!sec) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数sector,position:SeAnalysisTool.STAMP.viewShed');
                }
                return;
            }

            var desType = _valueOrDefault(options, 'desType', 3);
            var angle = _valueOrDefault(options, 'angle', 53);
            var h = _valueOrDefault(options, 'height', 2);
            var srcType = _valueOrDefault(options, 'srcType', 3);

            if (isNaN(desType) || desType < 1 || desType > 3) {
                desType = 3;
            }
            if (isNaN(srcType) || srcType < 1 || srcType > 3) {
                srcType = 3;
            }

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onViewShed && typeof options.onViewShed === 'function') {
                    options.onViewShed({
                        result: result
                    });
                }
            }
            var res = earth.Analysis.ViewShed(desType, angle, h, sec, srcType);
            _analysisLoaclResult = res;

            return res;
        },

        /**
         * 视域分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[desType]:{seEnums.seAnalysisDestination} 分析目标 default:seEnums.seAnalysisDestination.both,即分析地形和模型,
         * 		[angle]:{float} 扇面角 default:53(度)，
         * 		[height]:{float} 高度 default:2(m),
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onViewShed]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * 		[onAnalysisViewShed]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {void}
         */
        analysisViewShed: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisViewShed')) {
                return;
            }

            var me = this;

            var angle = _valueOrDefault(options, 'angle', 53);

            earth.Event.OnCreateGeometry = function(sector, type) {
                earth.Event.OnCreateGeometry = function() {};

                //sector instanceof SESector
                //type == 13

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    sector: sector
                });
                var res = me.viewShed(op);

                if (options.onAnalysisViewShed && typeof options.onAnalysisViewShed) {
                    options.onAnalysisViewShed({
                        result: res
                    });
                }
            };
            earth.ShapeCreator.CreateSector(angle);
        },

        /**
         * 根据日期和时间、地点计算太阳高度角和方位角
         * @param  {object} options
         * options:{
         * 		[timeZone]:{int} 时区,
         * 		[time]:{string|Date} 时间
         * }
         * @return {[object]} {
         *         elevationAngle:太阳高度角,
         *         azimuthAngle:方位角
         * }
         */
        calEleAndAzi: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.calEleAndAzi')) {
                return;
            }

            var tz = _valueOrDefault(options, 'timeZone', 8);
            var time = _valueOrDefault(options, 'time', new Date());
            time = _parseDate(time);

            var v2 = earth.GeometryAlgorithm.CalculateSunElevationAndAzimuthAngle(tz,
                time.getFullYear(), time.getMonth() + 1, time.getDate(),
                time.getHours(), time.getMinutes(), time.getSeconds());

            try {
                return {
                    elevationAngle: v2.X,
                    azimuthAngle: v2.Y
                };
            } catch (e) {
                return null;
            }
        },

        /**
         * 阴影分析
         * @param  {object} options
         * options:{
         * 		circle:{SECircle} 一个表示视野的扇面,
         * 		[elevationAngle]:{float} 太阳高度角，
         * 		[azimuthAngle]:{float} 方位角,
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onShinning]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {null|SEAnalysisResult} 分析结果,同步(客户端)分析时可用,异步(服务端)分析时为null
         */
        shinning: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.shinning')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.shinning');
                }
                return;
            }
            var circle = options.circle;
            if (!circle) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数circle,position:SeAnalysisTool.STAMP.shinning');
                }
                return;
            }

            var me = this;

            var eleAzi = me.calEleAndAzi({
                timeZone: 8,
                time: new Date()
            });
            if (!eleAzi) {
                return null;
            }

            var ele = _valueOrDefault(options, 'elevationAngle', eleAzi.elevationAngle);
            var azi = _valueOrDefault(options, 'azimuthAngle', eleAzi.azimuthAngle);

            var srcType = _valueOrDefault(options, 'srcType', 3);
            if (isNaN(srcType) || srcType < 1 || srcType > 3) {
                srcType = 3;
            }

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onShinning && typeof options.onShinning === 'function') {
                    options.onShinning({
                        result: result
                    });
                }
            }
            var res = earth.Analysis.Shinning(ele, azi, circle, srcType);
            _analysisLoaclResult = res;

            return res;
        },

        /**
         * 阴影分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[elevationAngle]:{float} 太阳高度角，
         * 		[azimuthAngle]:{float} 方位角,
         * 		[srcType]:{seEnums.seAnalysisSource} 分析方式 default:seEnums.seAnalysisSource.serviceLocal,
         * 		[onShinning]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		},
         * 		[onAnalysisShinning]:{function(option)} 分析完成回调
         * 		option:{
         * 			result:{SEAnalysisResult}
         * 		}
         * }
         * @return {null|SEAnalysisResult} 分析结果,同步(客户端)分析时可用,异步(服务端)分析时为null
         */
        analysisShinning: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisShinning')) {
                return;
            }

            var me = this;

            earth.Event.OnCreateGeometry = function(circle, type) {
                earth.Event.OnCreateGeometry = function() {};

                //circle instanceof SECircle
                //type == 8

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    circle: circle
                });
                var res = me.shinning(op);

                if (options.onAnalysisShinning && typeof options.onAnalysisShinning) {
                    options.onAnalysisShinning({
                        result: res
                    });
                }
            };
            earth.ShapeCreator.CreateCircle();
        },

        /**
         * 天际线分析
         * @param  {object} options
         * options:{
         * 		startPoint:{SEGeoPoint} 一个指定起点的坐标点,
         * 		endPoint:{SEGeoPoint} 一个指定终点的坐标点,
         * 		[height]:{double} 高度 default:0(m),
         * 		[distance]:{double} 距离 default:600(m),
         * 		[showTerrain]:{bool} 是否分析地形 default:true,
         * 		[onSkylineFinished]:{function()} 天际线分析结束回调
         * }
         * @return {void}
         */
        skyline: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.skyline')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.skyline');
                }
                return;
            }

            var sp = options.startPoint;
            var ep = options.endPoint;
            if (!sp || !ep) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数circle,position:SeAnalysisTool.STAMP.skyline');
                }
                return;
            }

            var h = _valueOrDefault(options, 'height', 0);
            var d = _valueOrDefault(options, 'distance', 600);
            var terr = _valueOrDefault(options, 'showTerrain', true);

            earth.Event.OnFixedPointObserveFinished = function() {
                earth.Event.OnFixedPointObserveFinished = function() {};

                if (options.onSkylineFinished && typeof options.onSkylineFinished === 'function') {
                    options.onSkylineFinished();
                }
            }
            earth.GlobeObserver.FixedPointObserveEx(sp, ep, h, d, terr);
        },

        /**
         * 天际线分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[height]:{double} 高度 default:0(m),
         * 		[distance]:{double} 距离 default:600(m),
         * 		[showTerrain]:{bool} 是否分析地形 default:true,
         * 		[onSkylineFinished]:{function()} 天际线分析结束回调
         * }
         * @return {void}
         */
        analysisSkyline: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisSkyline')) {
                return;
            }

            var me = this;
            earth.Event.OnCreateGeometry = function(points, type) {
                earth.Event.OnCreateGeometry = function() {};

                //points instanceof SEGeoPoints
                //type == 3

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    startPoint: points.GetPointAt(0),
                    endPoint: points.GetPointAt(1)
                });
                return me.skyline(op);
            };
            earth.ShapeCreator.CreateLine();
        },

        /**
         * 定点观察
         * @param  {object} options
         * options:{
         * 		lon:{double} 观察点经度(度),
         * 		lat:{double} 观察点纬度(度),
         * 		alti:{double} 观察点高程(米),
         * 		tilt:{double} 观察点俯仰角(度),
         * 		heading:{double} 观察点方位角(度)
         * 		roll:{double} 观察点旋转角(度),
         * 		[onFixedObserver]:{function()} 定点观察结束回调
         * }
         * @return {void}
         */
        fixedObserver: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.fixedObserver')) {
                return;
            }

            if (!options) {
                return;
            }
            if (options.lon == undefined || options.lat == undefined || options.alti == undefined) {
                return;
            }
            if (options.tilt == undefined || options.heading == undefined || options.roll == undefined) {
                return;
            }

            earth.Event.OnFixedPointObserveFinished = function() {
                earth.Event.OnFixedPointObserveFinished = function() {};

                if (options.onFixedObserver && typeof options.onFixedObserver === 'function') {
                    options.onFixedObserver();
                }
            };
            earth.GlobeObserver.FixedPointObserve(options.lon, options.lat, options.alti,
                options.tilt, options.heading, options.roll);
        },

        /**
         * 定点观察
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[height]:{double} 参考高度 default:0(m),
         * 		[onFixedObserver]:{function()} 定点观察结束回调
         * }
         * @return {void}
         */
        analysisFixedObserver: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisFixedObserver')) {
                return;
            }

            var me = this;

            var h = _valueOrDefault(options, 'height', 0);
            earth.Event.OnCreateGeometry = function(pose, type) {
                earth.Event.OnCreateGeometry = function() {};

                //pose instanceof SEObserverPose
                //type

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    lon: pose.Longitude,
                    lat: pose.Latitude,
                    alti: pose.Altitude,
                    tilt: pose.tilt,
                    heading: pose.heading,
                    roll: pose.roll
                });
                me.fixedObserver(op);

            }
            earth.ShapeCreator.CreatePose(h);
        },

        /**
         * 退出定点观察和天际线分析
         * @return {void}
         */
        stopFixedPointObserve: function() {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.stopFixedPointObserve')) {
                return;
            }

            earth.GlobeObserver.StopFixedPointObserve();
        },

        /**
         * 挖填方分析
         * @param  {object} options
         * options:{
         * 		vector3s:{SEVector3s}:指定一个多边形的三维矢量集合,
         * 		[deep]:{double} 开挖深度 default:5,
         * 		[onSurfaceExcaAndFill]:{function(option)} 挖填方分析成功回调
         * }
         * @return {void}
         */
        surfaceExcaAndFill: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.surfaceExcaAndFill')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.surfaceExcaAndFill');
                }
                return;
            }

            var v3s = options.vector3s;
            if (!v3s) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数vector3s,position:SeAnalysisTool.STAMP.surfaceExcaAndFill');
                }
                return;
            }
            if (v3s.Count < 3) {
                if (globalScope.debugMode) {
                    console.error('err5001:不合法的多边形点列,position:SeAnalysisTool.STAMP.surfaceExcaAndFill');
                }
                return;
            }

            var deep = _valueOrDefault(options, 'deep', 5);
            // var alt = earth.Measure.MeasureTerrainAltitude(v3s.Items(0).X, v3s.Items(0).Y, false);
            // params changed after updating client at 2014-10-29
            var alt = earth.Measure.MeasureTerrainAltitude(v3s.Items(0).X, v3s.Items(0).Y);
            alt = alt.toFixed(3) - deep;

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onSurfaceExcaAndFill && typeof options.onSurfaceExcaAndFill === 'function') {
                    options.onSurfaceExcaAndFill({
                        result: result,
                        excavation: result.Excavation,
                        fill: result.Fill,
                        unit: '立方米'
                    });
                }

            };
            earth.Analysis.SurfaceExcavationAndFill(alt, v3s);
        },

        /**
         * 挖填方分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[deep]:{double} 开挖深度 default:5,
         * 		[onSurfaceExcaAndFill]:{function(option)} 挖填方分析成功回调
         * }
         * @return {void}
         */
        analysisSurfaceExcaAndFill: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.surfaceExcaAndFill')) {
                return;
            }

            var me = this;

            earth.Event.OnCreateGeometry = function(vector3s, type) {
                earth.Event.OnCreateGeometry = function() {};

                //vector3s instanceof SEVector3s
                //type

                if (vector3s.Count < 3) {
                    return;
                }

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    vector3s: vector3s
                });
                me.surfaceExcaAndFill(op);
            };
            earth.ShapeCreator.CreatePolygon();
        },

        /**
         * 水淹分析
         * @param  {object} options
         * options:{
         * 		circle:{SECircle} 圆域,
         * 		alti:{double} 高程,
         * 		[onSubmerge]:{function(options)} 分析完成回调
         * }
         * @return {void}
         */
        submerge: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.submerge')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.submerge');
                }
                return;
            }
            if (!options.circle || options.alti == undefined || isNaN(options.alti)) {
                return;
            }

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onSubmerge && typeof options.onSubmerge === 'function') {
                    options.onSubmerge({
                        result: result
                    });
                }
            };
            earth.Analysis.Submerge(options.alti, options.circle);
        },

        /**
         * 流域分析
         * @param  {object} options
         * options:{
         * 		vector3s:{SEVector3s} 指定一条折线的三维矢量集合
         * 		[radius]:{doublc} 缓冲半径 default:200(米),
         * 		[onValley]:{function(options)} 分析完成回调,
         * 		option:{
         * 			result:result
         * 		}
         * }
         * @return {void}
         */
        valley: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.valley')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.valley');
                }
                return;
            }

            if (!options.vector3s || options.alti == undefined || isNaN(options.alti)) {
                return;
            }

            var r = _valueOrDefault(options, 'radius', 200);

            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onValley && typeof options.onValley === 'function') {
                    options.onValley({
                        result: result
                    });
                }
            };
            earth.Analysis.Valley(options.alti, r, options.vector3s);
        },

        /**
         * 流域分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[radius]:{doublc} 缓冲半径 default:200(米),
         * 		[onValley]:{function(options)} 分析完成回调,
         * 		option:{
         * 			result:result
         * 		}
         * }
         * @return {void}
         */
        analysisValley: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisValley')) {
                return;
            }

            var me = this;

            earth.Event.OnCreateGeometry = function(vector3s, type) {
                earth.Event.OnCreateGeometry = function() {};

                //vector3s instanceof SEVector3s
                //type

                if (vector3s.Count < 2) {
                    return;
                }

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var op = {};
                $.extend(op, options, {
                    vector3s: vector3s
                });
                me.valley(op);
            };
            earth.ShapeCreator.CreatePolyline();
        },

        /**
         * 地形路径
         * @param  {object} options
         * options:{
         * 		points:{SEGeoPoints},
         * 		[upAngle]:{double} 上坡度 default:20,
         * 		[downAngle]:{double} 下坡度 default:20,
         * 		[onBestPath]:{function(options)}
         * 		option:{
         * 			result:result
         * 		}
         * }
         * @return {void}
         */
        bestPath: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.bestPath')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.bestPath');
                }
                return;
            }
            var pts = options.points;
            if (!pts) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数points,position:SeAnalysisTool.STAMP.bestPath');
                }
                return;
            }

            var ua = _valueOrDefault(options, 'upAngle', 20);
            var da = _valueOrDefault(options, 'downAngle', 20);
            if (isNaN(ua)) {
                ua = 20;
            }
            if (isNaN(da)) {
                da = 20;
            }


            earth.Event.OnAnalysisFinished = function(result) {
                earth.Event.OnAnalysisFinished = function() {};

                _analysisServerResult = result;

                if (options.onBestPath && typeof options.onBestPath === 'function') {
                    options.onBestPath({
                        result: result
                    });
                }
            };
            earth.Analysis.BestPath(ua, da, pts);
        },

        /**
         * 陪面分析
         * @param  {object} options
         * options:{
         * 		points:{SEGeoPoints} 指定一条线路的坐标点集合
         * 		[step]:{double} 采样间距 default:2(米),
         * 		[type]:{seEnums.seProfileType} default:seEnums.seProfileType.defaultAlt
         * }
         * @return {string} xml格式的字符串
         */
        profile: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.bestPath')) {
                return;
            }

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:SeAnalysisTool.STAMP.bestPath');
                }
                return;
            }
            var pts = options.points;
            if (!pts) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数points,position:SeAnalysisTool.STAMP.bestPath');
                }
                return;
            }
            if (pts.Count < 2) {
                return;
            }

            var t = _valueOrDefault(options, 'type', 1);
            var step = _valueOrDefault(options, 'step', 2);

            var result = seearth.Analysis.Profile(t, step, pts);

            return result;
        },

        /**
         * 陪面分析
         * @param  {object} options
         * options:{
         * 		[clearShape]:{bool} 分析完成后是否清除绘制的shape default:true,
         * 		[step]:{double} 采样间距 default:2(米),
         * 		[type]:{seEnums.seProfileType} default:seEnums.seProfileType.defaultAlt,
         * 		[onAnalysisProfile]:{function(options)}
         * 		options:{
         * 			result:{string}
         * 		}
         * }
         * @return {string} xml格式的字符串
         */
        analysisProfile: function(options) {
            if (!_checkEarth(true, 'SeAnalysisTool.STAMP.analysisProfile')) {
                return;
            }

            var me = this;

            earth.Event.OnCreateGeometry = function(vector3s, type) {
                earth.Event.OnCreateGeometry = function() {};

                if (vector3s.Count < 2) {
                    return;
                }

                if (_valueOrDefault(options, 'clearShape', true)) {
                    earth.ShapeCreator.Clear();
                }

                var pts = earth.Factory.CreateGeoPoints();;
                for (var i = 0; i < vector3s.Count; i++) {
                    pts.AddPoint(_vector32point(vector3s.Items(i)));
                }

                if (options.onAnalysisProfile && typeof options.onAnalysisProfile === 'function') {
                    var op = {};
                    $.extend(op, options, {
                        points: pts
                    });
                    var result = me.profile(op);

                    options.onAnalysisProfile({
                        result: result
                    });
                }

            };
            earth.ShapeCreator.CreatePolyline();
        },
        /*-----  End of analysis  ------*/





        /*==========  the version  ==========*/
        version: VERSION
    });



    /*=============================================
    =            expose SeAnalysisTool            =
    =============================================*/
    if (globalScope == undefined) {
        globalScope = window;
    }
    if (globalScope.STAMP == undefined) {
        globalScope.STAMP = {};
    }

    if ((typeof module) !== 'undefined' && module.exports) {
        //check for CommonJS
        module.exports = SeAnalysisTool;
    } else if ((typeof define) === 'function' && define.amd) {
        //check for AMD
        define([], function() {
            return SeAnalysisTool;
        });
    } else {
        globalScope.STAMP.SeAnalysisTool = SeAnalysisTool;
    }
    /*-----  End of expose SeAnalysisTool  ------*/


})(window);
