/**
 * filename:animationManager.js
 * desc:animation
 * deps:jQuery,util,enums
 * authors:zc
 * time:2014-09-03 09:04:10
 * version:0.0.1
 * updatelog:
 * 0.0.1:the first version
 **/

(function(ns) {
    var SeAnimationManager = null,
        VERSION = '0.0.1',

        ANIMATION_PATH = 'visit\\',
        ANIMATION_LIST_NAME = 'visit',
        ANIMATION_NAME = 'visit',
        ROOT_VISIT_ID = 1,
        ROOT_VISIT_NAME = 'visit',
        DEFAULT_FOLDER_NAME = 'RecordFolder',
        SEPARATOR = '',

        visitPath = 'visit\\',
        roorVisitId = 1,
        rootVisitName = 'visit',

        earth = null,
        globalScope = ns;


    /**
     * 检查earth是否可用
     * @param {bool} earth是否必须,若为false,则空球时仍判定成功并打印一个警告(debugMode == true时)
     * @param {string} [position] 位置,一般是函数名
     * @return {bool} earth是否可用
     */
    var _checkEarth = function(necessary, position) {
        if (position == undefined) {
            position = ''
        }

        if (earth == null) {
            if (necessary) {
                if (globalScope.debugMode) {
                    console.error('err3005:earth initialize fail,position:' + position);
                }
                return false;
            } else {
                if (globalScope.debugMode) {
                    console.warn('warn3006:earth initialize fail,position:' + position);
                }
                return true;
            }
        } else {
            return true;
        }
    }

    var _valueOrDefault = function(obj, property, defaultValue) {
        try {
            if (!obj || obj[property] == undefined || obj[property] == null) {
                return defaultValue;
            }

            return obj[property];
        } catch (e) {
            return defaultValue;
        }
    }



    SeAnimationManager = function(options) {
        if (!(this instanceof SeAnimationManager)) {
            return new SeAnimationManager(options);
        }

        this.initialize.apply(this, arguments);
    }

    $.extend(SeAnimationManager.prototype, {
        /**
         * 初始化
         * @param  {object} options
         * options:{
         * 		earth:{object} 三维球,
         * 		[visitPath]:{string} 动画保存路径
         * }
         * @return {void}
         */
        initialize: function(options) {
            if (options.earth == null) {
                if (globalScope.debugMode) {
                    console.warn('warn3001:null earth,position:STAMP.SeAnimationManager.initialize');
                }
            }

            earth = options.earth;
            visitPath = options.visitPath || ANIMATION_PATH;

        },

        /**
         * 从客户端获取xml字符串格式的动画列表
         * @param  {object} [options]
         * [options]:{
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {string} xml字符串格式的动画列表
         */
        getAniListXmlFromClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.getAniListXmlFromClient')) {
                return;
            }

            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;

            try {
                var aniListXml = earth.UserDocument.LoadXmlFile(url + '.xml');
                if (aniListXml == "") {
                    aniListXml = "<Xml><Visit  NAME='" + rootVisitName + "' ID='" + roorVisitId + "'></Visit></Xml>";
                    earth.UserDocument.saveXmlFile(url, aniListXml);
                    aniListXml = earth.UserDocument.LoadXmlFile(url + '.xml');
                }
                return aniListXml;
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.getAniListXmlFromClient');
                }

                return null;
            }
        },

        /**
         * 保存xml字符串格式的动画列表到客户端
         * @param  {object} [options]
         * [options]:{
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'，
         * 		[aniListXml]:{string} xml字符串格式的动画列表 default:''
         * }
         * @return {string} xml字符串格式的动画列表
         */
        saveAniListXmlToClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.saveAniListXmlToClient')) {
                return;
            }

            try {
                var path = _valueOrDefault(options, 'path', visitPath);
                var url = earth.RootPath + path + ANIMATION_LIST_NAME;

                var aniListXml = '';
                if (!options || options.aniListXml == undefined || options.aniListXml == '') {
                    aniListXml = '';
                } else {
                    aniListXml = options.aniListXml;
                }

                earth.UserDocument.saveXmlFile(url, aniListXml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.saveAniListXmlToClient');
                }
            }
        },

        /**
         * 从客户端获取动画列表
         * @param  {object} [options]
         * [options]:{
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {object} json格式的动画列表对象
         */
        getAniListFromClient: function(options) {
            var me=this;
            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                if (aniListXml) {
                    return globalScope.STAMP.seUtil.xml2json(aniListXml);
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.getAniListFromClient');
                }

                return null;
            }
        },

        /**
         * 创建动画folder节点到客户端
         * @param  {object} options
         * [options]:{
         * 		[folderId]:{string} 节点ID,
         * 		[folderName]:{string} 节点名称 default:'RecordFolder',
         * 		[open]:{bool} default:true,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {void}
         */
        createFolderNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.createFolderNode')) {
                return;
            }

            var me=this;
            var folderId=_valueOrDefault(options,'folderId',earth.Factory.CreateGUID());
            var folderName = _valueOrDefault(options, 'folderName', DEFAULT_FOLDER_NAME);
            var isOpen = _valueOrDefault(options, 'open', 'true');
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;


            try {

                var xmlData = "<RecordFolder ID='" +
                    folderId + "' NAME='" + folderName + "' TYPE='folder' OPEN='" + isOpen + "'   >" +
                    "</RecordFolder>";
                var xmlDoc = globalScope.STAMP.seUtil.loadXMLStr("<xml>" + xmlData + "</xml>");

                var aniListXml = me.getAniListXmlFromClient(options);
                var rootDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);
                var rootVisitEl = rootDoc.documentElement.firstChild;

                var isExist = false;
                var nodesTemp = rootVisitEl.childNodes;
                for (var i = 0; i < nodesTemp.length; i++) {
                    if (nodesTemp[i].getAttribute('ID') == folderId) {
                        isExist = true;
                        break;
                    }
                }

                if (!isExist) {
                    rootVisitEl.appendChild(xmlDoc.documentElement.firstChild);
                }

                var op = {};
                $.extend(op, options, {
                    aniListXml: rootDoc.xml
                });
                me.saveAniListXmlToClient(op);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.createFolderNode');
                }
            }
        },

        /**
         * 从客户端删除一个folder节点,包括其子节点
         * @param  {object} options
         * options:{
         * 		folderId:{string} 节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[deleteData]:{bool} 是否同时删除相应的数据 default:true
         * }
         * @return {void}
         */
        deleteFolderNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.deleteFolderNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.deleteFolderNode');
                }
                return;
            }

            var folderId = options.folderId;
            if (!folderId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数folderId,position:STAMP.SeAnimationManager.deleteFolderNode');
                }
                return;
            }

            var me=this;
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;
            var deleteData = _valueOrDefault(options, 'deleteData', true);
            var recordId = '';


            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);

                var folderNodes = aniListXmlDoc.getElementsByTagName("RecordFolder");
                

                for (var j = 0; j < folderNodes.length; j++) {
                    if (folderNodes[j].getAttribute("ID") == options.folderId) {
                        folderNodes[j].parentNode.removeChild(folderNodes[j]);

                        if(deleteData){
                        	var recordNodes = folderNodes[j].childNodes;

			                for (var i = 0; i < recordNodes.length; i++) {
			                    recordId = recordNodes[i].getAttribute('ID');
			                    me.deleteRecordData({
			                        recordId: recordId,
			                        path: path
			                    });
			                }
                        }
                    }
                }

                earth.UserDocument.saveXmlFile(url, aniListXmlDoc.xml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.deleteFolderNode');
                }
            }
        },

        /**
         * 修改folder节点
         * @param  {object} options
         * options:{
         * 		folderId:{string} 节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[name]:{string} 节点名称,为''或undefined时不做修改 default:'',
         * 		[open]:{bool} undefined时不做修改 default:undefined
         * }
         * @return {void}
         */
        updateFolderNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.updateFolderNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.updateFolderNode');
                }
                return;
            }

            var folderId = options.folderId;
            if (!folderId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数folderId,position:STAMP.SeAnimationManager.updateFolderNode');
                }
                return;
            }

            var me=this;
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;

            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);

                var folderNodes = aniListXmlDoc.getElementsByTagName("RecordFolder");
                for (var i = 0; i < folderNodes.length; i++) {
                    if (folderNodes[i].getAttribute("ID") == folderId) {
                        if (options.name != undefined && options.name != '') {
                            folderNodes[i].setAttribute('NAME', options.name);
                        }
                        if (options.open != undefined) {
                            folderNodes[i].setAttribute('OPEN', options.open ? 'true' : 'false');
                        }
                    }
                }

                earth.UserDocument.saveXmlFile(url, aniListXmlDoc.xml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.updateFolderNode');
                }
            }
        },

        /**
         * 创建record节点
         * @param  {[type]} options
         * options:{
         * 		[recordId]:{string} record节点ID,
         * 		folderId:{string} 父节点(folder)ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[name]:{string} 节点名称 default:'record',
         * 		[fps]:{int} 帧率 default:25
         * }
         * @return {void}
         */
        createRecordNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.createRecordNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.createRecordNode');
                }
                return;
            }

            var folderId = options.folderId;
            if (!folderId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数folderId,position:STAMP.SeAnimationManager.createRecordNode');
                }
                return;
            }

            var me=this;
            var recordId=_valueOrDefault(options,'recordId',earth.Factory.CreateGUID());
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;

            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);
                var folderNodes = aniListXmlDoc.getElementsByTagName("RecordFolder");

                for (var i = 0; i < folderNodes.length; i++) {
                    if (folderNodes[i].getAttribute("ID") == folderId) {
                        var recordName = _valueOrDefault(options, 'name', 'record');
                        var fps = _valueOrDefault(options, 'fps', 25);

                        var recordXml = "<Record ID='" + recordId + "' NAME='" + recordName + "' FPS='" + fps + "'></Record>";
                        var recordXmlDoc = globalScope.STAMP.seUtil.loadXMLStr("<xml>" + recordXml + "</xml>");

                        folderNodes[i].appendChild(recordXmlDoc.documentElement.firstChild);

                        break;
                    }
                }

                earth.UserDocument.saveXmlFile(url, aniListXmlDoc.xml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.createRecordNode');
                }
            }
        },

        /**
         * 删除record节点
         * @param  {ojbect} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[deleteData]:{bool} 是否同时删除相应的数据 default:true
         * }
         * @return {void}
         */
        deleteRecordNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.deleteRecordNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.deleteRecordNode');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.deleteRecordNode');
                }
                return;
            }

            var me=this;
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;
            var deleteData = _valueOrDefault(options, 'deleteData', true);

            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);
                var recordNodes = aniListXmlDoc.getElementsByTagName("Record");

                for (var i = 0; i < recordNodes.length; i++) {
                    if (recordNodes[i].getAttribute('ID') == recordId) {
                        recordNodes[i].parentNode.removeChild(recordNodes[i]);
                    }

                    if (deleteData) {
                        me.deleteRecordData({
                            recordId: recordId,
                            path: path
                        });
                    }
                }

                earth.UserDocument.saveXmlFile(url, aniListXmlDoc.xml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.deleteRecordNode');
                }
            }
        },

        /**
         * 修改record节点
         * @param  {object} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[name]:{string} 节点名称 default:undefined
         * }
         * @return {void}
         */
        updateRecordNode: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.updateRecordNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.updateRecordNode');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.updateRecordNode');
                }
                return;
            }

            var me=this;
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;

            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);
                var recordNodes = aniListXmlDoc.getElementsByTagName("Record");

                for (var i = 0; i < recordNodes.length; i++) {
                    if (recordNodes[i].getAttribute('ID') == recordId) {
                        if (options.name != undefined && options.name != '') {
                            recordNodes[i].setAttribute('NAME', options.name);
                        }
                    }
                }

                earth.UserDocument.saveXmlFile(url, aniListXmlDoc.xml);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.updateRecordNode');
                }
            }
        },

        /**
         * 根据ID获取动画节点信息
         * @param  {object} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {object} 动画节点信息,json格式的对象
         */
        getRecordNode:function(options){
        	if (!_checkEarth(true, 'STAMP.SeAnimationManager.getRecordNode')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.getRecordNode');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.getRecordNode');
                }
                return;
            }

            var me=this;
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_LIST_NAME;

            try {
                var aniListXml = me.getAniListXmlFromClient(options);
                var aniListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(aniListXml);
                var recordNodes = aniListXmlDoc.getElementsByTagName("Record");

                for (var i = 0; i < recordNodes.length; i++) {
                    if (recordNodes[i].getAttribute('ID') == recordId) {
                        var obj={
                        	id:recordId,
                        	name:recordNodes[i].getAttribute('NAME'),
                        	fps:recordNodes[i].getAttribute('FPS')
                        };

                        return obj;
                    }
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeAnimationManager.getRecordNode');
                }
            }

            return null;
        },

        /**
         * 保存录制数据
         * @param  {object} options
         * options:{
         * 		[recordId]:{string} record节点ID,
         * 		points:{string} 录制的数据,xml格式的字符串,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {void}
         */
        createRecordData: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.createRecordData')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.createRecordData');
                }
                return;
            }
            var points = options.points;
            if (!points) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数points,position:STAMP.SeAnimationManager.createRecordData');
                }
                return;
            }

            var recordId=_valueOrDefault(options,'recordId',earth.Factory.CreateGUID());
            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_NAME + SEPARATOR + recordId;

            earth.UserDocument.SaveXmlFile(url, points);
        },

        /**
         * 删除录制的数据
         * @param  {object} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {void}
         */
        deleteRecordData: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.deleteRecordData')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.deleteRecordData');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.deleteRecordData');
                }
                return;
            }

            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_NAME + SEPARATOR + recordId;

            earth.UserDocument.DeleteXmlFile(url + '.xml');
        },

        /**
         * 根据ID获取录制的数据
         * @param  {object} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\'
         * }
         * @return {string} 录制的数据,xml格式的字符串
         */
        getRecordData:function(options){
        	if (!_checkEarth(true, 'STAMP.SeAnimationManager.getRecordData')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.getRecordData');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.getRecordData');
                }
                return;
            }

            var path = _valueOrDefault(options, 'path', visitPath);
            var url = earth.RootPath + path + ANIMATION_NAME + SEPARATOR + recordId;

            return earth.UserDocument.LoadXmlFile(url + '.xml');
        },

        /**
         * 开始动画回放
         * @param  {object} options
         * options:{
         * 		recordId:{string} record节点ID,
         * 		[path]:{string} 动画列表保存路径 default:'visit\\',
         * 		[onPlayOneFrame]:{function(options) 回放每帧的回调,
         * 		options:{
         * 			recordId:{string} record节点ID
         * 			count:{int},
         * 			frame:{int}
         * 		}
         * 		[onPlayOver]:{function(options) 回放结束的回调
         * 		options:{
         * 			recordId:{string} record节点ID
         * 			count:{int},
         * 			frame:{int}
         * 		}
         * }
         * @return {void}
         */
        startPlayBack:function(options){
        	if (!_checkEarth(true, 'STAMP.SeAnimationManager.startPlayBack')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeAnimationManager.startPlayBack');
                }
                return;
            }
            var recordId = options.recordId;
            if (!recordId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的参数recordId,position:STAMP.SeAnimationManager.startPlayBack');
                }
                return;
            }

            var me=this;

            var recObj=me.getRecordNode(options);
            if(!recObj){
            	if(globalScope.debugMode){
            		console.warn('warn3005:未获取到动画信息');
            	}
            	return;
            }
            var points=me.getRecordData(options);
            if(!points){
            	if(globalScope.debugMode){
            		console.warn('warn3005:未获取到动画数据');
            	}
            	return;
            }

            earth.Event.OnPosePlaybackOneFrame = function (ncount,nframe){
                //earth.Event.OnPosePlaybackOneFrame = function(){};

                if(options.onPlayOneFrame &&typeof options.onPlayOneFrame ==='function'){
                	options.onPlayOneFrame({
                		recordId:recordId,
                		count:ncount,
                		frame:nframe
                	});
                }

                if(ncount == nframe){
                    if(options.onPlayOver &&typeof options.onPlayOver ==='function'){
	                	options.onPlayOver({
	                		recordId:recordId,
	                		count:ncount,
	                		frame:nframe
	                	});
	                }
                }
            };
            earth.PoseRecorder.StartPlayback(recObj.fps,points);
        },

        /**
         * 结束动画回放
         * @return {void}
         */
        stopPlayBack:function(){
        	if (!_checkEarth(true, 'STAMP.SeAnimationManager.stopPlayBack')) {
                return;
            }

            earth.Event.OnPosePlaybackOneFrame = function(){};
        	earth.PoseRecorder.StopPlayback();
        },

        /**
         * 开始录制
         * @param  {object} options
         * [options]:{
         * 		[fps]:{int} 帧率 default:25
         * }
         * @return {void}
         */
        startRecord: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.startRecord')) {
                return;
            }

            var fps = _valueOrDefault(options, 'fps', 25);
            if (isNaN(fps)) {
                if (globalScope.debugMode) {
                    console.error('err5001:不合法的帧率,position:STAMP.SeAnimationManager.startRecord');
                }
                return;
            }

            earth.PoseRecorder.StartRecord(fps);
        },

        /**
         * 暂停/继续录制
         * @param  {object} options
         * [options]:{
         * 		[isPause]:{bool} 是否暂停,true:暂停;false:继续 default:true
         * }
         * @return {void}
         */
        pauseRecord: function(options) {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.pauseRecord')) {
                return;
            }

            var isPause = _valueOrDefault(options, 'isPause', true);
            earth.PoseRecorder.Pause(isPause);
        },

        /**
         * 结束录制
         * @param  {void}
         * @return {string} 录制的数据
         */
        stopRecord: function() {
            if (!_checkEarth(true, 'STAMP.SeAnimationManager.stopRecord')) {
                return;
            }

            return earth.PoseRecorder.StopRecord();
        },



        version: VERSION
    });




    /*=================================================
	=            expose SeAnimationManager            =
	=================================================*/
    if (globalScope == undefined) {
        globalScope = window;
    }
    if (globalScope.STAMP == undefined) {
        globalScope.STAMP = {};
    }

    if ((typeof module) !== 'undefined' && module.exports) {
        //check for CommonJS
        module.exports = SeAnimationManager;
    } else if ((typeof define) === 'function' && define.amd) {
        //check for AMD
        define([], function() {
            return SeAnimationManager;
        });
    } else {
        globalScope.STAMP.SeAnimationManager = SeAnimationManager;
    }
    /*-----  End of expose SeAnimationManager  ------*/



})(window);
