//! fileName : drawTools.js
//! desc : 在三维球中的绘制
//! deps : jquery.js util.js
//! authors : YanZhipeng
//! time : 2014-09-03 17:21
//! version : 0.0.1
//! updateLog : 
(function(ns) {
    /************************************
        Constants
    ************************************/
    var SeDrawTool = {},
        VERSION = '0.0.1',
        earth,
        globalScope = typeof ns != 'undefined' ? ns : window,
        debugMode = typeof ns.debugMode != 'undefined' ? ns.debugMode : false,
        b = 1;

    SeDrawTool = function(options) {
        this.initialize.apply(this, arguments);
    };

    /************************************
        Internal function
    ************************************/
    /**
     * 检查当前球是否已经定义
     * @param {string} [position] 出现问题的函数路径
     * @param  {bool} [forcedNeedEarth] 是否强制需要使用球，默认为true
     * @return {bool}                 当前球是否被定义
     */
    function _checkEarth(position, forcedNeedEarth) {
        if (typeof position == 'undefined' || position == null) {
            position = '';
        }
        if (typeof forcedNeedEarth == 'undefined' || forcedNeedEarth == null || typeof forcedNeedEarth != 'boolean') {
            forcedNeedEarth = true;
        }
        var earthIsDefined = false;
        if (typeof earth != 'undefined') {
            earthIsDefined = true;
        } else {
            if (globalScope.debugMode) {
                if (forcedNeedEarth) {
                    console.error('err3005, 位置：' + position + '.  三维球没有初始化，三维球被强制需要。请确保执行了"initialize"函数');
                } else {
                    console.warn('warn3006, 位置：' + position + '.  三维球没有正常初始化, 建议执行"initialize"函数对其进行初始化');
                }
            }
        }
        return earthIsDefined;
    };

    /**
     * 将回调函数绑定到三维球的OnCreateGeometry时间上
     * @param  {function} callbackFun 回调函数
     */
    function _bindCallBackFunction(callbackFun) {
        earth.Event.OnCreateGeometry = callbackFun;
    };

    $.extend(SeDrawTool.prototype, {
        initialize: function(options) {
            if (typeof options == 'undefined' || options == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeDrawTool.initialize. 初始化参数强制需要，请参阅API文档');
                }
                return;
            }
            if (typeof options.earth == 'undefined' || options.earth == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeDrawTool.initialize. 参数中"earth"属性不能为空');
                    return;
                }
            }
            earth = options.earth;
        },
        /**
         * 将创建的图形清楚掉
         */
        clearShpeCreator: function() {
            earth.ShapeCreator.Clear();
        },
        /**
         * 生成辅助线
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createLine: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateLine();
        },
        /**
         * 生成辅助折线
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param  {double} lineWidth   线宽
         * @param  {uint} lineColor   线的颜色
         */
        createPolyline: function(callbackFun, lineWidth, lineColor) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePolyline(lineWidth, lineColor);
        },
        /**
         * 指定起始点生成辅助折线
         * @param  {function} callbackFun 回调函数
         * @param  {double} lineWidth   线宽
         * @param  {uint} lineColor   线的颜色
         */
        createPolylineWithFirstPoint: function(callbackFun, lineWidth, lineColor) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePolylineWithFirstPoint(lineWidth, lineColor);
        },
        /**
         * 创建多边形
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param  {uint} fillColor   填充色
         * @param  {uint} lineColor   轮廓线颜色
         */
        createPolygon: function(callbackFun, fillColor, lineColor) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePolygon(fillColor, lineColor);
        },
        /**
         * 生成辅助圆
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param  {uint} fillColor   填充色
         * @param  {uint} lineColor   轮廓线颜色
         */
        createCircle: function(callbackFun, fillColor, lineColor) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateCircle(fillColor, lineColor);
        },
        /**
         * 生成辅助矩形
         */
        createRectangle: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateRectangle();
        },
        /**
         * 生成辅助扇形
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param  {double} angle       扇形的扇面角
         */
        createSector: function(callbackFun, angle) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateSector(angle);
        },
        /**
         * 生成辅助椭圆
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createEllipse: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateEllipse();
        },
        /**
         * 生成辅助曲线
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createCurve: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateCurve();
        },
        /**
         * 生成辅助简单箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotSArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotSArrow();
        },
        /**
         * 生成辅助多箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotXArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotXArrow();
        },
        /**
         * 生成辅助直箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotEqualSArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotEqualSArrow();
        },
        /**
         * 生成辅助燕尾箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotTailSArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotTailSArrow();
        },
        /**
         * 生成辅助双箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotDoubleArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotDoubleArrow();
        },
        /**
         * 生成辅助自定义燕尾箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotCustomTailArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotCustomTailArrow();
        },
        /**
         * 生成辅助自定义箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotCustomArrow: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotCustomArrow();
        },
        /**
         * 生成辅助辅助三角旗标
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotTriangleFlag: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotTriangleFlag();
        },
        /**
         * 生成辅助辅助直角旗标
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotRectFlag: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotRectFlag();
        },
        /**
         * 生成辅助辅助曲线旗标
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotCurveFlag: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotCurveFlag();
        },
        /**
         * 生成辅助集结区域
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotAssemblyArea: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotAssemblyArea();
        },
        /**
         * 生成辅助球
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {uint} color 颜色
         */
        createSphere: function(callbackFun, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateSphere(color);
        },
        /**
         * 生成辅助box
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {uint} color 颜色
         */
        createBox: function(callbackFun, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateBox(color);
        },
        /**
         * 生成辅助圆柱体
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {uint} color 颜色
         */
        createCylinder: function(callbackFun, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateCylinder(color);
        },
        /**
         * 生成辅助圆锥体
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {uint} color 颜色
         */
        createCone: function(callbackFun, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateCone(color);
        },
        /**
         * 生成辅助正棱柱
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {int} sides 连数
         * @param {uint} color 颜色
         */
        createPrism: function(callbackFun, sides, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePrism(sieds, color);
        },
        /**
         * 生成辅助正棱锥
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {int} sides 连数
         * @param {uint} color 颜色
         */
        createPyramid: function(callbackFun, sides, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePyramid(sieds, color);
        },
        /**
         * 生成辅助立体多边形
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {uint} color 颜色
         */
        createVolume: function(callbackFun, color) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreateVolume(color);
        },
        /**
         * 生成辅助球面姿态
         * @param  {function} callbackFun 绘制完成后的回调函数
         * @param {double} refHeight 参考高度
         */
        createPose: function(callbackFun, refHeight) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePose(refHeight);
        },
        /**
         * 生成辅助球面姿态
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPoseEx: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePoseEx();
        },
        /**
         * 生成辅助点
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPoint: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePoint();
        },
        /**
         * 生成辅助三维直箭头
         * @param  {function} callbackFun 绘制完成后的回调函数
         */
        createPlotEqualSArrow3D: function(callbackFun) {
            _bindCallBackFunction(callbackFun);
            earth.ShapeCreator.CreatePlotEqualSArrow3D();
        }
    });

    if (typeof globalScope == 'undefined') {
        globalScope = window;
    }
    if (typeof globalScope.STAMP == 'undefined') {
        globalScope.STAMP = {};
    }

    //check for CommonJS
    if (typeof module !== 'undefined' && module.exports) {
        module.exports = SeDrawTool;
    } else if (typeof define === 'function' && define.amd) {
        define([], function() {
            return SeDrawTool;
        });
    } else {
        globalScope.STAMP.SeDrawTool = SeDrawTool;
    }
}).call(this, window);
