//! layerManager.js
//! version : 0.0.1
//! authors : YanZhipeng
//! includeIn : stampgisLib
//! updateLogs : ZhangDang,添加了管线图层数据获取(pipeLineLayers),标注模块中用到
(function(ns) {
    /************************************
        Constants
    ************************************/
    var SeLayerManager = {},
        VERSION = '0.0.1',
        IMAGE_PATH = 'images/zTree/diy/',
        earth,
        globalScope = typeof ns != 'undefined' ? ns : window,
        //图层树中变量的名称，结合前端树控件所要求的结构类型进行自定义
        treeNodeAttributes = {
            'id': 'id',
            'name': 'name',
            'children': 'children',
            'checked': 'checked'
        },
        //是否对返回的树的数据结构进行重新定义
        redefineTreeNodeAttrName = false,
        layerTreeData = [],

        pipeLineLayers = [],
        pipeModelLayers = [],
        importPipeLines = [],
        projectList = [],
        guihuaArr = [],

        //if the debugMode is true, the browser's console will throw some information
        debugMode = typeof ns.debugMode != 'undefined' ? ns.debugMode : false;

    /************************************
        Internal function
    ************************************/
    /**
     * 检查当前球是否已经定义
     * @param string position 出现问题的函数路径
     * @param  bool forcedNeedEarth 是否强制需要使用球，默认为true
     * @return bool                 当前球是否被定义
     */
    function _checkEarth(position, forcedNeedEarth) {
        if (typeof position == 'undefined' || position == null) {
            position = '';
        }
        if (typeof forcedNeedEarth == 'undefined' || forcedNeedEarth == null || typeof forcedNeedEarth != 'boolean') {
            forcedNeedEarth = true;
        }
        var earthIsDefined = false;
        if (typeof earth != 'undefined') {
            earthIsDefined = true;
        } else {
            if (globalScope.debugMode) {
                if (forcedNeedEarth) {
                    console.error('err3005, 位置：' + position + '.  三维球没有初始化，三维球被强制需要。请确保执行了"initialize"函数');
                } else {
                    console.warn('warn3006, 位置：' + position + '.  三维球没有正常初始化, 建议执行"initialize"函数对其进行初始化');
                }
            }
        }
        return earthIsDefined;
    };

    /**
     * 将图层的英文名称改为中文名称
     * @param  string name 英文名称
     * @return string      转义后的中文名称
     */
    function _enName2cnName(name) {
        var map = {
            "equipment": "附属设施",
            "container": "管线",

            "well": "井",
            "joint": "附属点",
            "plate": "井盖",
            "room": "井室",

            "container_og": "地上管线",
            "joint": "特征",
            "joint_og": "地上特征"
        };
        if (map[name]) {
            name = map[name];
        }
        return name;
    };

    /**
     * 递归创建图层树结构
     * @param  object layer 当前从earth中取到的图层
     * @param  Boolean en2cn 是否对图层名称进行转义
     * @return object       构造的图层树
     */
    function _createLayerTreeData(layer, en2cn) {
        if (!layer) {
            layer = earth.LayerManager.LayerList;
        }
        var layerData = [];
        var childCount = layer.GetChildCount();
        for (var i = 0; i < childCount; i++) {
            var childLayer = layer.GetChildAt(i);
            var name = en2cn ? _enName2cnName(childLayer.Name) : childLayer.Name;

            var data = {};
            //如果对树结构属性进行了重新定义，则使用定义后的名称
            if (redefineTreeNodeAttrName) {
                data[treeNodeAttributes['id']] = childLayer.Guid;
                data[treeNodeAttributes['name']] = name;
                data[treeNodeAttributes['checked']] = childLayer.Visibility;
                if (childLayer.GetChildCount() > 0) {
                    data[treeNodeAttributes['children']] = _createLayerTreeData(childLayer, en2cn);
                }
            } else {
                data['id'] = childLayer.Guid;
                data['name'] = name;
                data['checked'] = childLayer.Visibility;
                if (childLayer.GetChildCount() > 0) {
                    data.children = _createLayerTreeData(childLayer, en2cn);
                }
            }

            var layerType = childLayer.LayerType;
            data['icon'] = _getNodeIcon(layerType);
            if (name != "buffer" && name != "room") {
                layerData.push(data);
            }
        }
        return layerData;
    };

    function _getNodeIcon(layerType) {
        var layerType = layerType.toLowerCase();
        if (layerType == 'project' || layerType == 'folder') {
            return '';
        } else if (layerType == 'model') {
            return IMAGE_PATH + 'model.png';
        }
        //console.info(layerType);
    };

    SeLayerManager = function(options) {
        this.initialize.apply(this, arguments);
    };

    $.extend(SeLayerManager.prototype, {
        initialize: function(options) {
            if (typeof options == 'undefined' || options == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 初始化参数强制需要，请参阅API文档');
                }
                return;
            }
            if (typeof options.earth == 'undefined' || options.earth == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 参数中"earth"属性不能为空');
                    return;
                }
            }
            earth = options.earth;
        },

        /**
         * 对图层树结构中的属性名称进行重定义，方便不同的树形控件使用
         * @param  {object} options {
         *                          id: 图层的guid,
         *                          name: 图层的名称,
         *                          checked: 当前图层是否可见,
         *                          children: 存储子节点的数组
         * }    该参数可以不必包含上面所有的属性，需要改什么属性的名称，就定义什么属性。
         * @return {array}        没有根节点，数组中直接是第一层树节点，节点中包含该节点的一些属性信息和子节点
         */
        defineTreeNodeAttrName: function(options) {
            if (typeof options == 'undefined' || options == null) {
                return;
            }
            redefineTreeNodeAttrName = true;
            for (var attr in options) {
                if (treeNodeAttributes.hasOwnProperty(attr)) {
                    treeNodeAttributes[attr] = options[attr];
                }
            }
        },

        /**
         * getLayerTreeData 获得球的数据图层树结构
         * @param  {string} parentLayerName 限制返回来的图层的根节点位置，为空则返回默认根节点
         * @param {bool} en2cn 是否将图层名称从英文转义为中文，默认为true
         * @return {object}                 树状结构
         */
        getLayerTreeData: function(parentLayerName, en2cn) {
            if (!_checkEarth('STAMP.SeLayerManager.getLayerTreeData', true)) {
                return null;
            }
            if (typeof en2cn == 'undefined' || typeof en2cn != 'boolean') {
                en2cn = true;
            }
            var layerTreeData = {};
            if (typeof parentLayerName == 'undefined' || parentLayerName == null || parentLayerName == "") {
                layerTreeData = earth.LayerManager.LayerList;
            }
            if (typeof layerTreeData == 'undefined' || layerTreeData == null) {
                return null;
            }
            var newLayerTreeData = _createLayerTreeData(layerTreeData, en2cn);
            try {
                var rootLayer = this.getRootLayer();
                var childCount = rootLayer.GetChildCount();
                for (var i = 0; i < childCount; i++) {
                    var childLayer = rootLayer.GetChildAt(i);
                    if (globalScope.STAMP.sysInfo.project === childLayer.Guid) {
                        var children = this.getPipelineLayerData(childLayer);
                    }
                }
            } catch (e) {

            }

            return newLayerTreeData;
        },

        /**
         * 显示某图层
         * @param  {string} guid 图层的guid
         */
        showLayer: function(guid) {
            if (guid) {
                if (_checkEarth("STAMP.SeLayerManager.showLayer", true)) {
                    var layer = earth.LayerManager.GetLayerByGUID(guid);
                    if (layer) {
                        layer.Visibility = true;
                    }
                }
            } else {
                if (globalScope.debugMode) {
                    console.error("err3005, 位置：STAMP.LayerManager.showLayer 参数guid不能为空");
                };
            }
        },

        /**
         * 隐藏某图层
         * @param  {string} guid 图层的guid
         */
        hideLayer: function(guid) {
            if (guid) {
                if (_checkEarth("STAMP.SeLayerManager.showLayer", true)) {
                    var layer = earth.LayerManager.GetLayerByGUID(guid);
                    if (layer) {
                        layer.Visibility = false;
                    }
                }
            } else {
                if (globalScope.debugMode) {
                    console.error("err3005, 位置：STAMP.LayerManager.hideLayer 参数guid不能为空");
                };
            }
        },

        zoomToLayerByGuid: function(guid) {
            if (guid) {
                if (_checkEarth('STAMP.SeLayerManager.zoomToLayerByGuid', false)) {
                    var layer = earth.LayerManager.GetLayerByGUID(guid);
                    this.zoomToLayer(layer);
                } else {
                    return;
                }
            }
        },

        zoomToLayer: function(layer) {
            if (_checkEarth('STAMP.SeLayerManager.zoomToLayerByGuid', false)) {
                var lonLatRect = layer.LonLatRect;
                var centerX = (lonLatRect.East + lonLatRect.West) / 2;
                var centerY = (lonLatRect.North + lonLatRect.South) / 2;
                var width = (parseFloat(lonLatRect.North) - parseFloat(lonLatRect.South)) / 2;
                var range = width / 180 * Math.PI * 6378137 / Math.tan(22.5 / 180 * Math.PI);
                earth.GlobeObserver.FlytoLookat(centerX, centerY, 0, 0, 90, 0, range, 4);
            }
        },

        /**
         * 显示/隐藏二维地图
         * @param  {object} options
         * [options]:{
         *     [visible]:{bool} 是否显示二维地图
         * }
         * @return {void}
         */
        show2DMap: function(options) {
            if (!_checkEarth('STAMP.SeLayerManager.show2DMap', true)) {
                return;
            }

            var me = this;

            var v = false;
            if (options && options.visible != undefined) {
                v = options.visible;
            }

            function _showLayer(layer, visible, layerType) {
                try {
                    if (layer == null || layer == undefined) {
                        return;
                    }

                    if (!$.isArray(layerType)) {
                        layerType = [layerType];
                    }

                    var lt = layer.LayerType;
                    if (lt && $.inArray(lt, layerType) > -1) {
                        layer.Visibility = visible;
                    }

                    for (var i = 0, l = layer.GetChildCount(); i < l; i++) {
                        var cl = layer.GetChildAt(i);
                        _showLayer(cl, visible, layerType);
                    }
                } catch (e) {
                    return;
                }
            }

            var rootLayer = earth.LayerManager.LayerList;
            _showLayer(rootLayer, v, ['WMS', 'GISVector']);
        },
        /**
         * 功能：获取图层根节点
         * 参数：无
         * 返回值：图层根节点
         */
        getRootLayer: function() {
            var rootLayer = earth.LayerManager.LayerList;
            return rootLayer;
        },
        /**
         * 功能：获取管线图层数据
         * 参数：layer-图层根节点
         * 返回值：图层管线数据数组
         */
        getPipelineLayerData: function(layer) {
            var layerData = [];
            var childCount = layer.GetChildCount();
            for (var i = 0; i < childCount; i++) {
                var childLayer = layer.GetChildAt(i);
                if (((Number(childLayer.Rtti) === 103 || Number(childLayer.Rtti) === 104)) && Number(layer.Rtti) != 114) { //BuildingLayer精模图层 BlockLayer白膜图层
                    if (!(childLayer.getChildAt(0) && childLayer.getChildAt(0).name === "Equipment" && childLayer.getChildAt(1).name === "container" && childLayer.getChildAt(2).name === "well")) {
                        if (typeof pipeModelLayers == 'undefined' || pipeModelLayers == null) {
                            if (globalScope.debugMode == true) {
                                console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量pipeModelLayers未初始化');
                            }
                            return;
                        }
                        pipeModelLayers.push(childLayer);
                    }
                }
                var id = childLayer.Guid;
                var name = childLayer.Name;
                if (name === "buffer") {
                    childLayer.Visibility = false;
                }
                if (name === "loukuai") {
                    modelLayer.layer = childLayer;
                    modelLayer.guid = id;
                    modelLayer.name = name;
                }
                //如果是重点管线则添加到数组对象中 2014.1.6
                if (childLayer.KeyLine) {
                    if (typeof importPipeLines == 'undefined' || importPipeLines == null) {
                        if (globalScope.debugMode == true) {
                            console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量importPipeLines未初始化');
                        }
                        return;
                    }
                    importPipeLines.push({
                        guid: childLayer.guid,
                        name: childLayer.name,
                        mScope: childLayer.MScope,
                        pScope: childLayer.PScope
                    });
                }
                var visibility = childLayer.Visibility;
                var layerType = childLayer.LayerType;
                var count = childLayer.GetChildCount();
                if (count > 0) {
                    if (layerType === "Project" || layerType === "Folder" || layerType === "Pipeline") {
                        var children = this.getPipelineLayerData(childLayer);
                        if (children.length > 0) {
                            var data = {};
                            data.id = id;
                            data.name = name;
                            data.checked = visibility;
                            // data.icon = this.getLayerIcon(layerType);
                            data.children = children;
                            layerData.push(data);
                            if (typeof pipeLineLayers == 'undefined' || pipeLineLayers == null) {
                                if (globalScope.debugMode == true) {
                                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量pipeLineLayers未初始化');
                                }
                                return;
                            }
                            pipeLineLayers.push({
                                'id': data.id,
                                'name': data.name,
                                'server': childLayer.GISServer,
                                'pltype': childLayer.PipeLineType,
                                customColor: childLayer.CustomColor
                            });
                            if (layerType === "Project") {
                                if (typeof projectList == 'undefined' || projectList == null) {
                                    if (globalScope.debugMode == true) {
                                        console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量projectList未初始化');
                                    }
                                    return;
                                }
                                projectList.push({
                                    'id': id,
                                    'name': name,
                                    'server': childLayer.GISServer,
                                    'pltype': childLayer.PipeLineType
                                });
                            }
                            if (layerType === "Folder") {
                                if (typeof guihuaArr == 'undefined' || guihuaArr == null) {
                                    if (globalScope.debugMode == true) {
                                        console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量guihuaArr未初始化');
                                    }
                                    return;
                                }
                                guihuaArr.push({
                                    'id': id,
                                    'name': name
                                });
                            }
                        }
                    }
                } else {
                    if ((layerType === "Container") || (layerType === "Container_Og") || (layerType === "Equipment") || (layerType === "Joint") || (layerType === "Joint_Og") || (layerType === "Well") || (layerType === "Plate") || (layerType === "Buffer") || (layerType.toUpperCase() === "ROOM")) {
                        if (layerType === "Equipment") {
                            name = "附属设施";
                        } else if (layerType === "Container") {
                            name = "管线";
                        } else if (layerType === "Well") {
                            name = "井";
                        } else if (layerType === "Joint") {
                            name = "特征";
                            if (typeof pipeLineLayers == 'undefined' || pipeLineLayers == null) {
                                if (globalScope.debugMode == true) {
                                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 私有变量pipeLineLayers未初始化');
                                }
                                return;
                            }
                            pipeLineLayers.push({
                                'id': id,
                                'name': name,
                                'server': childLayer.GISServer,
                                'pltype': childLayer.PipeLineType,
                                customColor: childLayer.CustomColor
                            });
                        } else if (layerType === "Plate") {
                            name = "井盖";
                        } else if (name === "buffer") { //这里为啥图层的name为buffer呢
                            childLayer.visibility = false;
                        } else if (layerType.toUpperCase() == "ROOM") {
                            name = "井室";
                        } else if (layerType === "Container_Og") {
                            name = "地上管线";
                        } else if (layerType === "Joint_Og") {
                            name = "地上特征";
                        }
                        var data = {};
                        data.id = id;
                        data.name = name;
                        data.checked = visibility;
                        // data.icon = this.getLayerIcon(layerType);
                        layerData.push(data);
                    }
                }
            }
            return layerData;
        },
        getPipelineData: function() {
            var pdata = {
                pipeLineLayers: pipeLineLayers,
                pipeModelLayers: pipeModelLayers,
                importPipeLines: importPipeLines,
                projectList: projectList,
                guihuaArr: guihuaArr
            };
            return pdata;
        },

        version: VERSION
    });


    if (typeof globalScope == 'undefined') {
        globalScope = window;
    }
    if (typeof globalScope.STAMP == 'undefined') {
        globalScope.STAMP = {};
    }
    //check for CommonJS
    if (typeof module !== 'undefined' && module.exports) {
        module.exports = SeLayerManager;
    } else if (typeof define === 'function' && define.amd) {
        define([], function() {
            return SeLayerManager;
        });
    } else {
        globalScope.STAMP.SeLayerManager = SeLayerManager;
    }

}).call(this, window);
