//! fileName : queryTools.js
//! desc : 三维球中的查询
//! deps : jquery.js util.js
//! authors : YanZhipeng
//! time : 2014-09-04 11:46
//! version : 0.0.1
//! updateLog : 
(function(ns) {
    /************************************
        Constants
    ************************************/
    var SeQueryTool = {},
        VERSION = '0.0.1',
        earth,
        globalScope = typeof ns != 'undefined' ? ns : window,
        debugMode = typeof ns.debugMode != 'undefined' ? ns.debugMode : false,
        b = 1;

    SeQueryTool = function(options) {
        this.initialize.apply(this, arguments);
    };

    /************************************
        Internal function
    ************************************/
    /**
     * 检查当前球是否已经定义
     * @param {string} [position] 出现问题的函数路径
     * @param  {bool} [forcedNeedEarth] 是否强制需要使用球，默认为true
     * @return {bool}                 当前球是否被定义
     */
    function _checkEarth(position, forcedNeedEarth) {
        if (typeof position == 'undefined' || position == null) {
            position = '';
        }
        if (typeof forcedNeedEarth == 'undefined' || forcedNeedEarth == null || typeof forcedNeedEarth != 'boolean') {
            forcedNeedEarth = true;
        }
        var earthIsDefined = false;
        if (typeof earth != 'undefined') {
            earthIsDefined = true;
        } else {
            if (globalScope.debugMode) {
                if (forcedNeedEarth) {
                    console.error('err3005, 位置：' + position + '.  三维球没有初始化，三维球被强制需要。请确保执行了"initialize"函数');
                } else {
                    console.warn('warn3006, 位置：' + position + '.  三维球没有正常初始化, 建议执行"initialize"函数对其进行初始化');
                }
            }
        }
        return earthIsDefined;
    };

    /**
     * 加载系统配置信息
     */
    function _loadSysInfo() {
        if (globalScope.STAMP.sysInfo == undefined) {
            globalScope.STAMP.sysInfo = {};
        }
        var rootPath = earth.Environment.RootPath + "temp\\SystemConfig";
        var configPath = rootPath + ".xml";
        var configXmlStr = earth.UserDocument.LoadXmlFile(configPath);

        //var systemDoc = globalScope.STAMP.seUtil.loadXMLStr(configXml);
        var systemJson = globalScope.STAMP.seUtil.xml2json(configXmlStr);
        if (typeof systemJson != 'undefined' && systemJson != null) {
            globalScope.STAMP.sysInfo.project = systemJson.Project;
            globalScope.STAMP.sysInfo.position = systemJson.Position;
        } else {
            alert('加载系统配置信息失败');
        }
    };

    /**
     * 将获取的字段映射文件xml内容转换为object对象
     * @param  {xml} xmldoc 字段映射xml
     */
    function _createFieldMapObj(xmldoc) {
        var fieldMapObj = {};
        fieldMapObj['lineFieldInfo'] = {};
        fieldMapObj['pointFieldInfo'] = {};
        if (xmldoc) {
            var lineFieldInfo = {};
            var pointFieldInfo = {};
            var configXML = xmldoc;
            var lineData = configXML.getElementsByTagName("LineFieldInfo")[0] ? configXML.getElementsByTagName("LineFieldInfo")[0].getElementsByTagName("SystemFieldList")[0] : null;
            var pointData = configXML.getElementsByTagName("PointFieldInfo")[0] ? configXML.getElementsByTagName("PointFieldInfo")[0].getElementsByTagName("SystemFieldList")[0] : null;
            if (lineData) {
                for (var i = 0; i < lineData.childNodes.length; i++) {
                    var node = lineData.childNodes[i];
                    if (!("getAttribute" in node)) {
                        continue;
                    }
                    if (node.getAttribute("FieldName") == "") {
                        continue;
                    }
                    lineFieldInfo[node.getAttribute("FieldName").toUpperCase()] = {
                        CaptionName: node.getAttribute("CaptionName"),
                        StandardName: node.getAttribute("StandardName")
                    };
                }
            }
            if (pointData) {
                for (var i = 0; i < pointData.childNodes.length; i++) {
                    var node = pointData.childNodes[i];
                    if (!("getAttribute" in node)) {
                        continue;
                    }
                    if (node.getAttribute("FieldName") == "") {
                        continue;
                    }
                    pointFieldInfo[node.getAttribute("FieldName").toUpperCase()] = {
                        CaptionName: node.getAttribute("CaptionName"),
                        StandardName: node.getAttribute("StandardName")
                    };
                }
            }
            fieldMapObj = {
                lineFieldInfo: lineFieldInfo,
                pointFieldInfo: pointFieldInfo
            };
        }
        globalScope.STAMP.sysInfo.pipeFieldMap = fieldMapObj;
    };

    function _loadFieldMap() {
        //说明在之前已经进行了初始化，不用再进行第二次初始化了
        if (typeof globalScope.STAMP.sysInfo == 'undefined' || globalScope.STAMP.sysInfo == null) {
            _loadSysInfo();
        }
        if (typeof globalScope.STAMP.fieldMap == 'undefined' || globalScope.STAMP.fieldMap == null) {
            var layer = earth.LayerManager.GetLayerByGUID(globalScope.STAMP.sysInfo.project);
            if (layer) {
                var projectSetting = layer.ProjectSetting;
                var fieldMap = projectSetting.FieldMapFile; //字段映射配置文件
                if (fieldMap != "") {
                    var filedPath = "http://" + fieldMap.substr(2).replace("/", "/sde?/") + "_sde";
                    //no need to modify the url after updating StampManager2 at 2014-10-08
                    filedPath = projectSetting.FieldMapFile;
                    earth.Event.OnEditDatabaseFinished = function(pRes, pFeature) {
                        if (pRes.ExcuteType == 27) {
                            var fieldMapXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(pRes.AttributeName);
                            _createFieldMapObj(fieldMapXmlDoc);
                        }
                    }
                    earth.DatabaseManager.GetXml(filedPath);
                }
            }
        }
    };


    /**
     * 根据标准名称返回显示字段名称
     * @param  {string}     standardName     标准字段名称
     * @param  {seEnums.sePipeType}         pipeType         管线类型 1 -- 管线 0 -- 管点
     * @param  {string}     returnFiledName  true返回FiledName false返回CaptionName
     * @return {string}                      显示名称
     */
    function _getName(standardName, pipeType, returnFiledName) {
        if (standardName === "" || standardName === undefined) {
            return;
        }
        if (pipeType === "" || pipeType === undefined) {
            return;
        }
        var fieldData;
        if (pipeType === globalScope.STAMP.seEnums.sePipeType.line) {
            fieldData = globalScope.STAMP.sysInfo.pipeFieldMap.lineFieldInfo;
        } else if (pipeType === globalScope.STAMP.seEnums.sePipeType.point) {
            fieldData = globalScope.STAMP.sysInfo.pipeFieldMap.pointFieldInfo;
        }
        for (var i in fieldData) {
            var item = fieldData[i];
            if (item['StandardName'].toUpperCase() == standardName.toUpperCase()) {
                if (returnFiledName) {
                    return i;
                } else {
                    return item['CaptionName'];
                }
            }
        }
        return '';
    };

    /**
     * 处理一条查询结果，根据字段映射文件，处理键值对中键的显示名称
     * @param  {object} record   查询结果的键值对
     * @param  {sePipetype} pipeType 是点还是线
     * @return {object}          处理后的键值对
     */
    function _sloveResult(record, pipeType) {
        var slovedRecord = {};
        if (pipeType == globalScope.STAMP.seEnums.sePipeType.line) {
            var fieldMap = globalScope.STAMP.sysInfo.pipeFieldMap.lineFieldInfo;
        } else if (pipeType == globalScope.STAMP.seEnums.sePipeType.point) {
            var fieldMap = globalScope.STAMP.sysInfo.pipeFieldMap.pointFieldInfo;
        }
        for (var key in record) {
            if (key == "") continue;
            var value = record[key];
            if (value == null || value == undefined || value == "") continue;
            var mapItem = fieldMap[key.toUpperCase()];
            var showField = null;
            if (mapItem)
                showField = mapItem['CaptionName'];
            else
                continue;

            //确保是数字且本身有小数点需要进行处理的
            if (!isNaN(value) && value.indexOf('.') > -1) {
                value = parseFloat(value).toFixed(2);
            }
            slovedRecord[showField] = value;
        }
        return slovedRecord;
    };

    $.extend(SeQueryTool.prototype, {
        initialize: function(options) {
            if (typeof options == 'undefined' || options == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeQueryTool.initialize. 初始化参数强制需要，请参阅API文档');
                }
                return;
            }
            if (typeof options.earth == 'undefined' || options.earth == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeQueryTool.initialize. 参数中"earth"属性不能为空');
                    return;
                }
            }
            earth = options.earth;
            _loadFieldMap();
        },
        getName: function(options) {
            var standardName = options.standardName;
            var pipeType = options.pipeType;
            if (isNaN(pipeType) || pipeType < 0 || pipeType > 1) {
                pipeType = 1;
            }
            var returnFieldName = options.returnFieldName;

            return _getName(standardName, pipeType, returnFieldName);
        },
        /**
         * 属性查询
         * @param  {function} callBackFun 查询成功后的回调函数，返回结果
         * @param {int} [pickType] 拾取目标类型 default:383
         */
        propertyQuery: function(callBackFun, pickType) {
            earth.Event.OnPickObjectEx = function(resultObj) {
                resultObj.Underground = true;
                resultObj.ShowHighLight();
                var parentLayerName = resultObj.GetParentLayerName();
                var errMsg = '';
                if (parentLayerName == '' || parentLayerName == null) {
                    errMsg = '获得父层名称失败！';
                    callBackFun(null, null, errMsg);
                }
                var key = resultObj.GetKey();
                var layerInfoArr = parentLayerName.split("=");
                var guidInfoArr = layerInfoArr[1].split("_");
                var guid = guidInfoArr[0];
                var layer = earth.LayerManager.GetLayerByGUID(guid);

                var keyFieldInRealData = "US_KEY";
                var queryResult = {};
                //判断是一般模型还是管线数据
                if (guidInfoArr.length > 1) {
                    //判断是不是为管线
                    var bLine = parentLayerName.indexOf("container") > -1;
                    var type = bLine ? globalScope.STAMP.seEnums.sePipeType.line : globalScope.STAMP.seEnums.sePipeType.point;
                    if (layer) {
                        keyFieldInRealData = _getName("US_KEY", type, true);
                    }
                    var strParam = "(and,equal," + keyFieldInRealData + "," + key + ")";
                    var queryParam = layer.QueryParameter;
                    queryParam.Filter = strParam;
                    queryParam.QueryType = 16; // SE_AttributeData
                    queryParam.QueryTableType = (bLine ? 1 : 0);
                    queryParam.PageRecordCount = 1;
                    var result = layer.SearchFromGISServer();
                    //点击查询只会有一条结果
                    var resutlXmlStr = result.GotoPage(0);
                    var resultJson = globalScope.STAMP.seUtil.xml2json(resutlXmlStr);
                    if (resultJson) {
                        var records = {};
                        var resultNum = resultJson.Result.num;
                        if (resultNum == 0) {
                            records = {};
                        } else if (resultNum == 1) {
                            records = resultJson.Result.Record;
                        } else {
                            records = resultJson.Result.Record[0];
                        }
                        queryResult = _sloveResult(records, type);
                        queryResult['图层'] = layer.Name;
                    }
                } else {
                    queryResult = {
                        '名称': key,
                        '图层': layer.Name
                    };
                }
                var rect = resultObj.GetLonLatRect();
                var north = rect.North;
                var south = rect.South;
                var east = rect.East;
                var west = rect.West;
                var top = rect.MaxHeight;
                var bottom = rect.MinHeight;
                var location = {
                    x: (east + west) / 2,
                    y: (north + south) / 2,
                    z: (top + bottom) / 2
                };
                //执行回调函数，将查询结果传递回去
                if (typeof callBackFun != 'undefined' && typeof callBackFun == 'function') {
                    callBackFun(queryResult, location);
                }
                //showpropertyQuery(earth, parentLayerName, key, resultObj);
            };
            //原来是127，改成383可以选取树进行查询
            if (pickType == undefined || isNaN(pickType)) {
                pickType = 383;
            }
            earth.Query.PickObjectEx(pickType);
        },
        /**
         * 条件查询
         * @param  {object} queryParam  查询条件
         * @param  {function} callBackFun 查询结束后的回调函数
         */
        conditionQuery: function(queryParam, callBackFun) {
                var conditionFilter = queryParam.conditionFilter; //条件过滤条件
                var geometry = queryParam.geometry; //空间过滤条件
                var queryType = queryParam.queryType; //seEnums.seQueryType查询方式
                var paging = queryParam.paging; //分页方式
                var geometryType = queryParam.geometryType; //查询的是空间几何类型
                var layerId = queryParam.layerId; //参与查询的图层的guid
                var keyWord = queryParam.keyWord; //关键字查询
                if (typeof layerId == 'undefined' || layerId == null) {
                    if (debugMode) {
                        console.error("err 3005 查询参数中缺少layerid属性");
                        return;
                    }
                    return;
                }

                var layer = earth.LayerManager.GetLayerByGUID(layerId);
                if (layer) {
                    var searchParam = layer.LocalSearchParameter;
                    //searchParam.SetFilter(keyword, keyWord);
                    if (geometry) {
                        searchParam.SetFilter("", "");
                        searchParam.SetSpatialFilter(geometry);
                    }
                    if (keyWord != "") {
                        searchParam.SetFilter(keyWord, "");
                    }
                    searchParam.PageRecordCount = 10;
                    searchParam.HasDetail = true;
                    searchParam.HasMesh = true;
                    searchParam.ReturnDataType = 1; //0 返回所有数据，1 返回xml数据，2 返回渲染数据
                    var result = layer.SearchFromLocal();
                    var resultXml = result.GotoPage(0);
                    callBackFun(resultXml);
                }

            }
            //暂时不能使用
            /*conditionQuery: function(queryParam, callBackFun) {
                var conditionFilter = queryParam.conditionFilter; //条件过滤条件
                var geometry = queryParam.geometry; //空间过滤条件
                var queryType = queryParam.queryType; //seEnums.seQueryType查询方式
                var paging = queryParam.paging; //分页方式
                var geometryType = queryParam.geometryType; //查询的是空间几何类型
                var layerId = queryParam.layerId; //参与查询的图层的guid
                var ip = "192.168.1.228"; //暂时的，后面加配置
                if (typeof layerId == 'undefined' || layerId == null) {
                    if (debugMode) {
                        console.error("err 3005 查询参数中缺少layerid属性");
                        return;
                    }
                    return;
                }
                var geometryStr = "";
                if (typeof geometry != 'undefined' && geometry != null) {
                    if (geometryType == globalScope.STAMP.seEnums.seGeometryType.polygon) {
                        var pointString = "";
                        for (var i = 0; i < geometry.Count; i++) {
                            if (pointString === "") {
                                pointString = pointString + geometry.Items(i).X + "," + geometry.Items(i).Y + "," + 0;
                            } else {
                                pointString = pointString + "," + geometry.Items(i).X + "," + geometry.Items(i).Y + "," + 0;
                            }
                        }
                        geometryStr = "&sc(2" + "," + geometry.Count + "," + pointString + ")";
                    } else if (geometryType == globalScope.STAMP.seEnums.seGeometryType.circle) {
                        geometryStr = geometry.Radius + geometry.Longitude + "," + geometry.Latitude;
                        geometryStr = '&sc(3,0,' + geometryStr + ')';
                    }
                }
                if (typeof queryType == 'undefined' || queryType == null) {
                    //设置默认查询方式
                    queryType = globalScope.STAMP.seEnums.seQueryType.attributeData;
                    if (debugMode) {
                        console.warn('warn3006 查询下属中没有设定queryType属性');
                    }
                }
                var conditionFilterStr = "";
                if (typeof conditionFilter != 'undefined' && conditionFilter != null || conditionFilter != '') {
                    conditionFilterStr = "&pc=" + conditionFilter;
                }
                var queryUrl = "http://" + ip + "/geoserver?service=" + layerId + geometryStr + "&qt=" + queryType + conditionFilterStr;
                console.info(queryUrl);
                earth.Event.OnEditDatabaseFinished = function(response) {
                    callBackFun(response);
                }

                //使用三维球去掉服务
                earth.DatabaseManager.GetXml(queryUrl);
            }*/

    });

    if (typeof globalScope == 'undefined') {
        globalScope = window;
    }
    if (typeof globalScope.STAMP == 'undefined') {
        globalScope.STAMP = {};
    }

    //check for CommonJS
    if (typeof module !== 'undefined' && module.exports) {
        module.exports = SeQueryTool;
    } else if (typeof define === 'function' && define.amd) {
        define([], function() {
            return SeQueryTool;
        });
    } else {
        globalScope.STAMP.SeQueryTool = SeQueryTool;
    }
}).call(this, window)
