//! fileName : sysInfoHelper.js
//! desc : 
//! deps : jquery.js util.js
//! authors : YanZhipeng,zc
//! time : 2014-09-04 11:46
//! version : 0.0.1
//! updateLog : 
(function(ns) {
    /************************************
        Constants
    ************************************/
    var SeSysInfoHelper = undefined,

        VERSION = '0.0.1',
        SYS_CONF_PATH = 'temp\\SystemConfig',

        earth = null,
        globalScope = typeof ns != 'undefined' ? ns : window,
        debugMode = typeof ns.debugMode != 'undefined' ? ns.debugMode : false,
        sysInfo = null,
        /**
        * explaination about data
        * data is used to keep custom data instead of using global variable derictly
        * never set it to null or undefined
        * please use the related function:
        * getData(),
        * setData(),
        * clearData(),
        * _getData()
        * take care of that _getData() is not suggested to use,
        * for it get a reference of data as a returnValue
        **/
        data={};



    /************************************
        Internal function
    ************************************/
    /**
     * 检查当前球是否已经定义
     * @param {string} [position] 出现问题的函数路径
     * @param  {bool} [forcedNeedEarth] 是否强制需要使用球，默认为true
     * @return {bool}                 当前球是否被定义
     */
    function _checkEarth(position, forcedNeedEarth) {
        if (typeof position == 'undefined' || position == null) {
            position = '';
        }
        if (typeof forcedNeedEarth == 'undefined' || forcedNeedEarth == null || typeof forcedNeedEarth != 'boolean') {
            forcedNeedEarth = true;
        }
        var earthIsDefined = false;
        if (typeof earth != 'undefined') {
            earthIsDefined = true;
        } else {
            if (globalScope.debugMode) {
                if (forcedNeedEarth) {
                    console.error('err3005, 位置：' + position + '.  三维球没有初始化，三维球被强制需要。请确保执行了"initialize"函数');
                } else {
                    console.warn('warn3006, 位置：' + position + '.  三维球没有正常初始化, 建议执行"initialize"函数对其进行初始化');
                }
            }
        }
        return earthIsDefined;
    };

    var _valueOrDefault = function(obj, property, defaultValue) {
        try {
            if (!obj || obj[property] == undefined || obj[property] == null) {
                return defaultValue;
            }

            return obj[property];
        } catch (e) {
            return defaultValue;
        }
    }

    function _loadFieldMap() {
        //说明在之前已经进行了初始化，不用再进行第二次初始化了
        if (typeof globalScope.STAMP.sysInfo != 'undefined' && globalScope.STAMP.sysInfo != null) {
            return;
        }

        

    };

    /**
     * 设置系统配置项
     * @param {object} options
     * options:{
     *     key:{string} config item,
     *     value:{string} value of item
     * }
     * @return {string} old value of item
     */
    function _saveSysConfItem(options) {
        if (!_checkEarth('_saveSysConfItem', true)) {
            return;
        }

        if (!options) {
            return;
        }

        var ik = options.key;
        var iv = options.value;
        if (ik == undefined || iv == undefined) {
            return;
        }

        try {
            var sysXmlDoc = _loadSysConf();
            if (sysXmlDoc == null) {
                return undefined;
            }

            var root = sysXmlDoc.documentElement;
            var node = root.getElementsByTagName(ik)[0];
            var oiv = node.text;
            node.text = iv;

            var sysConfPath = earth.Environment.RootPath + SYS_CONF_PATH;
            earth.UserDocument.SaveXmlFile(sysConfPath, sysXmlDoc.xml);

            return iv;
        } catch (e) {
            return undefined;
        }
    };

    /**
     * 初始化系统设置
     * @param  {[object]} options
     * [options]:{
     *     [projectId]:{string} 当前项目ID
     * }
     * @return {string} 系统设置(字符串)
     */
    function _initSysConf(options) {
        if (!_checkEarth('_initSysConf', true)) {
            return;
        }

        var sysXml = '<xml>';

        //当前项目
        var projectId = '';
        if (options && options.projectId) {
            projectId = options.projectId;
        }
        sysXml += '<Project>' + projectId + '</Project>';

        //摄像相Pose
        sysXml += '<Position></Position>';

        sysXml += '</xml>';

        var sysConfPath = earth.Environment.RootPath + SYS_CONF_PATH;
        earth.UserDocument.SaveXmlFile(sysConfPath, sysXml);

        return sysXml;
    };

    /**
     * 加载系统设置
     * @return {XMLDocument} 系统设置(XML文档)
     */
    function _loadSysConf() {
        if (!_checkEarth('_loadSysConf', true)) {
            return;
        }

        try {
            var sysConfPath = earth.Environment.RootPath + SYS_CONF_PATH;
            var sysXml = earth.UserDocument.LoadXmlFile(sysConfPath + '.xml');
            if (sysXml == '') {
                sysXml = _initSysConf();
            }
            var sysXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(sysXml);

            return sysXmlDoc;
        } catch (e) {
            return null;
        }
    };
    /*-----  End of Internal function  ------*/




    SeSysInfoHelper = function(options) {
        if (!(this instanceof SeSysInfoHelper)) {
            return new SeSysInfoHelper(options);
        }

        this.initialize.apply(this, arguments);
    };

    $.extend(SeSysInfoHelper.prototype, {
        initialize: function(options) {
            if (typeof options == 'undefined' || options == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeQueryTool.initialize. 初始化参数强制需要，请参阅API文档');
                }
                return;
            }
            if (typeof options.earth == 'undefined' || options.earth == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeQueryTool.initialize. 参数中"earth"属性不能为空');
                    return;
                }
            }
            earth = options.earth;
        },

        /**
         * 加载系统配置
         * @param  {object} options
         * options:{
         *     cache:{bool} 是否允许从已缓存中读取配置 default:true
         * }
         * @return {object} 系统配置信息 
         * return:{
         *     Project:{string} 当前项目ID,
         *     Position:{string} 摄像机Pose
         * }
         */
        loadSysConfig: function(options) {
            var cache = _valueOrDefault(options, 'cache', true);
            if (sysInfo == null || !cache) {
                var sysXmlDoc = _loadSysConf();
                if (sysXmlDoc) {
                    sysInfo = globalScope.STAMP.seUtil.xml2json(sysXmlDoc);
                }
            }

            return sysInfo;
        },

        /**
         * 保存当前项目ID
         * @param  {object} options
         * options:{
         *     projectId:{string} 项目ID
         * }
         * @return {string} 上次保存的项目ID
         */
        saveProjectId: function(options) {
            var pid = _valueOrDefault(options, 'projectId', undefined);
            if (pid == undefined) {
                return;
            }

            return _saveSysConfItem({
                key: 'Project',
                value: pid
            });
        },

        /**
         * 保存摄像机Pose
         * options:{
         *     pose:{string} 摄像机Pose
         * }
         * @return {string} 上次保存的摄像机Pose
         */
        saveObserverPose: function(options) {
            var pose = _valueOrDefault(options, 'pose', '');

            return _saveSysConfItem({
                key: 'Position',
                value: pose
            });
        },

        //获取data的一个副本,用于读取
        getData:function(){
            var dataCopy={};
            $.extend(dataCopy,data);
            return dataCopy;
        },
        //向data中添加属性
        setData:function(options){
            if(options==null||options==undefined){
                return;
            }

            $.extend(data,options);
        },
        //清空data中的属性
        clearData:function(){
            data={};
        },

        _getData:function(){
            return data;
        },

        version: VERSION
    });


    /*==============================================
    =            expose SeSysInfoHelper            =
    ==============================================*/
    if (typeof globalScope == 'undefined') {
        globalScope = window;
    }
    if (typeof globalScope.STAMP == 'undefined') {
        globalScope.STAMP = {};
    }


    if (typeof module !== 'undefined' && module.exports) {
        //check for CommonJS
        module.exports = SeSysInfoHelper;
    } else if (typeof define === 'function' && define.amd) {
        //check for AMD
        define([], function() {
            return SeSysInfoHelper;
        });
    } else {
        globalScope.STAMP.SeSysInfoHelper = SeSysInfoHelper;
    }
    /*-----  End of expose SeSysInfoHelper  ------*/

}).call(this, window)
