/**
 * filename:trackManager.js
 * desc:track
 * deps:jQuery,util,enums
 * authors:zc
 * time:2014-08-27 17:20:01
 * version:0.0.1
 * updatelog:
 * 0.0.1:the first version
 *
 **/

(function(ns) {
    var SeTrackManager = null,
        VERSION = '0.0.1',

        TRACK_URL = 'track\\',
        TRACKLIST_NAME = 'trackList',
        TRACK_NAME = 'trackList',
        SEPARATOR = '',

        SERVICE_ADD_URL = '/se_city_plan?type=add',
        SERVICE_UPDATE_URL = '/se_city_plan?type=update',
        SERVICE_REMOVE_URL = '/se_city_plan?type=delete',
        SERVICE_QUERY_URL = '/se_city_plan?type=query',

        trackUrl = 'track\\',

        earth = null,
        dataProcess = null,
        globalScope = ns;

    /**
     * 检查earth是否可用
     * @param {bool} earth是否必须,若为false,则空球时仍判定成功并打印一个警告(debugMode == true时)
     * @param {string} [position] 位置,一般是函数名
     * @return {bool} earth是否可用
     */
    var _checkEarth = function(necessary, position) {
        if (position == undefined) {
            position = ''
        }

        if (earth == null) {
            if (necessary) {
                if (globalScope.debugMode) {
                    console.error('err3005:earth initialize fail,position:' + position);
                }
                return false;
            } else {
                if (globalScope.debugMode) {
                    console.warn('warn3006:earth initialize fail,position:' + position);
                }
                return true;
            }
        } else {
            return true;
        }
    }

    var _getTrackData = function(url) {
        if (!url) {
            return {
                say: {
                    statusCode: '3001',
                    errMsg: '服务url异常'
                },
                getMe: ''
            };
        }

        var queryXml = "<QUERY><CONDITION />" +
            "<RESULT><CPROUTE><FIELD>ID</FIELD><FIELD>NAME</FIELD><FIELD>POINTS</FIELD></CPROUTE></RESULT></QUERY>";
        var response = globalScope.STAMP.seUtil.postData({
            url: url,
            data: queryXml
        });

        return response;
    }

    /**
     * 根据目标对象中的修改源对象中的指定属性
     * @param  {object} srcObj 源对象
     * @param  {object} desObj 目标对象
     * @param  {object} property 需要修改的属性
     * @return {object} 修改后的属性
     */
    var _getValue = function(srcObj, desObj, property) {
        var value = globalScope.STAMP.seUtil.invoke(desObj, property);
        if (value) {
            srcObj[property] = value;
        }

        return srcObj[property];
    }

    var _valueOrDefault = function(obj, property, defaultValue) {
        try {
            if (!obj || obj[property] == undefined || obj[property] == null) {
                return defaultValue;
            }

            return obj[property];
        } catch (e) {
            return defaultValue;
        }
    }

    /**
     * 根据xml将源route添加到目标track中
     * @param  {SETrack} 目标track
     * @param  {string}  源route节点
     * @return {void}
     */
    var _appendRoute = function(track, routeNode) {
        var p = routeNode;
        var route = earth.Factory.CreateStationRoute(p.getAttribute("id"), p.getAttribute("name"));
        //遍历route里的pass
        var elementList = p.getElementsByTagName("Pass");
        for (var i = 0; i < elementList.length; i++) {
            try {
                var passNode = elementList[i];
                var passID = passNode.getAttribute("id");
                var passName = passNode.getAttribute("name");
                var pass = earth.Factory.CreateStationPass(passID, passName);
                pass.Longitude = passNode.getAttribute("Longitude");
                pass.Latitude = passNode.getAttribute("Latitude");
                pass.Altitude = passNode.getAttribute("Altitude");
                route.Yaw = passNode.getAttribute("Heading");
                route.Pitch = passNode.getAttribute("Tilt");
                //pass.Roll = 0;
                pass.FlyHeight = passNode.getAttribute("FlyHeight");
                pass.Speed = passNode.getAttribute("Speed");
                route.Rate = passNode.getAttribute("Rate");
                route.AddStation(pass);
            } catch (e) {
                continue;
            }
        }
        track.AddStation(route);
        track.CommitChanges();
        //遍历route的个数
        // $.each($(globalScope.STAMP.seUtil.text2xml(xml)).find("Route"), function(r, p) {
        //     var route = earth.Factory.CreateStationRoute(p.getAttribute("id"), p.getAttribute("name"));
        //     //遍历route里的pass
        //     var elementList = p.getElementsByTagName("Pass");
        //     for (var i = 0; i < elementList.length; i++) {
        //         var passNode = elementList[i];
        //         var passID = passNode.getAttribute("id");
        //         var passName = passNode.getAttribute("name");
        //         var pass = earth.Factory.CreateStationPass(passID, passName);
        //         pass.Longitude = passNode.getAttribute("Longitude");
        //         pass.Latitude = passNode.getAttribute("Latitude");
        //         pass.Altitude = passNode.getAttribute("Altitude");
        //         route.Yaw = passNode.getAttribute("Heading");
        //         route.Pitch = passNode.getAttribute("Tilt");
        //         //pass.Roll = 0;
        //         pass.FlyHeight = passNode.getAttribute("FlyHeight");
        //         pass.Speed = passNode.getAttribute("Speed");
        //         route.Rate = passNode.getAttribute("Rate");
        //         route.AddStation(pass);
        //     }
        //     track.AddStation(route);
        //     track.CommitChanges();
        // });
    };

    /**
     * 根据xml将源lookat添加到目标track中
     * @param  {SETrack} 目标track
     * @param  {string}  源lookat节点
     * @return {void}
     */
    var _appendLookat = function(track, lookatNode) {
        var lookat = earth.Factory.CreateStationLookat(lookatNode.getAttribute("id"), lookatNode.getAttribute("name"));
        try {
            lookat.Longitude = lookatNode.getAttribute("Longitude");
            lookat.Latitude = lookatNode.getAttribute("Latitude");
            lookat.Altitude = lookatNode.getAttribute("Altitude");
            lookat.Heading = lookatNode.getAttribute("Heading");
            lookat.Tilt = lookatNode.getAttribute("Tilt");
            lookat.Roll = lookatNode.getAttribute("Roll");
            lookat.range = lookatNode.getAttribute("Range");
            //lookat.time = lookatNode.getAttribute("Time");
            lookat.StopTime = lookatNode.getAttribute("StopTime");
        } catch (e) {
        	
        }
        track.AddStation(lookat);
        track.CommitChanges();

        // var $lookat = $(globalScope.STAMP.seUtil.text2xml(xml)).find("Lookat").first();
        // var lookat = earth.Factory.CreateStationLookat($lookat.attr("id"), $lookat.attr("name"));
        // lookat.Longitude = $lookat.attr("Longitude");
        // lookat.Latitude = $lookat.attr("Latitude");
        // lookat.Altitude = $lookat.attr("Altitude");
        // lookat.Heading = $lookat.attr("Heading");
        // lookat.Tilt = $lookat.attr("Tilt");
        // lookat.Roll = $lookat.attr("Roll");
        // lookat.range = $lookat.attr("Range");
        // //lookat.time = $lookat.attr("Time");
        // lookat.StopTime = $lookat.attr("StopTime");
        // track.AddStation(lookat);
        // track.CommitChanges();
    };

    /**
     * 根据xml将源surround添加到目标track中
     * @param  {SETrack} 目标track
     * @param  {string}  源surround节点
     * @return {void}
     */
    var _appendSurround = function(track, surroundNode) {
        var surround = earth.Factory.CreateStationSurround(surroundNode.getAttribute("id"), surroundNode.getAttribute("name"));
        try {
            surround.Longitude = surroundNode.getAttribute("Longitude");
            surround.Latitude = surroundNode.getAttribute("Latitude");
            surround.Altitude = surroundNode.getAttribute("Altitude");
            surround.FlyHeight = surroundNode.getAttribute("FlyHeight");
            surround.Speed = surroundNode.getAttribute("Speed");
            surround.Radius = surroundNode.getAttribute("Radius");
            surround.NumberOfCycle = surroundNode.getAttribute("NumberOfCycle");
        } catch (e) {

        }
        track.AddStation(surround);
        track.CommitChanges();

        // var $surround = $(globalScope.STAMP.seUtil.text2xml(xml)).find("Surround").first();
        // var surround = earth.Factory.CreateStationSurround($surround.attr("id"), $surround.attr("name"));
        // surround.Longitude = $surround.attr("Longitude");
        // surround.Latitude = $surround.attr("Latitude");
        // surround.Altitude = $surround.attr("Altitude");
        // surround.FlyHeight = $surround.attr("FlyHeight");
        // surround.Speed = $surround.attr("Speed");
        // surround.Radius = $surround.attr("Radius");
        // surround.NumberOfCycle = $surround.attr("NumberOfCycle");
        // track.AddStation(surround);
        // track.CommitChanges();
    };

    /**
     * 插入newElement节点到targetElement节点后面
     * @param  {xmlnode} newElement
     * @param  {xmlnode} targetElement
     * @return {void}
     */
    var _insertAfter = function(newElement, targetElement) {
        var parent = targetElement.parentNode;
        if (parent.lastChild == targetElement) {
            parent.appendChild(newElement);
        } else {
            parent.insertBefore(newElement, targetElement.nextSibling);
        }
    };



    SeTrackManager = function(options) {
        if (!(this instanceof SeTrackManager)) {
            return new SeTrackManager(options);
        }

        this.initialize.apply(this, arguments);
    }

    $.extend(SeTrackManager.prototype, {
        /**
         * 初始化
         * @param {object} options:
         * options:{
         * 		earth:{object} 三维球,
         * 		[trackUrl]:{string} 轨迹保存的位置 default:'track\\'
         * }
         **/
        initialize: function(options) {
            if (options.earth == null) {
                if (globalScope.debugMode) {
                    console.warn('warn3001:null earth,position:STAMP.SeTrackManager.initialize');
                }
            }

            earth = options.earth;
            dataProcess = options.dataProcess;
            trackUrl = options.trackUrl || TRACK_URL;

        },

        /**
         * 获取运动对象
         * @param {object} options:
         * options:{
         * 			[onDynamicObject]:function(SEDynamicObject) 动态对象回调,
         * 			[onFlyObject]:function(SEDynamicObject) 飞行对象回调,
         * 			[onObjectGot]:function(SEDynamicObject) 运动对象回调,
         * 			[onDynamicListLoaded]:{function(SEDynamicList)} 获取动态物体列表完成
         * }
         * @return {void}
         **/
        getDynamicObject: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.getDynamicObject')) {
                    return;
                }

                earth.Event.OnDynamicListLoaded = function(list) {
                    earth.Event.OnDynamicListLoaded = function() {};

                    if (list == null || list.Count == 0) {
                        return;
                    }

                    for (var i = 0; i < list.Count; i++) {
                        var dynamic = list.Items(i);
                        var type = dynamic.Type;
                        try {
                            if ((type == "DynamicObject") || (type == "DynamicPeople")) {
                                if (options.onDynamicObject && typeof options.onDynamicObject === "function") {
                                    options.onDynamicObject(dynamic);
                                }
                            }
                            if (type == "FlyObject") {
                                if (options.onFlyObject && typeof options.onFlyObject === "function") {
                                    options.onFlyObject(dynamic);
                                }
                            }

                            if (options.onObjectGot && typeof options.onObjectGot === 'function') {
                                options.onObjectGot(dynamic);
                            }

                        } catch (e) {
                            continue;
                        }

                    }

                    if (options.onDynamicListLoaded && typeof options.onDynamicListLoaded === 'function') {
                        options.onDynamicListLoaded(list);
                    }
                };
                earth.DynamicSystem.ApplyDynamicList();
            } catch (e) {

            }
        },

        /**
         * 切换动态物体角色
         * @param {object} options
         * options:{
         * 		dynamicId:string 动态物体GUID,
         * 		[type]:{seEnums.seTrackingModeType} 跟随模式,default:seEnums.seTrackingModeType.thirdPerson,第三人称,
         * 		[onTrackStarted]:function(options) 切换动态物体角色回调
         * }
         **/
        startTrack: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.startTrack')) {
                    return;
                }

                if (options == null) {
                    if (globalScope.debugMode) {
                        console.error('err3001:null options,position:STAMP.SeTrackManager.startTrack');
                    }
                    return;
                }
                var did = options.dynamicId;
                var type = options.type;
                if (did == null) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不明确的运动对象(null dynamic id,position: STAMP.SeTrackManager.startTrack)');
                    }
                    return;
                }
                if (type == null) {
                    if (globalScope.debugMode) {
                        console.warn('warn3001:不明确的跟随模式,默认选择第三人称跟随模式,position:STAMP.SeTrackManager.startTrack');
                    }
                    type = 2;
                } else if (isNaN(type) || type < 1 || type > 3) {
                    if (globalScope.debugMode) {
                        console.warn('warn5001:不合法的跟随模式,默认选择第三人称跟随模式,position:STAMP.SeTrackManager.startTrack');
                    }
                    type = 2;
                }
                earth.GlobeObserver.StartTracking(did, type);

                if (options.onTrackStarted && typeof options.onTrackStarted === 'function') {
                    options.onTrackStarted(options);
                }
            } catch (e) {

            }
        },

        /**
         * 将给定的摄像机朝向转换到合理的范围[0,360),单位:度
         * @param {object} options
         * options:{
         * 		heading:{float} 摄像机朝向 default:180
         * }
         * @return {float} 转换后的摄像机朝向,单位:度
         **/
        calcHeading: function(options) {
            var heading = 180;
            try {
                if (!isNaN(options.heading)) {
                    heading = options.heading % 360;
                    if (heading < 0) {
                        heading += 360;
                    }
                } else {
                    if (globalScope.debugMode) {
                        console.warn('warn5003:不合法的摄像机朝向,position:STAMP.SeTrackManager.calcHeading');
                    }
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.calcHeading');
                }
            }

            return heading;
        },

        /**
         * 将给定的摄像机俯仰角转换到合理的范围(-90,90),单位:度
         * @param {object} options
         * options:{
         * 	tilt:{float} 摄像机俯仰角 default:15
         * }
         * @return {float} 转换后的摄像机俯仰角,单位:度
         **/
        calcTilt: function(options) {
            var tilt = 15;
            try {
                if (!isNaN(options.tilt)) {
                    tilt = options.tilt % 360;
                    if (tilt < 0) {
                        tilt += 360;
                    }

                    if (tilt > 90 && tilt <= 270) {
                        tilt -= 180;
                    } else if (tilt > 270) {
                        tilt -= 360;
                    }

                    if (tilt == 90) {
                        tilt = 89.999;
                    } else if (tilt == -90) {
                        tilt = -89.999;
                    }
                } else {
                    if (globalScope.debugMode) {
                        console.warn('warn5003:不合法的俯仰角,position:STAMP.SeTrackManager.calcTilt');
                    }
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.calcTilt');
                }
            }
            return tilt;
        },

        /**
         * 加载动态物体到对应位置
         * @param {object} options
         * options:{
         * 		dynamicId:{string} 动态物体GUID,
         * 		position:{SEObserverPose} 观察位置,
         * 		[heading]:{float} 摄像机朝向 default:180(度),
         * 		[tilt]:{float} 摄像机俯仰角 default:15(度),
         * 		[type]:{seEnums.seTrackingModeType} 跟随模式 default:seEnums.seTrackingModeType.thirdPerson,即第三人称,
         * 		[autoStart]:{bool} 加载动态物体后是否立即切换到该物体并启动控制 default:true,
         * 		[onBeforeLoad]:{function(options)} 加载动态物体前回调,若函数返回false,则不加载动态物体,也不会触发后续事件,
         * 		[onDynamicLoaded]:{function(options)} 加载动态物体成功回调,
         * 		[onTrackStarted]:{function(options)} 切换动态物体角色回调
         * }
         * @return {void}
         **/
        loadDynamicModel: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.loadDynamicModel')) {
                    return;
                }

                if (options == undefined) {
                    if (globalScope.debugMode) {
                        console.error('err3001:null options,position:STAMP.SeTrackManager.loadDynamicModel');
                    }
                    return;
                }

                var me = this;
                var did = options.dynamicId;
                if (!did) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不明确的运动对象,position:STAMP.SeTrackManager.loadDynamicModel');
                    }
                    return;
                }
                var position = options.position;
                if (!position) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不明确的目标位置,position:STAMP.SeTrackManager.loadDynamicModel');
                    }
                    return;
                }
                var type = 2; //seEnums.seTrackingModeType.thirdPerson,第三人称
                if (!isNaN(options.type) && options.type > 0 && options.type < 4) {
                    type = options.type;
                }
                var heading2 = me.calcHeading(options);
                var tilt = me.calcTilt(options);
                var autoStart = true;
                if (options.autoStart != undefined) {
                    autoStart = options.autoStart;
                }

                earth.Event.OnDocumentChanged = function(type) {
                    earth.Event.OnDocumentChanged = function() {};

                    var dynamicObj = earth.DynamicSystem.GetSphericalObject(did);
                    if (dynamicObj == null) {
                        if (globalScope.debugMode) {
                            console.warn('warn3007:获取运动对象失败,position:STAMP.SeTrackManager.loadDynamicModel');
                        }
                        return;
                    }
                    var heading = earth.GlobeObserver.Pose.Heading;

                    dynamicObj.SphericalTransform.SetPose(position.Longitude, position.Latitude, position.Altitude, heading, 0, 0);
                    earth.GlobeObserver.InitThirdTrack(heading2, tilt);

                    if (options.onDynamicLoaded && typeof options.onDynamicLoaded === 'function') {
                        options.onDynamicLoaded(options);
                    }

                    if (autoStart) {
                        var op = {
                            dynamicId: did,
                            type: type,
                            onTrackStarted: options.onTrackStarted
                        };
                        me.startTrack(op);
                    }
                };

                if (options.onBeforeLoad && typeof options.onBeforeLoad === 'function') {
                    var cbRtn = options.onBeforeLoad(options);
                    if (!cbRtn) {
                        return;
                    }
                }

                earth.DynamicSystem.LoadDynamicObject(did);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.loadDynamicModel');
                }
            }
        },

        /**
         * 开始动态物体漫游
         * @param {object} options
         * options:{
         * 		dynamicId:{string} 动态物体GUID,
         * 		[heading]:{float} 摄像机朝向 default:180(度),
         * 		[tilt]:{float} 摄像机俯仰角 default:15(度),
         * 		[type]:{seEnums.seTrackingModeType} 跟随模式 default:seEnums.seTrackingModeType.thirdPerson,即第三人称,
         * 		[autoStart]:{bool} 加载动态物体后是否立即切换到该物体并启动控制 default:true,
         * 		[onBeforeEnter]:{function(SEObserverPose)} 拾取点后,加载动态物体前回调,若函数返回false,则不加载动态物体,也不会触发后续事件
         * 		[onBeforeLoad]:{function(options)} 加载动态物体前回调,若函数返回false,则不加载动态物体,也不会触发后续事件,
         * 		[onDynamicLoaded]:{function(options)} 加载动态物体成功回调,
         * 		[onTrackStarted]:{function(options)} 切换动态物体角色回调
         * }
         * @return {void}
         **/
        enter: function(options) {
            var me = this;
            earth.Event.OnCreateGeometry = function(position) {
                earth.Event.OnCreateGeometry = function() {};

                if (options.onBeforeEnter && typeof options.onBeforeEnter === 'function') {
                    var cbRtn = options.onBeforeEnter(position);
                    if (!cbRtn) {
                        return;
                    }
                }

                if (position != null) {
                    options.position = position;
                    me.loadDynamicModel(options);

                    if (options.onEntered && typeof options.onEntered === "function") {
                        options.onEntered(position);
                    }
                }
            };
            earth.ShapeCreator.CreatePoint();
        },

        /**
         * 退出动态物体漫游
         * @param  {object} options
         * options:{
         * 		[dynamicId]:{string} 动态物体GUID,若该参数不为空,则卸载该运动物体
         * }
         * @return {void}
         */
        out: function(options) {
            earth.GlobeObserver.StopTracking(); //摄像机停止跟随
            earth.GlobeObserver.Stop(); //摄像机停止动作
            if (options && options.dynamicId) {
                earth.DynamicSystem.UnLoadDynamicObject(options.dynamicId); //卸载运动物体对象
            }
        },

        /**
         * 退出创建图元
         * @return {void}
         */
        exitCreateShape: function() {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.exitCreateShape')) {
                return;
            }

            earth.Event.OnCreateGeometry = function() {};
            earth.Environment.SetCursorStyle(209);
        },

        /**
         * 创建轨迹
         * @param  {object} options
         * options:{
         * 		{string} trackId:轨迹ID,
         * 		{string} trackName:轨迹名称
         * }
         * @return {object} {
         * 		track:{SETrack} 轨迹,
         * 		trackId:{string} 轨迹ID
         * }
         */
        createTrack: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.createTrack')) {
                    return;
                }

                var trackId = options.trackId;
                var trackName = options.trackName;
                if (!trackId) {
                    trackId = globalScope.STAMP.seUtil.createGuid(earth);
                }
                if (!trackName) {
                    trackName = '漫游轨迹';
                }

                var track = earth.Factory.CreateTrack(trackId, trackName);
                return {
                    track: track,
                    trackId: trackId
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.createTrack');
                }
            }
        },

        /**
         * 删除指定轨迹
         * @param  {object} options
         * options:{
         * 		trackId:{string} 轨迹ID
         * }
         * @return {void}
         */
        deleteTrack: function(options) {
            try {
                if (!(options && options.trackId)) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.deleteTrack');
                    }
                    return;
                }
                if (!_checkEarth(true, 'STAMP.SeTrackManager.deleteTrack')) {
                    return;
                }

                var track = earth.TrackControl.GetTrack(options.trackId);
                track.Visibility = false;
                if (track.Status === 1) {
                    //停止飞行状态
                    track.Stop();
                }

                return earth.TrackControl.DeleteTrack(options.trackId);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.deleteTrack');
                }
            }
        },

        /**
         * 根据ID获取轨迹
         * @param  {object} options
         * options:{
         * 		trackId:{string} 轨迹ID
         * }
         * @return {SETrack}
         */
        getTrack: function(options) {
            try {
                if (!(options && options.trackId)) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.getTrack');
                    }
                    return;
                }
                if (!_checkEarth(true, 'STAMP.SeTrackManager.getTrack')) {
                    return;
                }

                return earth.TrackControl.GetTrack(options.trackId)
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.getTrack');
                }

                return null;
            }
        },

        /**
         * 设置轨迹回放的速度模式
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 目标轨迹,
         * 		[tag]:{seEnums.seTrackRateType} 加/减速 default:seEnums.seTrackRateType.normal,即常速
         * }
         * @return {void}
         */
        updateRate: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:position:STAMP.SeTrackManager.updateRate');
                }
                return;
            }

            var track = options.track;
            if (!track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:position:STAMP.SeTrackManager.updateRate');
                }
                return;
            }

            var tag = _valueOrDefault(options, 'tag', 2);
            if (isNaN(tag) || tag < 0 || tag > 2) {
                tag = 2;
            }

            track.UpdateRate(tag);
            track.CommitChanges();
        },

        /**
         * 加载动态物体
         * @param  {object} options
         * options:{
         * 		dynamicId:{string} 动态物体ID
         * 		[onDynamicLoaded]:{function(options)} 加载后回调
         * 		options:{
         * 			id:{string} 动态物体ID,
         * 			type:{int} 值域:2:飞行对象加载成功
         * 		}
         * }
         * @return {void}
         */
        loadDynamicObject: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.loadDynamicObject')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:position:STAMP.SeTrackManager.loadDynamicObject');
                }
                return;
            }

            var dynamicId = options.dynamicId;
            if (!dynamicId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少的参数dynamicId,position:position:STAMP.SeTrackManager.loadDynamicObject');
                }
                return;
            }

            earth.Event.OnDocumentChanged = function(type, newGuid) {
                earth.Event.OnDocumentChanged = function() {};

                if (options.onDynamicLoaded && typeof options.onDynamicLoaded === 'function') {
                    options.onDynamicLoaded({
                        id: newGuid,
                        type: type
                    });
                }
            }
            earth.DynamicSystem.LoadDynamicObject(dynamicId);
        },

        /**
         * 卸载动态物体
         * options:{
         * 		dynamicId:{string} 动态物体ID
         * }
         * @return {void}
         */
        unloadDynamicObject: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.unloadDynamicObject')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:position:STAMP.SeTrackManager.unloadDynamicObject');
                }
                return;
            }

            var dynamicId = options.dynamicId;
            if (!dynamicId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少的参数dynamicId,position:position:STAMP.SeTrackManager.unloadDynamicObject');
                }
                return;
            }

            earth.DynamicSystem.UnLoadDynamicObject(dynamicId);
        },

        /**
         * 回放轨迹
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 待回放的轨迹,
         * 		dynamicId:{string} 动态物体ID,
         * 		[trackType]:{seEnums.seTrackingModeType} 跟随模式,default:seEnums.seTrackingModeType.thirdPerson,第三人称,
         * 		[mode]:{seEnums.seTrackRateType} 加/减速 default:seEnums.seTrackRateType.normal,即常速
         * 		[smooth]:{bool} 是否平滑进入轨迹起点 default:false,
         * 		[onTrackFinish]:{function(options)} 回放结束回调
         * 		options:{
         * 			tId:{string} 轨迹ID,
         * 			dId:{string} 动态物体ID
         * 		}
         * }
         * @return {void}
         */
        playTrack: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.playTrack')) {
                return;
            }
            var track = options.track;
            if (!track) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少的参数track,position:position:STAMP.SeTrackManager.playTrack');
                }
                return;
            }
            var dynamicId = options.dynamicId;
            if (!dynamicId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少的参数dynamicId,position:position:STAMP.SeTrackManager.playTrack');
                }
                return;
            }

            var trackType = _valueOrDefault(options, 'trackType', 2);
            var mode = _valueOrDefault(options, 'mode', 2);
            var smooth = _valueOrDefault(options, 'smooth', false);
            if (isNaN(mode) || mode < 0 || mode > 2) {
                mode = 2;
            }

            earth.Event.OnTrackFinish = function(tId, dId) {


                earth.GlobeObserver.StopTracking();
                earth.GlobeObserver.Stop();

                if (options.onTrackFinish && typeof options.onTrackFinish === 'function') {
                    options.onTrackFinish({
                        tId: tId,
                        dId: dId
                    });
                }
            }

            track.BindObject = dynamicId;
            earth.TrackControl.SetMainTrack(track.Guid, trackType);
            track.UpdateRate(mode); //维持现状
            track.CommitChanges();
            track.Play(smooth);
        },

        unbindTrackFinish: function() {
            earth.Event.OnTrackFinish = function() {};
        },

        /**
         * 创建飞行点(route)
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 要创建或追加飞行点的轨迹,
         * 		[routeId]:{string} 要创建或追加飞行点的路径,若省略该属性则新增路径，否则在该路径中追加飞行点,
         * 		[routeName]:{string} 路径名称 default:'pass',
         * 		pos:{SEVector3s} 一个表示飞行点序列的位置的三维矢量集合,
         * 		[heading]:{float} 朝向,
         * 		[tilt]:{float} 俯仰,
         * 		[rate]:{float} 速率,
         * 		params:{array<object>} [{
         * 			flyHeight:{float} 飞行高度,
         * 			flySpeed:{float} 飞行速率
         * 		}]
         * }
         * @return {SEStationRoute}
         */
        createRoute: function(options) {
            var me = this;

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.createRoute');
                }
                return null;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.createRoute');
                }
                return null;
            }
            if (!options.pos) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少飞行点序列pos,position:STAMP.SeTrackManager.createRoute');
                }
                return null;
            }

            try {
                var routeId = options.routeId;
                var routeName = options.routeName || 'pass';
                var isAppendPass = false;
                var pos = options.pos;
                var curTrack = options.track;
                var route = curTrack.GetStationByGuid(routeId);
                if (route) {
                    //追加
                    isAppendPass = true;
                } else {
                    //新增
                    if (pos.Count < 3) {
                        if (globalScope.debugMode) {
                            console.warn('warn8001:路径点不能少于3个,position:STAMP.SeTrackManager.createRoute');
                        }
                        return;
                    }
                    isAppendPass = false;
                    routeId = earth.Factory.CreateGUID();
                    route = earth.Factory.CreateStationRoute(routeId, routeName);
                }

                route.Yaw = _valueOrDefault(options, 'heading', 0);
                route.Pitch = _valueOrDefault(options, 'tilt', 0);
                route.Rate = _valueOrDefault(options, 'rate', 0.01);

                var op = {
                    station: route,
                    pos: pos,
                    params: options.params
                };
                route = me.setStationAttributes(op);

                if (route) {
                    if (!isAppendPass) {
                        curTrack.AddStation(route);
                    }
                    curTrack.CommitChanges();
                }

                return route;
            } catch (e) {
                return null;
            }
        },

        /**
         * 创建飞行点(route)
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 要创建或追加飞行点的轨迹,
         * 		[routeId]:{string} 要创建或追加飞行点的路径,若省略该属性则新增路径，否则在该路径中追加飞行点,
         * 		[routeName]:{string} 路径名称 default:'pass',
         * 		[heading]:{float} 朝向,
         * 		[tilt]:{float} 俯仰,
         * 		[rate]:{float} 速率,
         * 		params:{array<object>} [{
         * 			flyHeight:{float} 飞行高度,
         * 			flySpeed:{float} 飞行速率
         * 		}],
         * 		lineWidth:{int} 绘制路径的笔触宽度 default:2,
         * 		lineColor:{double} 绘制路径的笔触颜色 default:0xffff0000,
         * 		onRouteCreated:function(options) 创建路径成功回调
         * }
         * @return {void}
         */
        drawAndCreateRoute: function(options) {
            var me = this;
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.drawAndCreateRoute');
                }
                return;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.drawAndCreateRoute');
                }
                return;
            }

            try {
                earth.Event.OnCreateGeometry = function(pFeat) {
                    earth.Event.OnCreateGeometry = function(pFeat) {};

                    earth.ShapeCreator.Clear();

                    var routeId = options.routeId;
                    var routeName = options.routeName || 'pass';
                    var isAppendPass = false;
                    var curTrack = options.track;
                    var route = curTrack.GetStationByGuid(routeId);
                    if (route) {
                        //追加
                        isAppendPass = true;
                    } else {
                        //新增
                        if (pFeat.Count < 3) {
                            if (globalScope.debugMode) {
                                console.warn('warn8001:路径点不能少于3个,position:STAMP.SeTrackManager.drawAndCreateRoute');
                            }
                            return;
                        }
                        isAppendPass = false;
                        routeId = earth.Factory.CreateGUID();
                        route = earth.Factory.CreateStationRoute(routeId, routeName);
                    }

                    route.Yaw = _valueOrDefault(options, 'heading', 0);
                    route.Pitch = _valueOrDefault(options, 'tilt', 0);
                    route.Rate = _valueOrDefault(options, 'rate', 0.01);

                    var op = {
                        station: route,
                        pos: pFeat,
                        params: options.params
                    };
                    route = me.setStationAttributes(op);

                    if (route) {
                        if (!bAppendPass) {
                            curTrack.AddStation(route);
                        }
                        curTrack.CommitChanges();
                    }

                    if (options.onRouteCreated && typeof options.onRouteCreated === 'function') {
                        var op2 = {};
                        $.extend(op2, options, op, {
                            id: routeId,
                            name: routeName
                        });
                        options.onRouteCreated(op2);
                    }
                };

                var lineWidth = options.lineWidth;
                if (lineWidth == undefined) {
                    lineWidth = 2;
                }
                var lineColor = options.lineColor;
                if (lineColor == undefined) {
                    lineColor = 0xffff0000;
                }
                earth.ShapeCreator.CreatePolyline(lineWidth, lineColor);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.drawAndCreateRoute');
                }
            }
        },

        /**
         * 创建观察点
         * @param {object} options
         * options:{
         * 		track:{SETrack} 要创建观察点的轨迹,
         * 		[lookatName]:{string} 观察点名称 default:'注视点',
         * 		pos:{SEVector3} 一个表示观察点位置的三维矢量,
         * 		params:{
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[Heading]:{double} 摄像机朝向,
         * 			[Tilt]:{double} 摄像机俯仰角,
         * 			[Roll]:{double} 旋转,
         * 			[Range]:{double} 摄像机距观察目标的距离,
         * 			[StopTime]:{double} 观察目标停留时间
         * 		}
         * }
         * @return {SEStationLookat}
         */
        createLookat: function(options) {
            var me = this;
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.createLookat');
                }
                return null;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.createLookat');
                }
                return null;
            }
            if (!options.pos) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少观察点位置pos,position:STAMP.SeTrackManager.createLookat');
                }
                return null;
            }

            try {
                var curTrack = options.track;
                var lookatId = earth.Factory.CreateGUID();
                var lookatName = options.lookatName || "注视点";
                var lookat = earth.Factory.CreateStationLookat(lookatId, lookatName);
                lookat.Range = 100; //设置观察点默认高度值为100

                var params = {};
                $.extend(params, options.params, {
                    name: lookatName
                });
                var op = {
                    station: lookat,
                    pos: options.pos,
                    params: params
                };

                lookat = me.setStationAttributes(op);
                if (lookat) {
                    curTrack.AddStation(lookat);
                    curTrack.CommitChanges();
                }

                return lookat;
            } catch (e) {
                return null;
            }
        },

        /**
         * 创建观察点
         * @param {object} options
         * options:{
         * 		track:{SETrack} 要创建观察点的轨迹,
         * 		[lookatName]:{string} 观察点名称 default:'注视点',
         * 		params:{
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[Heading]:{double} 摄像机朝向,
         * 			[Tilt]:{double} 摄像机俯仰角,
         * 			[Roll]:{double} 旋转,
         * 			[Range]:{double} 摄像机距观察目标的距离,
         * 			[StopTime]:{double} 观察目标停留时间
         * 		},
         * 		onLookatCreated:function(options) 创建观察点成功回调
         * }
         * @return {void}
         */
        drawAndCreateLookat: function(options) {
            var me = this;
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.drawAndCreateLookat');
                }
                return;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.drawAndCreateLookat');
                }
                return;
            }

            try {
                earth.Event.OnCreateGeometry = function(pFeat) {
                    earth.Event.OnCreateGeometry = function() {};

                    earth.ShapeCreator.Clear();

                    var curTrack = options.track;
                    var lookatId = earth.Factory.CreateGUID();
                    var lookatName = options.lookatName || "注视点";
                    var lookat = earth.Factory.CreateStationLookat(lookatId, lookatName);
                    lookat.Range = 100; //设置观察点默认高度值为100

                    var params = {};
                    $.extend(params, options.params, {
                        name: lookatName
                    });
                    var op = {
                        station: lookat,
                        pos: pFeat,
                        params: params
                    };

                    lookat = me.setStationAttributes(op);
                    if (lookat) {
                        curTrack.AddStation(lookat);
                        curTrack.CommitChanges();
                    }

                    if (options.onLookatCreated && typeof options.onLookatCreated === 'function') {
                        var op2 = {};
                        $.extend(op2, options, op, {
                            id: lookatId,
                            name: lookatName
                        });
                        options.onLookatCreated(op2);
                    }
                };
                earth.ShapeCreator.CreatePoint();
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.createLookat');
                }
            }
        },

        /**
         * 创建环绕点
         * @param  {object} options
         * optiobns:{
         * 		track:{SETrack} 要创建环绕点的轨迹,
         * 		[surroundName]:{string} 环绕点名称 default:'环绕点',
         * 		pos:{SEVector3} 一个表示环绕点位置的三维矢量,
         * 		params:{
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度,
         * 			[Radius]:{double} 半径,
         * 			[NumberOfCycleoll]:{double} 环绕圈数,
         * 			[Speed]:{double} 速度
         * 		}
         * }
         * @return {SEStationSurround}
         */
        createSurround: function(options) {
            var me = this;

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.createSurround');
                }
                return null;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.createSurround');
                }
                return null;
            }
            if (!options.pos) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少环绕点位置pos,position:STAMP.SeTrackManager.createSurround');
                }
                return null;
            }

            try {
                var curTrack = options.track;
                var surroundId = earth.Factory.CreateGUID();
                var surroundName = options.surroundName || '环绕点';
                var surround = earth.Factory.CreateStationSurround(surroundId, surroundName);
                surround.Radius = 200; //环绕点范围半径默认值设置为200

                var params = {};
                $.extend(params, options.params, {
                    name: surroundName
                });
                var op = {
                    station: surround,
                    pos: options.pos,
                    params: params
                };

                surround = me.setStationAttributes(op);
                if (surround) {
                    curTrack.AddStation(surround);
                    curTrack.CommitChanges();
                }

                return surround;
            } catch (e) {
                return null;
            }
        },

        /**
         * 创建环绕点
         * @param  {object} options
         * optiobns:{
         * 		track:{SETrack} 要创建环绕点的轨迹,
         * 		[surroundName]:{string} 环绕点名称 default:'环绕点',
         * 		params:{
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度,
         * 			[Radius]:{double} 半径,
         * 			[NumberOfCycleoll]:{double} 环绕圈数,
         * 			[Speed]:{double} 速度
         * 		},
         * 		[onSurroundCreated]:{function(options)}
         * }
         * @return {void}
         */
        drawAndCreateSurround: function(options) {
            var me = this;

            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.drawAndCreateSurround');
                }
                return;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.drawAndCreateSurround');
                }
                return;
            }


            try {
                earth.Event.OnCreateGeometry = function(pFeat) {
                    earth.Event.OnCreateGeometry = function() {};

                    earth.ShapeCreator.Clear();

                    var curTrack = options.track;
                    var surroundId = earth.Factory.CreateGUID();
                    var surroundName = options.surroundName || '环绕点';
                    var surround = earth.Factory.CreateStationSurround(surroundId, surroundName);
                    surround.Radius = 200; //环绕点范围半径默认值设置为200

                    var params = {};
                    $.extend(params, options.params, {
                        name: surroundName
                    });
                    var op = {
                        station: surround,
                        pos: pFeat,
                        params: params
                    };

                    surround = me.setStationAttributes(op);
                    if (surround) {
                        curTrack.AddStation(surround);
                        curTrack.CommitChanges();
                    }

                    if (options.onSurroundCreated && typeof options.onSurroundCreated === 'function') {
                        var op2 = {};
                        $.extend(op2, options, op, {
                            id: surroundId,
                            name: surroundName
                        });
                        options.onSurroundCreated(op2);
                    }
                };
                earth.ShapeCreator.CreatePoint();
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.drawAndCreateSurround');
                }
            }
        },

        /**
         * 修改目标站点属性
         * @param {object} options
         * options:{
         * 		station:{SEStationRoute} 目标路径,
         * 		[pos]:{SEVector3s} 一个表示飞行点序列的位置的三维矢量集合,isEdit为true时该属性必须,
         * 		[isEdit]:{bool} 是否是编辑 default:false,
         * 		params:{array<object>} [{
         * 			flyHeight:{float} 飞行高度,
         * 			flySpeed:{float} 飞行速率
         * 		}]
         * }|{
         * 		station:{SEStationLookat} 目标观察点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			name:{string} 观察点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[Heading]:{double} 摄像机朝向,
         * 			[Tilt]:{double} 摄像机俯仰角,
         * 			[Roll]:{double} 旋转,
         * 			[Range]:{double} 摄像机距观察目标的距离,
         * 			[StopTime]:{double} 观察目标停留时间
         * 		}
         * }|{
         * 		station:{SEStationSurround} 目标环绕点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			name:{string} 环绕点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度,
         * 			[Radius]:{double} 半径,
         * 			[NumberOfCycleoll]:{double} 环绕圈数,
         * 			[Speed]:{double} 速度
         * 		}
         * }|{
         * 		station:{SEStationPass} 目标飞行点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			name:{string} 环绕点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度
         * 		}
         * }
         * @return {SEStation} 目标站点
         */
        setStationAttributes: function(options) {
            var me = this;
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.setStationAttributes');
                }
                return null;
            }
            var station = options.station;
            if (!station) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的目标站点,position:STAMP.SeTrackManager.setStationAttributes');
                }
                return null;
            }


            try {
                switch (station.Rtti) {
                    case 502:
                        //lookat
                        return me.setLookatAttr(options);
                        break;
                    case 503:
                        //surround
                        return me.setSurroundAttr(options);
                        break;
                    case 504:
                        //route
                        return me.setRouteAttr(options);
                        break;
                    case 501:
                        //pass
                        return me.setPassAttr(options);
                        break;
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.setStationAttributes');
                }

                return null;
            }
        },

        /**
         * 修改目标路径及其飞行点属性
         * @param {object} options
         * options:{
         * 		station:{SEStationRoute} 目标路径,
         * 		[pos]:{SEVector3s} 一个表示飞行点序列的位置的三维矢量集合,isEdit为false时该属性必须,
         * 		[isEdit]:{bool} 是否是编辑 default:false,
         * 		params:{array<object>} [{
         * 			flyHeight:{float} 飞行高度,
         * 			flySpeed:{float} 飞行速率
         * 		}]
         * }
         * @return {SEStationRoute} 目标路径
         */
        setRouteAttr: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.setRouteAttr');
                }
                return;
            }
            var station = options.station;
            if (!station) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的目标路径,position:STAMP.SeTrackManager.setRouteAttr');
                }
                return;
            }

            try {
                var name = 'Station#';

                var pos = options.pos;
                if (!options.isEdit) {
                    //新建时,pos必须
                    if (!pos) {
                        if (globalScope.debugMode) {
                            console.warn('warn3001:为空的位置序列,position:STAMP.SeTrackManager.setRouteAttr');
                        }
                        return;
                    }
                    if (pos.Count == 0) {
                        if (globalScope.debugMode) {
                            console.warn('warn3001:为空的位置序列,position:STAMP.SeTrackManager.setRouteAttr');
                        }
                        return;
                    }
                }

                var params = options.params;
                if (!params) {
                    if (globalScope.debugMode) {
                        console.warn('warn3001:为空的飞行点属性 ,position:STAMP.SeTrackManager.setRouteAttr');
                    }
                    return;
                }
                if (!$.isArray(params)) {
                    params = [params];
                }
                if (params.length == 0) {
                    if (globalScope.debugMode) {
                        console.warn('warn3001:为空的飞行点属性 ,position:STAMP.SeTrackManager.setRouteAttr');
                    }
                    return;
                }

                var pass = null;
                var childCount = station.GetChildCount();
                if (options.isEdit) {
                    //编辑路径中的飞行点 
                    for (var i = 0, l = station.GetChildCount(); i < l; i++) {
                        pass = station.GetChildAt(i);

                        var paramIndex = i < params.length ? i : params.length - 1;
                        pass.FlyHeight = params[paramIndex].flyHeight;
                        pass.Speed = params[paramIndex].flySpeed;
                    }
                } else {
                    //新建路径或已有路径追加飞行点
                    for (var i = 0; i < pos.Count; i++) {
                        pass = earth.Factory.CreateStationPass(earth.Factory.CreateGUID(), name + (childCount + i + 1));
                        pass.Longitude = pos.Items(i).X;
                        pass.Latitude = pos.Items(i).Y;
                        pass.Altitude = pos.Items(i).Z;

                        var paramIndex = i < params.length ? i : params.length - 1;
                        pass.FlyHeight = params[paramIndex].flyHeight;
                        pass.Speed = params[paramIndex].flySpeed;

                        station.AddStation(pass);
                    }
                }

                return station;
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.setRouteAttribute');
                }

                return null;
            }
        },

        /**
         * 修改目标观察点属性
         * @param {object} options
         * options:{
         * 		station:{SEStationLookat} 目标观察点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			[name]:{string} 观察点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[Heading]:{double} 摄像机朝向,
         * 			[Tilt]:{double} 摄像机俯仰角,
         * 			[Roll]:{double} 旋转,
         * 			[Range]:{double} 摄像机距观察目标的距离,
         * 			[StopTime]:{double} 观察目标停留时间
         * 		}
         * }
         * @return {SEStationLookat} 目标观察点
         */
        setLookatAttr: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.setLookatAttr');
                }
                return;
            }
            var station = options.station;
            if (!station) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的目标观察点,position:STAMP.SeTrackManager.setLookatAttr');
                }
                return;
            }

            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.setLookatAttr')) {
                    return null;
                }

                var pos = options.pos;
                var params = options.params;
                var position = earth.GlobeObserver.Pose;

                station.name = params.name || station.name || 'lookat';

                var objs = [pos, params, position];
                station.Longitude = _getValue(station, objs, 'Longitude');
                station.Latitude = _getValue(station, objs, 'Latitude');
                station.Altitude = _getValue(station, objs, 'Altitude');

                objs = [params, position];
                station.Heading = _getValue(station, objs, 'Heading');
                station.Tilt = _getValue(station, objs, 'Tilt');
                //四舍五入,底层做度与弧度转换时有精度误差
                station.Tilt = Math.round(station.Tilt);
                station.Roll = _getValue(station, objs, 'Roll');
                station.range = _getValue(station, objs, 'Range');

                station.StopTime = params.StopTime;

                return station;

            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.setLookatAttr');
                }

                return null;
            }
        },

        /**
         * 修改目标环绕点属性
         * @param {object} options
         * options:{
         * 		station:{SEStationSurround} 目标环绕点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			[name]:{string} 环绕点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度,
         * 			[Radius]:{double} 半径,
         * 			[NumberOfCycleoll]:{double} 环绕圈数,
         * 			[Speed]:{double} 速度
         * 		}
         * }
         * @return {SEStationSurround} 目标环绕点
         */
        setSurroundAttr: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.setSurroundAttr');
                }
                return;
            }
            var station = options.station;
            if (!station) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的目标环绕点,position:STAMP.SeTrackManager.setSurroundAttr');
                }
                return;
            }

            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.setSurroundAttr')) {
                    return null;
                }

                var pos = options.pos;
                var params = options.params;
                var position = earth.GlobeObserver.Pose;

                station.name = params.name || station.name || 'surround';

                var objs = [pos, params, position];
                station.Longitude = _getValue(station, objs, 'Longitude');
                station.Latitude = _getValue(station, objs, 'Latitude');
                station.Altitude = _getValue(station, objs, 'Altitude');

                station.FlyHeight = _getValue(station, params, 'FlyHeight');
                station.Radius = _getValue(station, params, 'Radius');
                station.NumberOfCycle = _getValue(station, params, 'NumberOfCycle');
                station.Speed = _getValue(station, params, 'Speed');

                return station;
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.setSurroundAttr');
                }

                return null;
            }
        },

        /**
         * 修改目标飞行点属性
         * @param {objcet} options
         * options:{
         * 		station:{SEStationPass} 目标飞行点,
         * 		pos:{SEGeoPoint} 坐标点(包括经纬度、高程),
         * 		params:{
         * 			name:{string} 环绕点名称,
         * 			[Longitude]:{double} 经度,
         * 			[Latitude]:{double} 纬度,
         * 			[Altitude]:{double} 高程,
         * 			[FlyHeight]:{double} 飞行高度
         * 		}
         * }
         */
        setPassAttr: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.setPassAttr');
                }
                return;
            }
            var station = options.station;
            if (!station) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的目标飞行点,position:STAMP.SeTrackManager.setPassAttr');
                }
                return;
            }

            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.setPassAttr')) {
                    return null;
                }

                var pos = options.pos;
                var params = options.params;
                var position = earth.GlobeObserver.Pose;

                station.name = params.name || station.name || 'station';

                var objs = [pos, params, position];
                station.Longitude = _getValue(station, objs, 'Longitude');
                station.Latitude = _getValue(station, objs, 'Latitude');
                station.Altitude = _getValue(station, objs, 'Altitude');

                station.FlyHeight = _getValue(station, params, 'FlyHeight');

                return station;
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.setPassAttr');
                }

                return null;
            }
        },

        /**
         * 从客户端获取轨迹列表
         * @param  {[type]} options
         * [options]:{
         * 		[url]:{string} 轨迹保存路径 default:'track\\'
         * }
         * @return {array<object>}[{
         *         id:{string} 轨迹ID
         *         name:{string} 轨迹名称
         * }]
         */
        getTrackListFromClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.getTrackListFromClient')) {
                return;
            }

            var url = trackUrl;
            if (options) {
                url = options.url;
                if (url == undefined) {
                    url = trackUrl;
                }
            }
            url = earth.RootPath + url + TRACKLIST_NAME;

            var result = [];
            var xmlTrackData = earth.UserDocument.LoadXmlFile(url + ".xml");
            var res = globalScope.STAMP.seUtil.xml2json(xmlTrackData);
            if (res && res.record) {
                if (res.record['ID']) {
                    result.push({
                        "id": res.record["ID"],
                        "name": res.record["NAME"]
                    });
                } else {
                    for (var r in res.record) {
                        if (res.record.hasOwnProperty(r)) {
                            result.push({
                                "id": res.record[r]["ID"],
                                "name": res.record[r]["NAME"]
                            });
                        }
                    }
                }
            }
            return result;
        },

        /**
         * 保存轨迹列表到客户端
         * @param  {object} options
         * options:{
         * 		tracks:{array<SETrack>} 轨迹数组,
         * 		[url]:{string} 轨迹保存路径
         * }
         * @return {void}
         */
        saveTrackListToClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.saveTrackListToClient')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.saveTrackListToClient');
                }
                return;
            }

            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }
            url = earth.RootPath + url + TRACKLIST_NAME;

            var tracks = options.tracks;
            if (tracks && $.isArray(tracks) && tracks.length > 0) {
                var xml = '<xml>';
                for (var i = 0; i < tracks.length; i++) {
                    xml += "\n\t<record ID='" + tracks[i].Guid + "' NAME='" + tracks[i].Name + "' />";
                };
                xml += "\n</xml>";
                earth.UserDocument.SaveXmlFile(url, xml);
            }
        },

        /**
         * 清理轨迹
         * @param  {object} options
         * [options]:{
         * 		[mode]:{int} 清理模式,可选值为:1:清理内存中的轨迹;2:根据客户端保存的xml清理轨迹;3:两者 default:3
         * }
         * @return {void}
         */
        clearTrack: function(options) {
            var me = this;
            if (!_checkEarth(true, 'STAMP.SeTrackManager.clearTrack')) {
                return;
            }

            var mode = _valueOrDefault(options, 'mode', 3);
            try {
                if (mode & 1 > 0) {
                    //清理内存中的轨迹
                    var count = earth.TrackControl.GetTrackSize();
                    if (count > 0) {
                        for (var i = 0; i < count; i++) {
                            var trackId = earth.TrackControl.GetTrackGuidByIndex(i);
                            me.deleteTrack({
                                trackId: trackId
                            });
                        }
                    }
                }
                if (mode & 2 > 0) {
                    //根据客户端保存的xml清理轨迹
                    var tracks = me.getTrackListFromClient();
                    var track = null;
                    for (var i = 0; i < tracks.length; i++) {
                        var trackId = tracks[i]["id"];
                        me.deleteTrack({
                            trackId: trackId
                        });
                    }
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.clearTrack');
                }
            }
        },

        /**
         * 保存轨迹到客户端
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 轨迹,
         * 		[url]:{string} 保存路径 default:'track\\'
         * }
         * @return {void}
         */
        saveTrackToClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.saveTrackToClient')) {
                return;
            }
            if (!(options && options.track)) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.saveTrackToClient');
                }
                return;
            }

            var me = this;
            var filename = TRACK_NAME + SEPARATOR + options.track.Guid;
            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }
            url = earth.RootPath + url + filename;

            try {
                earth.UserDocument.SaveXmlFile(url, me.track2Xml({
                    track: options.track
                }));
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.saveTrackToClient');
                }
            }

        },

        /**
         * 删除客户端的一条轨迹
         * @param  {object} options
         * options:{
         * 		trackId:{string} 轨迹ID,
         * 		[url]:{string} 保存路径 default:'track\\'
         * }
         * @return {void}
         */
        deleteTrackFromClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.deleteTrackFromClient')) {
                return;
            }
            if (!(options && options.trackId)) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.deleteTrackFromClient');
                }
                return;
            }

            var filename = TRACK_NAME + SEPARATOR + options.trackId + '.xml';
            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }

            var url2 = earth.RootPath + url + TRACKLIST_NAME;
            var trackListXml = earth.UserDocument.LoadXmlFile(url2 + ".xml");
            var trackListXmlDoc = globalScope.STAMP.seUtil.loadXMLStr(trackListXml);
            var nodes = trackListXmlDoc.getElementsByTagName("record");
            for (var i = 0; i < nodes.length; i++) {
                if (nodes[i].getAttribute("ID") == options.trackId) {
                    nodes[i].parentNode.removeChild(nodes[i]);
                }
            }
            earth.UserDocument.SaveXmlFile(url2, trackListXmlDoc.xml);

            url = earth.RootPath + url + filename;

            earth.UserDocument.DeleteXmlFile(url);
        },

        /**
         * 从客户端读取xml构造轨迹
         * @param  {object} options
         * options:{
         * 		trackId:{string} 轨迹ID,
         * 		[trackName]:{string} 轨迹名称,
         * 		[url]:{string} 轨迹保存的路径 default:'track\\',
         * 		[heading]:{double} 摄像机朝向 default:180,
         * 		[tilt]:{double} 摄像机俯仰 default:0,
         * 		[rangeScale]:{double} 摄像机距目标距离比例(目标外包框最大范围一半的倍数) default:1,
         * 		[time]:{double} 同线路上姿态变换所需时间 default:5,
         * 		[visible]:{bool} 是否可见 default:false
         * }
         * @return {SETrack} 轨迹
         */
        createTrackFromClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.createTrackFromClient')) {
                return;
            }

            if (!options || !options.trackId) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的轨迹ID,position:STAMP.SeTrackManager.createTrackFromClient');
                }

                return;
            }

            var me = this;
            var trackId = options.trackId;
            var trackName = options.trackName;
            if (trackName == undefined) {
                var trackList = me.getTrackListFromClient();
                if (trackList && trackList.length > 0) {
                    for (var i = 0, l = trackList.length; i < l; i++) {
                        if (trackList[i]['id'] == trackId) {
                            trackName = trackList[i]['name'];
                            break;
                        }
                    }
                }
            }

            var track = earth.Factory.CreateTrack(trackId, trackName);

            var filename = TRACK_NAME + SEPARATOR + trackId + '.xml';
            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }
            url = earth.RootPath + url + filename;
            var xml = earth.UserDocument.LoadXmlFile(url);

            if (xml != undefined && xml != '') {
                $(globalScope.STAMP.seUtil.text2xml(xml)).find("Track").first().children().each(function() {
                    switch (this.tagName) {
                        case "Route":
                            _appendRoute(track, this);
                            break;
                        case "Lookat":
                            _appendLookat(track, this);
                            break;
                        case "Surround":
                            _appendSurround(track, this);
                            break;
                    }
                });
            }

            var heading = _valueOrDefault(options, 'heading', 180);
            var tilt = _valueOrDefault(options, 'tilt', 0);
            var rangeScale = _valueOrDefault(options, 'rangeScale', 1);
            var time = _valueOrDefault(options, 'time', 5);
            track.InitFollowTrack(heading, tilt, rangeScale, time);
            track.CommitChanges();
            track.Visibility = _valueOrDefault(options, 'visible', false);
            return track;
        },

        /**
         * 修改客户端xml中的轨迹srcGuid和轨迹desGuid的顺序,顺序由moveType指定
         * @param  {object} options
         * options:{
         * 		[url]:{string} 轨迹保存的路径 default:'track\\',
         * 		srcGuid:{string} 轨迹ID,
         * 		desGuid:{string} 轨迹ID,
         * 		[moveType]:{string}  default:'prev'
         * }
         * @return {void}
         */
        sortTrackOnClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.sortTrackOnClient')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.sortTrackOnClient');
                }
                return;
            }

            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }
            url = earth.RootPath + url + TRACKLIST_NAME;

            var srcGuid = options.srcGuid;
            var desGuid = options.desGuid;
            var moveType = _valueOrDefault(options, 'moveType', 'prev');
            if (!srcGuid || !desGuid) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的轨迹ID,position:STAMP.SeTrackManager.sortTrackOnClient');
                }
                return;
            }

            try {

                var xmlTrackData = earth.UserDocument.LoadXmlFile(url + ".xml");
                var xmlDoc = globalScope.STAMP.seUtil.loadXMLStr(xmlTrackData);

                var sGuidNode = null,
                    dGuidNode = null;
                var records = xmlDoc.getElementsByTagName("record");
                for (var i = records.length - 1; i >= 0; i--) {
                    var guid = records[i].getAttribute("ID");
                    if (guid === srcGuid) {
                        sGuidNode = records[i];
                    } else if (guid === desGuid) {
                        dGuidNode = records[i];
                    }
                };

                if (sGuidNode && dGuidNode) {
                    if (moveType === "prev") {
                        dGuidNode.parentNode.insertBefore(sGuidNode, dGuidNode);
                    } else if (moveType === "next") {
                        _insertAfter(sGuidNode, dGuidNode);
                    };
                    earth.UserDocument.SaveXmlFile(url, xmlDoc.xml);
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.sortTrackOnClient');
                }
            }
        },

        /**
         * 修改客户端xml中的节点srcGuid和节点desGuid的顺序,顺序同moveType指定
         * @param  {object} options
         * options:{
         * 		trackId:{string} 轨迹ID,
         * 		[url]:{string} 轨迹保存的路径 default:'track\\',
         * 		srcGuid:{string} 轨迹ID,
         * 		desGuid:{string} 轨迹ID,
         * 		[moveType]:{string}  default:'prev',
         * 		[level]:{int} 节点类型,可选值为:2:pass;3:station; default:3
         * }
         * @return {void}
         */
        sortStationOnClient: function(options) {
            if (!_checkEarth(true, 'STAMP.SeTrackManager.sortStationOnClient')) {
                return;
            }
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.sortStationOnClient');
                }
                return;
            }
            var trackId = options.trackId;
            if (!trackId) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.sortStationOnClient');
                }
                return;
            }

            var srcGuid = options.srcGuid;
            var desGuid = options.desGuid;
            var moveType = _valueOrDefault(options, 'moveType', 'prev');
            var level = _valueOrDefault(options, 'level', 3);
            if (!srcGuid || !desGuid) {
                if (globalScope.debugMode) {
                    console.error('err3001:缺少必须的轨迹ID,position:STAMP.SeTrackManager.sortTrackOnClient');
                }
                return;
            }

            var filename = TRACK_NAME + SEPARATOR + trackId;
            var url = options.url;
            if (url == undefined) {
                url = trackUrl;
            }
            url = earth.RootPath + url + filename;

            try {
                var sGuidNode = null,
                    dGuidNode = null;
                var xml = earth.UserDocument.LoadXmlFile(url + '.xml');
                var xmlDoc = globalScope.STAMP.seUtil.loadXMLStr(xml);

                //遍历节点
                var pass = xmlDoc.getElementsByTagName("Pass");
                if (level === 3) {
                    for (var i = pass.length - 1; i >= 0; i--) {
                        var guid = pass[i].getAttribute("id");
                        if (guid === srcGuid) {
                            sGuidNode = pass[i];
                        } else if (guid === desGuid) {
                            dGuidNode = pass[i];
                        }
                    };
                } else if (level === 2) {
                    var nds = xmlDoc.getElementsByTagName("Track")[0].childNodes;
                    for (var j = nds.length - 1; j >= 0; j--) {
                        var guid = nds[j].getAttribute("id");
                        if (guid === srcGuid) {
                            sGuidNode = nds[j];
                        } else if (guid === desGuid) {
                            dGuidNode = nds[j];
                        }
                    }
                }

                if (sGuidNode && dGuidNode) {
                    if (moveType === "prev") {
                        dGuidNode.parentNode.insertBefore(sGuidNode, dGuidNode);
                    } else if (moveType === "next") {
                        //dGuidNode.appendChild(sGuidNode);
                        _insertAfter(sGuidNode, dGuidNode);
                    };
                    earth.UserDocument.SaveXmlFile(url, xmlDoc.xml);
                }
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.sortStationOnClient');
                }
            }
        },

        /**
         * 导出轨迹
         * @param  {object} options
         * options:{
         * 		filePath:{string} 导出路径,
         * 		[url]:{string} 轨迹保存的路径 default:'track\\'
         * }
         * @return {void}
         */
        exportTrack: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.exportTrack')) {
                    return;
                }
                if (!options || !options.filePath) {
                    if (globalScope.debugMode) {
                        console.error('err3001:缺少必须的路径参数,position:STAMP.SeTrackManager.exportTrack');
                    }
                    return;
                }

                var url = options.url;
                if (url == undefined) {
                    url = trackUrl.replace(/\\/g, '');
                }

                dataProcess.BaseFileProcess.FilePackage(earth.RootPath + url, filePath);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.exportTrack');
                }
            }
        },

        importTrack: function(options) {
            try {
                if (!_checkEarth(true, 'STAMP.SeTrackManager.importTrack')) {
                    return;
                }
                if (!options || !options.filePath) {
                    if (globalScope.debugMode) {
                        console.error('err3001:缺少必须的路径参数,position:STAMP.SeTrackManager.importTrack');
                    }
                    return;
                }

                var url = options.url;
                if (url == undefined) {
                    url = trackUrl.replace(/\\/g, '');
                }

                dataProcess.BaseFileProcess.FileUnPackage(filePath, earth.RootPath);
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.importTrack');
                }
            }
        },

        /**
         * 将轨迹数据转换为xml格式的字符串
         * @param  {ojbect} options
         * options:{
         * 		track:{SETrack} 轨迹
         * }
         * @return {string} xml格式和字符串
         */
        track2Xml: function(options) {
            if (!(options && options.track)) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.track2Xml');
                }
                return;
            }

            var me = this;
            var track = options.track;
            var result = "";
            var station = null;
            try {
                result += "<Track>";
                for (var i = 0; i < track.GetChildCount(); i++) {
                    station = track.GetChildAt(i);
                    switch (station.Rtti) {
                        case 502:
                            //lookat
                            result += '<Lookat id="' + station.Guid +
                                '" name="' + station.Name +
                                '" Longitude="' + station.Longitude +
                                '" Latitude="' + station.Latitude +
                                '" Altitude="' + station.Altitude +
                                '" Heading="' + station.Heading +
                                '" Tilt="' + station.Tilt +
                                '" Range="' + station.Range +
                                '" StopTime="' + station.StopTime +
                                '"></Lookat>';
                            break;
                        case 503:
                            //surround
                            result += '<Surround id="' + station.Guid +
                                '" name="' + station.Name +
                                '" Longitude="' + station.Longitude +
                                '" Latitude="' + station.Latitude +
                                '" Altitude="' + station.Altitude +
                                '" FlyHeight="' + station.FlyHeight +
                                '" Speed="' + station.Speed +
                                '" Radius="' + station.Radius +
                                '" Count="' + station.NumberOfCycle +
                                '"></Surround>';
                            break;
                        case 504:
                            //route
                            result += me.route2Xml({
                                route: station
                            });
                            break;
                    }
                }
                result += "</Track>";
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.track2Xml');
                }

                result = '';
            }

            return result;
        },

        /**
         * 将路径数据转换为xml格式的字符串
         * @param  {ojbect} options
         * options:{
         * 		route:{SETrack} 路径
         * }
         * @return {string} xml格式和字符串
         */
        route2Xml: function(options) {
            if (!(options && options.route)) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的路径,position:STAMP.SeTrackManager.route2Xml');
                }
                return;
            }

            var route = options.route;
            var result = "";
            var station = null;
            try {
                result += '<Route id="' + route.Guid + '" name="' + route.Name + '">';
                for (var i = 0; i < route.GetChildCount(); i++) {
                    station = route.GetChildAt(i);
                    result += '<Pass id="' + station.Guid +
                        '" name="' + station.Name +
                        '" Longitude="' + station.Longitude +
                        '" Latitude="' + station.Latitude +
                        '" Altitude="' + station.Altitude +
                        '" Heading="' + route.Yaw +
                        '" Tilt="' + route.Pitch +
                        '" FlyHeight="' + station.FlyHeight +
                        '" Speed="' + station.Speed +
                        '" Rate="' + route.Rate +
                        '"></Pass>';
                }
                result += '</Route>';
            } catch (e) {
                if (globalScope.debugMode) {
                    console.error(e.description + ',position:STAMP.SeTrackManager.route2Xml');
                }

                result = '';
            }

            return result;
        },

        /**
         * 保存轨迹信息到服务器
         * @param  {object} options
         * options:{
         * 		track:{SETrack} 轨迹
         * 		serviceIp:{string} 服务IP(包括端口)
         * 		[async]:{bool} 是否异步 default:true，
         * 		[isUpdate]:{bool} 是否是更新 default:false,
         * 		[url]:{string} 服务url,
         * 		[success]:function(data, textStatus, jqXHR) 请求成功回调,
         * 		[error]:function(XMLHttpRequest, textStatus, errorThrown) 请求服务失败回调
         * }
         * @return {object} 异步时可用
         * response:{
         * 		say:{
         * 			statusCode:{string} 服务请求状态码,0000:成功,0001:失败,
         * 			errMsg:{string} 错误信息
         * 		}
         * 		getMe:{string} 请求的数据,若服务请求失败,则该值为空字符串
         * }
         */
        saveTrackToServer: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.saveTrackToDb');
                }
                return;
            }
            if (!options.track) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,position:STAMP.SeTrackManager.saveTrackToDb');
                }
                return;
            }
            if (!options.url) {
                if (!options.serviceIp) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不合法的服务器IP,position:STAMP.SeTrackManager.saveTrackToDb');
                    }
                    return;
                }
            }

            var me = this;
            var track = options.track;
            var async = true;
            if (options.async != undefined) {
                async = options.async;
            }
            var url = '';
            var saveXml = me.track2Xml({
                track: track
            });
            if (saveXml == '') {
                return;
            }

            if (options.isUpdate) {
                //更新
                var beginStr = "<CPROUTE><ID>" + track.Guid + "</ID><NAME>" + track.name + "</NAME>";
                var endStr = "</CPROUTE>";
                saveXml = beginStr + saveXml + endStr;

                //建议调用者传入完整的url
                if (options.url) {
                    url = options.url;
                } else {
                    //无url时,需要serviceIp
                    url = /http:\/\//ig.test(options.serviceIp) ? options.serviceIp : 'http://' + options.serviceIp;
                    url += SERVICE_UPDATE_URL;
                }
            } else {
                //插入
                var beginStr = "<CPROUTE><CONDITION><ID>='" + track.Guid + "'</ID></CONDITION><CONTENT>";
                var endStr = "</CONTENT></CPROUTE>";
                saveXml = beginStr + saveXml + endStr;

                //建议调用者传入完整的url
                if (options.url) {
                    url = options.url;
                } else {
                    //无url时,需要serviceIp
                    url = /http:\/\//ig.test(options.serviceIp) ? options.serviceIp : 'http://' + options.serviceIp;
                    url += SERVICE_ADD_URL;
                }
            }

            var response = {
                say: {
                    statusCode: '0001',
                    errMsg: ''
                },
                getMe: ''
            };
            $.ajax({
                url: url,
                async: async,
                cache: false,
                type: 'POST',
                dataType: 'text',
                data: saveXml,
                success: function(data, textStatus, jqXHR) {
                    if (/true/.test(data)) {
                        response.say.statusCode = '0000';
                        response.say.errMsg = '';
                        response.getMe = data;
                    } else {
                        response.say.statusCode = '0001';
                        response.say.errMsg = '请求服务失败';
                        response.getMe = data;
                    }

                    if (options.success && typeof options.success === 'function') {
                        options.success(data, textStatus, jqXHR);
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    response.say.statusCode = '0001';
                    response.say.errMsg = '请求服务失败';
                    response.getMe = '';

                    if (options.error && typeof options.error === 'function') {
                        options.error(data, textStatus, jqXHR);
                    }
                }
            });

            if (!async) {
                return response;
            }
        },

        /**
         * 从服务器上删除指定轨迹
         * @param  {object} options
         * options:{
         * 		trackId:{SETrack} 轨迹ID
         * 		serviceIp:{string} 服务IP(包括端口)
         * 		[async]:{bool} 是否异步 default:true，
         * 		[url]:{string} 服务url,
         * 		[success]:function(data, textStatus, jqXHR) 请求成功回调,
         * 		[error]:function(XMLHttpRequest, textStatus, errorThrown) 请求服务失败回调
         * }
         * @return {object} 异步时可用
         * response:{
         * 		say:{
         * 			statusCode:{string} 服务请求状态码,0000:成功,0001:失败,
         * 			errMsg:{string} 错误信息
         * 		}
         * 		getMe:{string} 请求的数据,若服务请求失败,则该值为空字符串
         * }
         */
        deleteTrackOnServer: function(options) {
            if (!(options && options.trackId)) {
                if (globalScope.debugMode) {
                    console.error('err3001:不明确的轨迹,,position:STAMP.SeTrackManager.deleteTrackInDb');
                }
                return;
            }

            if (!options.url) {
                if (!options.serviceIp) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不合法的服务器IP,position:STAMP.SeTrackManager.deleteTrackInDb');
                    }
                    return;
                }
            }

            var async = true;
            if (options.async != undefined) {
                async = options.async;
            }
            var url = '';
            if (options.url) {
                url = options.url;
            } else {
                url = /http:\/\//ig.test(options.serviceIp) ? options.serviceIp : 'http://' + options.serviceIp;
                url += SERVICE_REMOVE_URL;
            }
            var deleteXml = "<CPROUTE><ID> ='" + options.trackId + "' </ID></CPROUTE>";

            var response = {
                say: {
                    statusCode: '0001',
                    errMsg: ''
                },
                getMe: ''
            };
            $.ajax({
                url: url,
                async: async,
                cache: false,
                type: 'POST',
                dataType: 'text',
                data: deleteXml,
                success: function(data, textStatus, jqXHR) {
                    if (/true/.test(data)) {
                        response.say.statusCode = '0000';
                        response.say.errMsg = '';
                        response.getMe = data;
                    } else {
                        response.say.statusCode = '0001';
                        response.say.errMsg = '请求服务失败';
                        response.getMe = data;
                    }

                    if (options.success && typeof options.success === 'function') {
                        options.success(data, textStatus, jqXHR);
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    response.say.statusCode = '0001';
                    response.say.errMsg = '请求服务失败';
                    response.getMe = '';

                    if (options.error && typeof options.error === 'function') {
                        options.error(data, textStatus, jqXHR);
                    }
                }
            });

            if (!async) {
                return response;
            }
        },

        /**
         * 从服务器获取轨迹信息
         * @param  {[type]} options [description]
         * @return {[type]}         [description]
         */
        getTrackFromServer: function(options) {
            if (!options) {
                if (globalScope.debugMode) {
                    console.error('err3001:null options,position:STAMP.SeTrackManager.getTrackFromDb');
                }
            }
            if (!options.url) {
                if (!options.serviceIp) {
                    if (globalScope.debugMode) {
                        console.error('err3001:不合法的服务器IP,position:STAMP.SeTrackManager.getTrackFromDb');
                    }
                    return;
                }
            }

            var async = true;
            if (options.async != undefined) {
                async = options.async;
            }
            var url = '';
            if (options.url) {
                url = options.url;
            } else {
                url = /http:\/\//ig.test(options.serviceIp) ? options.serviceIp : 'http://' + options.serviceIp;
                url += SERVICE_QUERY_URL;
            }
            var queryXml = "<QUERY><CONDITION />" +
                "<RESULT><CPROUTE><FIELD>ID</FIELD><FIELD>NAME</FIELD><FIELD>POINTS</FIELD></CPROUTE></RESULT></QUERY>";

            var response = {
                say: {
                    statusCode: '0001',
                    errMsg: ''
                },
                getMe: ''
            };
            $.ajax({
                url: url,
                async: async,
                cache: false,
                type: 'POST',
                dataType: 'text',
                data: queryXml,
                success: function(data, textStatus, jqXHR) {
                    if (/true/.test(data)) {
                        response.say.statusCode = '0000';
                        response.say.errMsg = '';
                        response.getMe = data;
                    } else {
                        response.say.statusCode = '0001';
                        response.say.errMsg = '请求服务失败';
                        response.getMe = data;
                    }

                    if (options.success && typeof options.success === 'function') {
                        options.success(data, textStatus, jqXHR);
                    }
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    response.say.statusCode = '0001';
                    response.say.errMsg = '请求服务失败';
                    response.getMe = '';

                    if (options.error && typeof options.error === 'function') {
                        options.error(data, textStatus, jqXHR);
                    }
                }
            });

            if (!async) {
                return response;
            }
        },

        /*==========  the version  ==========*/
        version: VERSION
    });



    /*===========================================
     =            expose SeTrackManager            =
     ===========================================*/
    if (globalScope == undefined) {
        globalScope = window;
    }
    if (globalScope.STAMP == undefined) {
        globalScope.STAMP = {};
    }

    if ((typeof module) !== 'undefined' && module.exports) {
        //check for CommonJS
        module.exports = SeTrackManager;
    } else if ((typeof define) === 'function' && define.amd) {
        //check for AMD
        define([], function() {
            return SeTrackManager;
        });
    } else {
        globalScope.STAMP.SeTrackManager = SeTrackManager;
    }
    /*-----  End of expose SeTrackManager  ------*/



})(window);
