//! fileName : userDataFactory.js
//! desc : 三维球中用户数据的创建
//! deps : jquery.js util.js
//! authors : YanZhipeng
//! time : 2014-09-02 10:33
//! version : 0.0.1
//! updateLog : 
(function(ns) {
    /************************************
        Constants
    ************************************/
    var SeUserDataFactory = {},
        VERSION = '0.0.1',
        earth,
        globalScope = typeof ns != 'undefined' ? ns : window,
        debugMode = typeof ns.debugMode != 'undefined' ? ns.debugMode : false,
        XML_FILE_NAME = 'MyPlace',
        //模型数组
        elements = [],
        //存放模型的数据的对象的数组
        objects = [],
        //摄像机数组
        cameras = [],
        //需要给模型赋的值，与从xml中读取相关，
        //数组中字符串的意义，'element对应的属性名称，xml中对应的名称，0-属于属性、1-属于标签名称'
        attributesMap = {
            common: [
                'id,id,0',
                'checked,checked,0',
                'guid,id,0',
                'name,name,0',
                'type,type,0',
                'desc,Description,1',
                'fillColor,FillColor,1'
            ],
            junbiao: [
                'lineColor,LineColor,1',
                'lineWidth,LineWidth,1',
                'Shadow,Shadow,1',
                'drawOrder,DrawOrder,1',
                'selectable,Selectable,1',
                'editable,Editable,1'
            ]
        },
        b = 1;

    /************************************
        Internal function
    ************************************/
    /**
     * 检查当前球是否已经定义
     * @param {string} [position] 出现问题的函数路径
     * @param  {bool} [forcedNeedEarth] 是否强制需要使用球，默认为true
     * @return {bool}                 当前球是否被定义
     */
    function _checkEarth(position, forcedNeedEarth) {
        if (typeof position == 'undefined' || position == null) {
            position = '';
        }
        if (typeof forcedNeedEarth == 'undefined' || forcedNeedEarth == null || typeof forcedNeedEarth != 'boolean') {
            forcedNeedEarth = true;
        }
        var earthIsDefined = false;
        if (typeof earth != 'undefined') {
            earthIsDefined = true;
        } else {
            if (globalScope.debugMode) {
                if (forcedNeedEarth) {
                    console.error('err3005, 位置：' + position + '.  三维球没有初始化，三维球被强制需要。请确保执行了"initialize"函数');
                } else {
                    console.warn('warn3006, 位置：' + position + '.  三维球没有正常初始化, 建议执行"initialize"函数对其进行初始化');
                }
            }
        }
        return earthIsDefined;
    };

    /**
     *   是否绑定对象到Earth上
     */
    function _attachObject(checked, obj) {
        if (Number(checked) === 1) {
            obj.Visibility = true;
            obj.Selectable = true;
        } else {
            obj.Visibility = false;
            obj.Selectable = false;
        };
    };

    //暂时保留函数
    function _setTransform(element, obj) {
        /*var rotation = obj.getElementsByTagName("Rotation")[0].text;
        var scale = obj.getElementsByTagName("Scale")[0].text;
        var position = obj.getElementsByTagName("Position")[0].text;

        rotation = rotation.split(",");
        scale = scale.split(",");
        position = position.split(",");

        var objType = element.rtti;
        if (objType === 250 || objType === 251 || objType === 252 || objType === 253 || objType === 254 || objType === 255 || objType === 256 || objType === 257 || objType === 258 || objType === 259 || objType === 260) {

        } else {
            // element.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
            //element.SphericalTransform.SetScaleEx (scale[0], scale[1], scale[2]);
            // element.SphericalTransform.SetLocationEx(Number(position[0]), Number(position[1]), Number(position[2]));
        }*/
    };




    /**
     * 从本地取得xml文件内容
     * @return {xml} xml对象
     */
    function _loadElementsXmlFromLocal() {
        url = earth.Environment.RootPath + "userdata\\" + XML_FILE_NAME + ".xml";
        var xmlData = earth.UserDocument.loadXmlFile(url); // 得到xml数据；
        if (xmlData == "") {
            var userdataGUID = earth.Factory.createGuid();
            var userdataXml = "<Xml><ElementDocument id='" + userdataGUID + "' name='MyPlace' im0='folderOpen.gif' type='folder' checked='0' open='false'></ElementDocument></Xml>";
            var userdata = earth.Environment.RootPath + "userData\\" + filename;
            earth.UserDocument.saveXmlFile(userdata, userdataXml);
            xmlData = userdataXml;
        }
        return globalScope.STAMP.seUtil.loadXMLStr(xmlData);
    };

    /**
     * 将之前保存的记录进行模型构造
     * @return {Array} 所有构造好的模型数组
     */
    function _initializeAllElements() {
        var xmlData = _loadElementsXmlFromLocal();
        elements = [];
        if (xmlData == null) {
            return;
        }
        var element = xmlData.getElementsByTagName("Element");
        for (var i = 0; i < element.length; i++) {
            var obj = {};
            var elementObj = element[i];
            var checked = elementObj.getAttribute("checked");
            obj.id = elementObj.getAttribute("id");
            obj.guid = obj.id;
            obj.name = elementObj.getAttribute("name");
            //alert(obj.name);
            obj.type = elementObj.getAttribute("type");
            obj.desc = elementObj.getElementsByTagName("Description")[0].text;
            var rotation = elementObj.getElementsByTagName("Rotation")[0].text;
            var scale = elementObj.getElementsByTagName("Scale")[0].text;
            rotation = rotation.split(",");
            scale = scale.split(",");
            if (elementObj.getElementsByTagName("FillColor")[0]) {
                obj.fillColor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.fillcolor = obj.fillColor;
            }
            obj.longitude = elementObj.getElementsByTagName("Longitude")[0].text;
            obj.latitude = elementObj.getElementsByTagName("Latitude")[0].text;
            obj.altitude = elementObj.getElementsByTagName("Altitude")[0].text;
            obj.type = elementObj.getAttribute("type");
            var objType = Number(obj.type);
            //军标部分
            if (objType === 250 || objType === 251 || objType === 252 || objType === 253 || objType === 254 || objType === 255 || objType === 256 || objType === 257 || objType === 258 || objType === 259 || objType === 260) {

                obj.fillColor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.lineColor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.fillcolor = obj.fillColor;
                obj.linecolor = obj.lineColor;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.Shadow = elementObj.getElementsByTagName("Shadow")[0].text;

                //新增属性
                if (objType === 250 || objType === 251 || objType === 252 || objType === 253 || objType === 254 || objType === 255 || objType === 256 || objType === 260) {
                    obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;
                    obj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                    obj.editable = elementObj.getElementsByTagName("Editable")[0].text;
                }
                var PointList = elementObj.getElementsByTagName("Coordinates")[0].text;
                var vecs = PointList.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var k = 0; k < vecs.length; k++) {
                    var v = vecs[k].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                obj.guid = obj.id;
                if (objType === 250) {
                    var editObj = earth.Factory.CreateElementPlotSArrow(obj.id, obj.name);
                } else if (objType === 253) {
                    var editObj = earth.Factory.CreateElementPlotCustomArrow(obj.id, obj.name);
                } else if (objType === 252) {
                    var editObj = earth.Factory.CreateElementPlotTailSArrow(obj.id, obj.name);
                } else if (objType === 254) {
                    var editObj = earth.Factory.CreateElementPlotCustomTailArrow(obj.id, obj.name);
                } else if (objType === 251) {
                    var editObj = earth.Factory.CreateElementPlotEqualSArrow(obj.id, obj.name);
                } else if (objType === 255) {
                    var editObj = earth.Factory.CreateElementPlotDoubleArrow(obj.id, obj.name);
                } else if (objType === 256) {
                    var editObj = earth.Factory.CreateElementPlotXArrow(obj.id, obj.name);
                } else if (objType === 260) {
                    var editObj = earth.Factory.CreateElementPlotAssemblyArea(obj.id, obj.name);
                } else if (objType === 257) {
                    var editObj = earth.Factory.CreateElementPlotTriangleFlag(obj.id, obj.name);
                } else if (objType === 258) {
                    var editObj = earth.Factory.CreateElementPlotRectFlag(obj.id, obj.name);
                } else if (objType === 259) {
                    var editObj = earth.Factory.CreateElementPlotCurveFlag(obj.id, obj.name);
                }
                editObj.BeginUpdate();
                editObj.name = obj.name;

                //新属性
                if (objType === 250 || objType === 251 || objType === 252 || objType === 253 || objType === 254 || objType === 255 || objType === 256 || objType === 260) {
                    editObj.drawOrder = obj.drawOrder; //暂不支持
                    editObj.selectable = obj.selectable;
                    editObj.editable = obj.editable;
                    editObj.AltitudeType = parseInt(obj.Shadow);
                }

                //alert(editObj.AltitudeType);
                editObj.SetControlPointArray(obj.vector3s);
                var linestyle = editObj.LineStyle;
                linestyle.LineWidth = obj.lineWidth;
                //alert(obj.lineColor);
                linestyle.LineColor = parseInt("0x" + obj.lineColor.toString().substring(1).toLowerCase());
                var fillstyle = editObj.FillStyle;
                fillstyle.FillColor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());

                editObj.EndUpdate();
                earth.AttachObject(editObj);
                //军标旋转后直接修改了原始坐标 不用给rotation赋值
                _setTransform(editObj, elementObj);
                _attachObject(checked, editObj);
                elements.push(editObj);
                objects.push(obj);
            } else if (obj.type === "fire" || obj.type === "mist" || obj.type === "fountain" || obj.type === "nozzle" || obj.type === "SprayNozzle") {

                var pType = 0;
                switch (obj.type) {
                    case "fire":
                        pType = 0;
                        break;
                    case "mist":
                        pType = 1;
                        break;
                    case "fountain":
                        pType = 2;
                        break;
                    case "nozzle":
                        pType = 3;
                        break;
                    case "SprayNozzle":
                        pType = 4;
                        break;
                    default:
                        pType = 0;
                        break;
                }

                var particle = earth.factory.CreateElementParticle(obj.id, obj.name);
                obj.guid = obj.id;
                //获取位置
                var pt = elementObj.getElementsByTagName("Position")[0].text.split(",");
                particle.SphericalTransform.SetLocationEx(pt[0], pt[1], pt[2]);
                particle.BeginUpdate();
                //火 = 0,  烟 = 1,  喷泉 = 2, 直流水枪 = 3,   喷雾水枪 = 4
                particle.Type = pType;
                particle.name = obj.name;
                particle.EndUpdate();
                earth.AttachObject(particle);
                _setTransform(particle, elementObj);
                var checked = elementObj.getAttribute("checked");

                if (Number(checked) === 1) {
                    particle.Visibility = true;
                    particle.Selectable = true;
                } else {
                    particle.Visibility = false;
                    particle.Selectable = true;
                }
                elements.push(particle);
                objects.push(obj);
                // particles[record["ID"]] = particle;

            } else if (obj.type === "dWater") {
                var editObj = earth.Factory.CreateElementWater(obj.id, obj.name);
                obj.guid = obj.id;
                var pt = elementObj.getElementsByTagName("Position")[0].text.split(",");
                editObj.SphericalTransform.SetLocationEx(pt[0], pt[1], pt[2]);
                editObj.BeginUpdate();
                editObj.name = obj.name;
                editObj.EndUpdate();
                earth.AttachObject(editObj);
                _setTransform(editObj, elementObj);
                elements.push(editObj);
                objects.push(obj);
            } else if (Number(obj.type) === 220) { //初始化线条
                obj.lineColor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;
                obj.fillcolor = obj.fillColor;
                obj.linecolor = obj.lineColor;
                obj.arrow = elementObj.getElementsByTagName("isShadowArrow")[0].text;
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                var Coordinates = elementObj.getElementsByTagName("Coordinates")[0].text;
                var vecs = Coordinates.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var j = 0; j < vecs.length; j++) {
                    var v = vecs[j].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                var visibility = elementObj.selectSingleNode("Visibility").text;
                //iconObj.Visibility = visibility;
                var lineObj = earth.Factory.CreateElementLine(obj.id, obj.name);
                //var lineColor=obj.lineColor.toString().substring(1).toLowerCase();
                lineObj.BeginUpdate();
                lineObj.name = obj.name;
                lineObj.SetPointArray(obj.vector3s);
                lineObj.Visibility = true;
                var Linestyle = lineObj.LineStyle;
                Linestyle.LineWidth = obj.lineWidth;
                lineObj.IsAddArrow = obj.arrow;
                Linestyle.LineColor = parseInt("0x" + obj.lineColor.toString().substring(1).toLowerCase());
                lineObj.AltitudeType = parseInt(obj.shadow);

                // lineObj.SphericalTransform.SetRotationEx(rotation[0],rotation[1],rotation[2]);
                //lineObj.SphericalTransform.SetScaleEx (scale[0],scale[1],scale[2]);

                //获取经纬度值 获取postion 传入
                //lineObj.SphericalTransform.SetLocationEx();

                lineObj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                lineObj.editable = elementObj.getElementsByTagName("Editable")[0].text;
                lineObj.drawOrder = obj.drawOrder;

                lineObj.EndUpdate();
                _setTransform(lineObj, elementObj);
                earth.AttachObject(lineObj);
                _attachObject(checked, lineObj);
                elements.push(lineObj);
                objects.push(obj);
            } else if (Number(obj.type) === 245) { //多边形贴图 矩形贴图
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                obj.expandX = elementObj.getElementsByTagName("TextureTiltX")[0].text;
                obj.expandY = elementObj.getElementsByTagName("TextureTiltY")[0].text;
                obj.picture = elementObj.getElementsByTagName("TextureImagePath")[0].text;
                obj.fillcolor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.linecolor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.textture = elementObj.getElementsByTagName("TextureMode")[0].text;
                //新增
                // obj.perimeter = elementObj.getElementsByTagName("Perimeter")[0].text;
                // obj.area = elementObj.getElementsByTagName("Area")[0].text;
                //obj.drawOrder=elementObj.getElementsByTagName("DrawOrder")[0].text;

                var Coordinates = elementObj.getElementsByTagName("Coordinates")[0].text;

                var vecs = Coordinates.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var j = 0; j < vecs.length; j++) {
                    var v = vecs[j].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                var polygon = earth.Factory.CreateElementTexturePolygon(obj.id, obj.name);
                polygon.BeginUpdate();
                polygon.name = obj.name;
                polygon.SetExteriorRing(obj.vector3s);

                polygon.FillStyle.FillColor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase());
                polygon.TextureImagePath = obj.picture; //"E:\\Users\\Administrator\\Desktop\\trunk\\trunk\\css\\zTree\\img\\diy\\1.png";
                polygon.TextureMode = parseInt(obj.textture); //  0 无纹理        1 平铺纹理   2 拉伸（必须四个顶点）
                polygon.TextureTiltX = parseInt(obj.expandX); //  横向平铺重复次shu
                polygon.TextureTiltY = parseInt(obj.expandY); //  纵向平铺重复次shu

                polygon.LineStyle.LineColor = parseInt("0x" + obj.linecolor.toString().substring(1).toLowerCase()); //options.linecolor;
                polygon.LineStyle.LineWidth = parseInt(obj.lineWidth);
                polygon.AltitudeType = parseInt(obj.shadow); // 0绝对 1贴地
                //polygon.drawOrder = obj.drawOrder;
                //polygon.SphericalTransform.SetRotationEx(rotation[0],rotation[1],rotation[2]);
                //polygon.SphericalTransform.SetScaleEx (scale[0],scale[1],scale[2]);
                polygon.EndUpdate();
                _setTransform(polygon, elementObj);
                earth.AttachObject(polygon);
                _attachObject(checked, polygon);
                elements.push(polygon);
                objects.push(obj);
            } else if (Number(obj.type) === 211) { //多边形
                obj.fillColor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.lineColor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                //新增
                obj.perimeter = elementObj.getElementsByTagName("Perimeter")[0].text;
                obj.area = elementObj.getElementsByTagName("Area")[0].text;
                obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;

                obj.linecolor = obj.lineColor;
                obj.fillcolor = obj.fillColor;
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                var Coordinates = elementObj.getElementsByTagName("Coordinates")[0].text;
                var vecs = Coordinates.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var j = 0; j < vecs.length; j++) {
                    var v = vecs[j].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                var polygon = earth.Factory.CreateElementPolygon(obj.id, obj.name);
                polygon.BeginUpdate();
                polygon.name = obj.name;
                polygon.SetExteriorRing(obj.vector3s);
                polygon.FillStyle.FillColor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());
                polygon.LineStyle.LineColor = parseInt("0x" + obj.lineColor.toString().substring(1).toLowerCase());

                polygon.LineStyle.LineWidth = obj.lineWidth;
                polygon.drawOrder = obj.drawOrder;
                //alert(obj.shadow);
                polygon.AltitudeType = obj.shadow;
                polygon.Visibility = true;

                polygon.EndUpdate();
                _setTransform(polygon, elementObj);
                earth.AttachObject(polygon);
                _attachObject(checked, polygon);
                objects.push(obj);
                elements.push(polygon);
            } else if (Number(obj.type) === 239) { //圆

                obj.Radius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.fillcolor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.lineColor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                //新增
                obj.perimeter = elementObj.getElementsByTagName("Perimeter")[0].text;
                obj.area = elementObj.getElementsByTagName("Area")[0].text;

                //obj.drawOrder=elementObj.getElementsByTagName("DrawOrder")[0].text;
                obj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                obj.editable = elementObj.getElementsByTagName("Editable")[0].text;

                //圆目前不支持该属性
                //obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;

                obj.guid = obj.id;
                obj.linecolor = obj.lineColor;
                var circle = earth.Factory.CreateElementCircle(obj.id, obj.name);
                circle.name = obj.name;
                var tran = circle.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                circle.BeginUpdate();
                circle.Radius = obj.Radius;

                circle.Selectable = obj.selectable;
                circle.Editable = obj.editable;
                //circle.drawOrder = obj.drawOrder;
                circle.AltitudeType = obj.shadow;
                circle.FillStyle.FillColor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase()); // 0xccff00ff;
                circle.LineStyle.LineWidth = obj.lineWidth;
                circle.LineStyle.LineColor = parseInt("0x" + obj.lineColor.toString().substring(1).toLowerCase()); // 0xccff00ff;
                circle.EndUpdate();
                earth.AttachObject(circle);
                _setTransform(circle, elementObj);
                _attachObject(checked, circle);
                elements.push(circle);
                objects.push(obj);
            } else if (Number(obj.type) === 243) { //椭圆
                obj.fillcolor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.ShortRadius = elementObj.getElementsByTagName("ShortRadius")[0].text;
                obj.LongRadius = elementObj.getElementsByTagName("LongRadius")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.lineColor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                //新增
                obj.perimeter = elementObj.getElementsByTagName("Perimeter")[0].text;
                obj.area = elementObj.getElementsByTagName("Area")[0].text;
                obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;

                obj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                obj.editable = elementObj.getElementsByTagName("Editable")[0].text;

                obj.guid = obj.id;
                obj.linecolor = obj.lineColor;
                var ellipse = earth.Factory.CreateElementEllipse(obj.id, obj.name);
                ellipse.name = obj.name;
                var tran = ellipse.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                ellipse.BeginUpdate();
                ellipse.LongRadius = obj.LongRadius;
                ellipse.ShortRadius = obj.ShortRadius;
                //ellipse.drawOrder = obj.drawOrder;
                ellipse.Selectable = obj.selectable;
                ellipse.Editable = obj.editable;

                var fillstyle = ellipse.FillStyle;
                fillstyle.FillColor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase()); //0xcc00ff00;
                ellipse.LineStyle.LineWidth = obj.lineWidth;
                ellipse.LineStyle.LineColor = parseInt("0x" + obj.lineColor.toString().substring(1).toLowerCase()); //0xcc00ff00;
                ellipse.AltitudeType = obj.shadow;
                ellipse.EndUpdate();
                ellipse.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                ellipse.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                earth.AttachObject(ellipse);
                _setTransform(ellipse, elementObj);
                _attachObject(checked, ellipse);
                elements.push(ellipse);
                objects.push(obj);
            } else if (Number(obj.type) === 240) {
                obj.fillcolor = elementObj.getElementsByTagName("FillColor")[0].text;
                obj.linecolor = elementObj.getElementsByTagName("LineColor")[0].text;
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                var ArcCenterList = elementObj.getElementsByTagName("ArcCenter")[0].text.split(",");
                obj.Angle = elementObj.getElementsByTagName("Angle")[0].text;
                obj.radius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                //新增
                obj.perimeter = elementObj.getElementsByTagName("Perimeter")[0].text;
                obj.area = elementObj.getElementsByTagName("Area")[0].text;
                //obj.drawOrder=elementObj.getElementsByTagName("DrawOrder")[0].text;

                obj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                obj.editable = elementObj.getElementsByTagName("Editable")[0].text;

                obj.guid = obj.id;
                obj.ArcCenter = earth.Factory.CreateVector3();
                obj.ArcCenter.X = ArcCenterList[0];
                obj.ArcCenter.Y = ArcCenterList[1];
                obj.ArcCenter.Z = ArcCenterList[2];
                //obj.ArcCenter.AddVector(v31);
                var sector = earth.Factory.CreateElementSector(obj.id, obj.name);
                sector.name = obj.name;
                var tran = sector.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                sector.BeginUpdate();
                sector.ArcCenter = obj.ArcCenter;
                sector.Angle = obj.Angle;
                sector.radius = obj.radius;
                sector.Selectable = obj.selectable;
                sector.Editable = obj.editable;
                //sector.drawOrder = obj.drawOrder;
                sector.AltitudeType = obj.shadow;
                sector.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                sector.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var fillstyle = sector.FillStyle;
                fillstyle.FillColor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase()); //4294963200;
                var sectorStyle = sector.LineStyle;

                sectorStyle.LineWidth = obj.lineWidth;
                sectorStyle.LineColor = parseInt("0x" + obj.linecolor.toString().substring(1).toLowerCase()); //4294963200;

                sector.EndUpdate();
                sector.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                sector.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                earth.AttachObject(sector);
                _setTransform(sector, elementObj);
                _attachObject(checked, sector);
                elements.push(sector);
                objects.push(obj);
            } else if (Number(obj.type) === 241) {
                if (elementObj.getElementsByTagName("ShadowType")[0]) {
                    obj.shadow = elementObj.getElementsByTagName("ShadowType")[0].text;
                };

                if (elementObj.getElementsByTagName("LineColor")[0]) {
                    obj.linecolor = elementObj.getElementsByTagName("LineColor")[0].text;
                }

                if (obj.arrow = elementObj.getElementsByTagName("isShadowArrow")[0]) {
                    obj.arrow = elementObj.getElementsByTagName("isShadowArrow")[0].text;
                };
                obj.lineWidth = elementObj.getElementsByTagName("LineWidth")[0].text;
                obj.drawOrder = elementObj.getElementsByTagName("DrawOrder")[0].text;
                var PointList;
                if (elementObj.getElementsByTagName("Coordinates")[0]) {
                    PointList = elementObj.getElementsByTagName("Coordinates")[0].text;
                    var vecs = PointList.split(" ");
                    var v3s = earth.Factory.CreateVector3s();
                    for (var k = 0; k < vecs.length; k++) {
                        var v = vecs[k].split(",");
                        var v3 = earth.Factory.CreateVector3();
                        v3.X = v[0];
                        v3.Y = v[1];
                        v3.Z = v[2];
                        v3s.AddVector(v3);
                    }
                    obj.vector3s = v3s;
                }

                obj.guid = obj.id;
                var curve = earth.Factory.CreateElementCurve(obj.id, obj.name);
                curve.name = obj.name;
                curve.BeginUpdate();
                if (obj.vector3s) {
                    curve.SetControlPointArray(obj.vector3s);
                }
                var linestyle = curve.LineStyle;
                linestyle.LineWidth = obj.lineWidth;
                if (obj.linecolor) {
                    linestyle.LineColor = parseInt("0x" + obj.linecolor.toString().substring(1).toLowerCase()); //0xffffff00;
                }
                curve.AltitudeType = obj.shadow;
                if (obj.arrow) {
                    curve.IsAddArrow = obj.arrow;
                }
                curve.drawOrder = obj.drawOrder;
                //curve.SphericalTransform.SetRotationEx(rotation[0],rotation[1],rotation[2]);
                //curve.SphericalTransform.SetScaleEx (scale[0],scale[1],scale[2]);
                curve.EndUpdate();
                earth.AttachObject(curve);
                _setTransform(curve, elementObj);
                _attachObject(checked, curve);
                elements.push(curve);
                objects.push(obj);
            } else if (Number(obj.type) === 216) {
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.Radius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                obj.editable = elementObj.getElementsByTagName("Editable")[0].text;
                obj.objectFlagType = elementObj.getElementsByTagName("ObjectFlagType")[0].text;

                var fillcolor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase());
                var radData = obj.Radius;
                //var params = this.getTextureParams(obj);      
                var sphere = earth.Factory.CreateElementSphere(obj.id, obj.name);
                /*var  rotation=obj.Rotation;*/
                var tran = sphere.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                sphere.name = obj.name;
                sphere.BeginUpdate();
                sphere.Radius = obj.Radius;
                sphere.selectable = obj.selectable;
                sphere.editable = obj.editable;
                sphere.Underground = obj.objectFlagType;
                sphere.FillColor = fillcolor;
                sphere.Visibility = true;
                //sphere.Name=obj.name;
                sphere.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                sphere.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var materialStyles = sphere.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[1];
                }
                sphere.EndUpdate();
                earth.AttachObject(sphere);
                _setTransform(sphere, elementObj);
                _attachObject(checked, sphere);
                elements.push(sphere);
                objects.push(obj);
            } else if (Number(obj.type) === 202) { //立方体
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.widthValue = elementObj.getElementsByTagName("Width")[0].text;
                obj.longValue = elementObj.getElementsByTagName("Length")[0].text;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;

                var fillcolor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());
                var visibility = true;
                var box = earth.Factory.CreateElementBox(obj.id, obj.name);
                box.name = obj.name;
                var tran = box.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                box.BeginUpdate();
                box.Width = obj.widthValue;
                box.Length = obj.longValue;
                box.Height = obj.heightValue;
                box.FillColor = fillcolor;
                box.Visibility = visibility;
                box.Name = obj.name;
                box.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                box.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var materialStyles = box.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }
                box.EndUpdate();
                earth.AttachObject(box);
                _setTransform(box, elementObj);
                _attachObject(checked, box);
                elements.push(box);
                objects.push(obj);
            } else if (Number(obj.type) === 207) { //立体多边形
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;
                var PointList = elementObj.getElementsByTagName("PointList")[0].text;
                var vecs = PointList.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var k = 0; k < vecs.length; k++) {
                    var v = vecs[k].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                obj.guid = obj.id;
                var fillcolor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());
                var volume = earth.Factory.CreateElementVolume(obj.id, obj.name);
                volume.name = obj.name;
                _setTransform(volume, elementObj);
                volume.BeginUpdate();
                volume.Underground = true;

                volume.Height = obj.heightValue;
                volume.Visibility = true;
                //volume.Name = obj.name;
                volume.Vectors = obj.vector3s;
                volume.FillColor = fillcolor;
                //volume.Description=obj.description;
                var materialStyles = volume.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }
                volume.EndUpdate();
                earth.AttachObject(volume);

                _attachObject(checked, volume);
                elements.push(volume);
                objects.push(obj);
            } else if (Number(obj.type) === 203) { //圆柱
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.bottomRadius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;

                var fillcolor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase());
                var cylinder = earth.Factory.CreateElementCylinder(obj.id, obj.name);
                cylinder.name = obj.name;
                var tran = cylinder.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                cylinder.BeginUpdate();
                cylinder.Radius = obj.bottomRadius;
                cylinder.Height = obj.heightValue;
                cylinder.Visibility = true;
                //cylinder.Name=obj.name;
                cylinder.FillColor = fillcolor;
                cylinder.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                cylinder.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var materialStyles = cylinder.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }

                cylinder.EndUpdate();
                earth.AttachObject(cylinder);
                _setTransform(cylinder, elementObj);
                _attachObject(checked, cylinder);
                elements.push(cylinder);
                objects.push(obj);
            } else if (Number(obj.type) === 204) { //圆锥
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.topRadius = elementObj.getElementsByTagName("TopRadius")[0].text;
                obj.bottomRadius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;

                var fillcolor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());
                var radiusBottom = obj.BottomRadius;
                var radiusTop = obj.TopRadius;
                var height = obj.Height;
                var cone = earth.Factory.CreateElementCone(obj.id, obj.name);
                cone.name = obj.name;
                var tran = cone.SphericalTransform;

                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                cone.BeginUpdate();
                cone.BottomRadius = obj.bottomRadius;
                cone.TopRadius = obj.topRadius;
                cone.Height = obj.heightValue;
                //cone.Name=obj.name;
                cone.Visibility = true;
                cone.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                cone.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                cone.FillColor = fillcolor;
                var materialStyles = cone.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }
                cone.EndUpdate();
                earth.AttachObject(cone);
                _setTransform(cone, elementObj);
                _attachObject(checked, cone);
                elements.push(cone);
                objects.push(obj);
            } else if (Number(obj.type) === 205) { //棱柱
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.bottomRadius = elementObj.getElementsByTagName("Radius")[0].text;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;
                obj.sides = elementObj.getElementsByTagName("Sides")[0].text;

                var fillcolor = parseInt("0x" + obj.fillcolor.toString().substring(1).toLowerCase());
                var prism = earth.Factory.CreateElementPrism(obj.id, obj.name);
                prism.name = obj.name;
                var tran = prism.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                prism.BeginUpdate();
                prism.Radius = obj.bottomRadius;
                prism.Height = obj.heightValue;
                prism.Sides = obj.sides;
                prism.Visibility = true;
                prism.FillColor = fillcolor;
                prism.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                prism.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var materialStyles = prism.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }

                prism.EndUpdate();
                earth.AttachObject(prism);
                _setTransform(prism, elementObj);
                _attachObject(checked, prism);
                elements.push(prism);
                objects.push(obj);
            } else if (Number(obj.type) === 206) { //棱锥
                var texturePath = elementObj.getElementsByTagName("TexturePath");
                var texturePathCount = texturePath.length;
                var texturesArr = [];
                for (var j = 0; j < texturePathCount; j++) {
                    texturesArr.push(texturePath[j].text);
                }
                obj.guid = obj.id;
                obj.textures = texturesArr;
                obj.texturePath = texturesArr;
                obj.topRadius = elementObj.getElementsByTagName("TopRadius")[0].text;
                obj.bottomRadius = elementObj.getElementsByTagName("BottomRadius")[0].text;
                obj.heightValue = elementObj.getElementsByTagName("Height")[0].text;
                obj.sides = elementObj.getElementsByTagName("Sides")[0].text;

                var fillcolor = parseInt("0x" + obj.fillColor.toString().substring(1).toLowerCase());
                var pyramid = earth.Factory.CreateElementPyramid(obj.id, obj.name);
                pyramid.name = obj.name;
                var tran = pyramid.SphericalTransform;
                tran.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                pyramid.BeginUpdate();

                pyramid.Sides = obj.sides;
                pyramid.BottomRadius = obj.bottomRadius;
                pyramid.TopRadius = obj.topRadius;
                pyramid.Height = obj.heightValue;
                pyramid.Visibility = true;
                pyramid.Name = obj.name;
                pyramid.FillColor = fillcolor;
                pyramid.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                pyramid.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                var materialStyles = pyramid.MaterialStyles;
                var count = materialStyles.Count;
                for (var m = 0; m < count; m++) {
                    var materialStyle = materialStyles.Items(m);
                    materialStyle.DiffuseTexture = obj.textures[m];
                }
                pyramid.EndUpdate();
                earth.AttachObject(pyramid);
                _setTransform(pyramid, elementObj);
                _attachObject(checked, pyramid);
                elements.push(pyramid);
                objects.push(obj);
            } else if (Number(obj.type) === 229) {
                obj.strLink = elementObj.getElementsByTagName("Link")[0].text;
                if (elementObj.getElementsByTagName("Tag").length > 0) {
                    obj.tag = elementObj.getElementsByTagName("Tag")[0].text;
                } else {
                    obj.tag = 3;
                }
                obj.link = obj.strLink;
                obj.guid = obj.id;
                var model = earth.Factory.CreateEditModelByLocal(obj.id, obj.name, obj.strLink, obj.tag);
                //model.SetIsCollapse(true)   ;
                model.SphericalTransform.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                model.Name = obj.name;
                model.Selectable = true;
                model.Editable = true;
                earth.AttachObject(model);
                _setTransform(model, elementObj);
                elements.push(model);
                objects.push(obj);
            } else if (Number(obj.type) === 217) {
                obj.width = elementObj.getElementsByTagName("Width")[0].text;
                obj.height = elementObj.getElementsByTagName("Height")[0].text;
                obj.imag = elementObj.getElementsByTagName("Icon")[0].text;
                obj.iconFileName = obj.imag;
                obj.guid = obj.id;
                var billboard = earth.Factory.CreateElementSimpleBillboard(obj.id, obj.name);
                billboard.name = obj.name;
                billboard.SphericalTransform.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                billboard.BeginUpdate();
                billboard.Width = Number(obj.width);
                billboard.Height = Number(obj.height);
                billboard.Image = obj.imag;
                //billboard.Name=obj.name;
                billboard.Visibility = true;
                billboard.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                billboard.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                billboard.EndUpdate();
                _setTransform(billboard, elementObj);
                earth.AttachObject(billboard);
                _attachObject(checked, billboard);
                elements.push(billboard);
                objects.push(obj);
            } else if (Number(obj.type) === 209) {
                var strIcon = elementObj.getElementsByTagName("Icon")[0].text;
                var strHightlightIcon = elementObj.getElementsByTagName("Icon")[1].text;
                obj.iconNormalFileName = strIcon;
                obj.guid = obj.id;
                obj.iconSelectedFileName = strHightlightIcon;
                var myicon = earth.Factory.CreateElementIcon(obj.id, obj.name);
                myicon.name = obj.name;
                myicon.Create(obj.longitude, obj.latitude, obj.altitude, strIcon, strHightlightIcon, obj.name);

                //新增属性
                myicon.textFormat = parseInt("0x100");
                var textColor = elementObj.getElementsByTagName("TextColor")[0].text;
                myicon.textColor = parseInt("0x" + textColor.toString().substring(1).toLowerCase());
                myicon.textHorizontalScale = elementObj.getElementsByTagName("TextHorizontalScale")[0].text;
                myicon.textVerticalScale = elementObj.getElementsByTagName("TextVerticalScale")[0].text;
                myicon.showHandle = elementObj.getElementsByTagName("ShowHandle")[0].text;
                myicon.handleHeight = elementObj.getElementsByTagName("HandleHeight")[0].text;
                var handleColor = elementObj.getElementsByTagName("HandleColor")[0].text;
                myicon.handleColor = parseInt("0x" + handleColor.toString().substring(1).toLowerCase());
                myicon.minVisibleRange = elementObj.getElementsByTagName("MinVisibleRange")[0].text;
                myicon.MaxVisibleRange = elementObj.getElementsByTagName("MaxVisibleRange")[0].text;
                myicon.selectable = elementObj.getElementsByTagName("Selectable")[0].text;
                myicon.editable = elementObj.getElementsByTagName("Editable")[0].text;

                myicon.Visibility = true;
                //myicon.Name= obj.name;
                myicon.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                myicon.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                earth.AttachObject(myicon);
                _setTransform(myicon, elementObj);
                _attachObject(checked, myicon);
                elements.push(myicon);
                objects.push(obj);
            } else if (Number(obj.type) === 280) {
                obj.floorHeight = elementObj.getElementsByTagName("FloorHeight")[0].text;
                obj.floorCount = elementObj.getElementsByTagName("FloorCount")[0].text;
                obj.roofType = elementObj.getElementsByTagName("RoofType")[0].text;
                obj.floorTexture = elementObj.getElementsByTagName("FloorTexture")[0].text;
                obj.roofTexture = elementObj.getElementsByTagName("RoofTexture")[0].text;
                obj.roofColor = elementObj.getElementsByTagName("RoofColor")[0].text;
                obj.floorColor = elementObj.getElementsByTagName("FloorColor")[0].text;
                var PointList = elementObj.getElementsByTagName("PointList")[0].text;
                var vecs = PointList.split(" ");
                var v3s = earth.Factory.CreateVector3s();
                for (var k = 0; k < vecs.length; k++) {
                    var v = vecs[k].split(",");
                    var v3 = earth.Factory.CreateVector3();
                    v3.X = v[0];
                    v3.Y = v[1];
                    v3.Z = v[2];
                    v3s.AddVector(v3);
                }
                obj.vector3s = v3s;
                obj.guid = obj.id;
                obj.roofTypeNode = obj.roofType;
                var simpleBuilding = earth.factory.CreateSimpleBuilding(obj.id, obj.name);
                simpleBuilding.name = obj.name;
                simpleBuilding.SphericalTransform.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                simpleBuilding.BeginUpdate();
                var polygon = earth.factory.CreatePolygon();
                polygon.AddRing(obj.vector3s);
                simpleBuilding.SetPolygon(0, polygon);
                var floorCount = parseInt(obj.floorCount);
                var floorHeight = parseFloat(obj.floorHeight);
                simpleBuilding.SetFloorsHeight(obj.floorHeight * obj.floorCount);
                simpleBuilding.SetFloorHeight(obj.floorHeight);
                simpleBuilding.SetRoofType(obj.roofType);
                var roofcolor = parseInt("0x" + obj.roofColor.toString().substring(1).toLowerCase());
                var floorcolor = parseInt("0x" + obj.floorColor.toString().substring(1).toLowerCase());
                simpleBuilding.FloorsColor = floorcolor;
                simpleBuilding.RoofColor = roofcolor;
                var floorMats = simpleBuilding.GetFloorsMaterialStyles();
                floorMats.Items(0).DiffuseTexture = obj.roofTexture;
                floorMats.Items(1).DiffuseTexture = obj.roofTexture;
                for (var m = 2; m < floorMats.Count; m++) {
                    floorMats.Items(m).DiffuseTexture = obj.floorTexture;
                }
                var roofMats = simpleBuilding.GetRoofMaterialStyles();
                //for(var i=0; i<roofMats.Count; i++){
                roofMats.Items(1).DiffuseTexture = obj.roofTexture;
                //}
                simpleBuilding.SphericalTransform.SetRotationEx(rotation[0], rotation[1], rotation[2]);
                simpleBuilding.SphericalTransform.SetScaleEx(scale[0], scale[1], scale[2]);
                simpleBuilding.EndUpdate();
                earth.AttachObject(simpleBuilding);

                _setTransform(simpleBuilding, elementObj);
                _attachObject(checked, simpleBuilding);
                elements.push(simpleBuilding);
                objects.push(obj);
            } else if (Number(obj.type) === 244) {
                if (EditLayer === null) {
                    createLayer();
                }
                obj.guid = obj.id;
                obj.path = elementObj.getElementsByTagName("Link")[0].text;
                obj.height = elementObj.getElementsByTagName("Height")[0].text;
                var grid = earth.Factory.CreateElementGrid(obj.id, obj.name, obj.path, obj.height /*高度2千米*/ );
                grid.name = obj.name;
                grid.SphericalTransform.SetLocationEx(obj.longitude, obj.latitude, obj.altitude);
                EditLayer.BeginUpdate();
                earth.AttachObject(grid);
                if (Number(checked) === 1) {
                    //EditLayer.AttachObject(grid);
                    grid.Visibility = true;
                } else {
                    grid.Visibility = false;
                    //EditLayer.DetachObject(grid);
                }

                EditLayer.EndUpdate();
                _setTransform(grid, elementObj);
                objects.push(obj);
                elements.push(grid);
            }

        }
        return elements;
    };


    /**
     * 创建图形元素
     * @param  {seEnums.sePrimitiveType} flag         创建的图元的类型
     * @param  {object} geometry 几何图形
     * @param {object} options 创建的图元的属性信息
     * @return {object}              返回新创建的图元
     */
    function _createPrimitives(flag, geometry, options) {
        if (typeof geometry == 'undefined' || geometry == null) {
            if (globalScope.debugMode) {
                console.error('err3005: 位置，STAMP.SeUserDataFactory.createPrimitives. 几何信息参数为空');
            }
            return null;
        }
        var flagArr = [];
        if (flag.indexOf("-") >= 0) {
            flagArr = flag.split("-");
            flag = flagArr[0];
        }
        if (typeof options == 'undefined' || options == null) {
            var options = {};
        }
        options.action = "add";
        options.earth = earth;
        earth.ShapeCreator.Clear();
        if (flag == "createline") {
            if (geometry.Count > 0) {
                options.path = earth.Environment.RootPath;
                options.type = 220;
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "line";
                }
                var lineGuid = earth.Factory.CreateGUID();
                options.guid = lineGuid;
                elementLine = earth.Factory.CreateElementLine(lineGuid, options.name);
                elementLine.BeginUpdate();
                elementLine.SetPointArray(geometry);
                elementLine.Visibility = true;
                elementLine.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase());
                elementLine.LineStyle.LineWidth = options.linewidth;
                elementLine.AltitudeType = options.shadow;
                elementLine.Visibility = true;
                elementLine.IsAddArrow = options.arrow;
                //新增属性
                elementLine.Selectable = options.selectable;
                elementLine.Editable = options.editable;
                elementLine.DrawOrder = options.drawOrder;

                elementLine.EndUpdate();
                earth.AttachObject(elementLine);

                //保存到本地xml
                options.lineLength = elementLine.length;
                _createElement(options, elementLine);
                elements.push(elementLine);
                earth.ShapeCreator.Clear();
            };
        } else if (flag == "icon") {
            options.path = earth.Environment.RootPath + "userres" + "\\";
            options.type = 209;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "icon";
            }
            var guid = earth.Factory.CreateGuid();
            options.guid = guid;
            //var position = earth.GlobeObserver.Pick(geometry.X, geometry.Y);
            options.longitude = geometry.Longitude;
            options.latitude = geometry.Latitude;
            options.altitude = geometry.Altitude;

            var strHightlightIcon = options.iconSelectedFileName;
            var iconName = options.name;
            var myicon = earth.Factory.CreateElementIcon(options.guid, options.name);
            myicon.Create(options.longitude, options.latitude, options.altitude, options.iconNormalFileName, options.iconSelectedFileName, options.name);
            myicon.Visibility = true;

            //新增属性
            myicon.textFormat = parseInt("0x100");
            myicon.textColor = parseInt("0x" + options.textColor.toString().substring(1).toLowerCase());
            myicon.textHorizontalScale = options.textHorizontalScale;
            myicon.textVerticalScale = options.textVerticalScale;
            myicon.showHandle = options.showHandle;
            myicon.handleHeight = options.handleHeight;
            myicon.handleColor = parseInt("0x" + options.handleColor.toString().substring(1).toLowerCase());
            myicon.minVisibleRange = options.minVisibleRange;
            myicon.maxVisibleRange = options.maxVisibleRange;
            myicon.selectable = options.selectable;
            myicon.editable = options.editable;

            earth.AttachObject(myicon);
            elements.push(myicon);
            _createElement(options, myicon);
        } else if (flag == "createpolygon") {
            if (geometry.Count) {
                options.type = 211;
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "polygon";
                }
                options.path = earth.Environment.RootPath;

                var polygonGuid = earth.Factory.CreateGUID();
                options.guid = polygonGuid;

                var elementPolygon = earth.Factory.CreateElementPolygon(polygonGuid, options.name);
                elementPolygon.BeginUpdate();
                elementPolygon.SetExteriorRing(geometry);
                elementPolygon.LineStyle.LineWidth = options.linewidth;
                elementPolygon.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase());
                elementPolygon.AltitudeType = options.shadow;
                elementPolygon.FillStyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase());
                elementPolygon.DrawOrder = options.drawOrder;
                elementPolygon.Selectable = options.selectable;
                elementPolygon.Editable = options.editable;

                elementPolygon.EndUpdate();
                earth.AttachObject(elementPolygon);

                options.perimeter = elementPolygon.perimeter;
                options.area = elementPolygon.area;

                _createElement(options, elementPolygon);
                elements.push(elementPolygon);
                earth.ShapeCreator.Clear();
            }
        } else if (flag == "createcurve") { //曲线
            if (geometry.Count) {
                options.path = earth.Environment.RootPath;
                options.type = 241;
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "curve";
                }
                var guid = earth.Factory.CreateGuid();
                options.guid = guid;

                var curve = earth.Factory.CreateElementCurve(guid, options.name);
                curve.BeginUpdate();
                curve.SetControlPointArray(geometry);
                var linestyle = curve.LineStyle;
                linestyle.LineWidth = options.linewidth;
                linestyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); //0xffffff00;
                curve.AltitudeType = options.shadow;
                curve.IsAddArrow = options.arrow;
                //新增属性
                curve.Selectable = options.selectable;
                curve.Editable = options.editable;
                curve.DrawOrder = options.drawOrder;

                curve.EndUpdate();
                elements.push(curve);
                earth.AttachObject(curve);
                _createElement(options, curve);
                earth.ShapeCreator.Clear();
            }
        } else if (flag == "createcircle") { //圆
            options.type = 239;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "circle";
            }
            options.path = earth.Environment.RootPath;

            //半径
            options.radius = geometry.Radius;
            var guid = earth.Factory.CreateGuid();
            options.guid = guid;
            var circle = earth.Factory.CreateElementCircle(guid, options.name);
            var tran = circle.SphericalTransform;
            tran.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            circle.BeginUpdate();

            options.perimeter = circle.perimeter;
            options.area = circle.area;

            //新增属性
            circle.Selectable = options.selectable;
            circle.Editable = options.editable;
            circle.AltitudeType = options.shadow;
            circle.Radius = options.radius;
            circle.FillStyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); // 0xccff00ff;
            circle.LineStyle.LineWidth = options.linewidth;
            circle.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); // 0xccff00ff;
            circle.EndUpdate();

            //这里有问题 属性取不到值
            options.perimeter = circle.perimeter;
            options.area = circle.area;

            elements.push(circle);
            earth.AttachObject(circle);
            _createElement(options, circle);
            earth.ShapeCreator.Clear();
        } else if (flag === "createellipse") { //椭圆
            options.path = earth.Environment.RootPath;
            options.type = 243;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "ellipse";
            }
            options.longRadius = geometry.longRadius;
            options.shortRadius = geometry.shortRadius;

            var guid = earth.Factory.CreateGuid();
            options.guid = guid;

            var ellipse = earth.Factory.CreateElementEllipse(guid, options.name);
            var tran = ellipse.SphericalTransform;
            tran.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            ellipse.BeginUpdate();
            ellipse.AltitudeType = options.shadow;
            //新增属性
            ellipse.Selectable = options.selectable;
            ellipse.Editable = options.editable;

            options.perimeter = ellipse.perimeter;
            options.area = ellipse.area;

            ellipse.LongRadius = options.longRadius;
            ellipse.ShortRadius = options.shortRadius;
            var fillstyle = ellipse.FillStyle;
            fillstyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //0xcc00ff00;
            var linestyle = ellipse.LineStyle;
            linestyle.LineWidth = options.linewidth;
            linestyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); //0xcc00ff00;
            ellipse.EndUpdate();
            elements.push(ellipse);
            earth.AttachObject(ellipse);
            _createElement(options, ellipse);
            earth.ShapeCreator.Clear();
        } else if (flag == "createsector") { //扇形
            options.type = 240;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "sector";
            }
            options.path = earth.Environment.RootPath;

            options.angle = geometry.angle;
            options.radius = geometry.radius;
            options.ArcCenter = geometry.ArcCenter;

            var guid = earth.Factory.CreateGuid();
            options.guid = guid;
            var sector = earth.Factory.CreateElementSector(guid, options.name);
            var tran = sector.SphericalTransform;
            tran.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            sector.BeginUpdate();
            sector.ArcCenter = geometry.ArcCenter;
            sector.radius = options.radius;
            //新增属性
            sector.Selectable = options.selectable;
            sector.Editable = options.editable;
            sector.AltitudeType = options.shadow;
            sector.Angle = Number(options.angle);

            var linestyle = sector.LineStyle;
            linestyle.LineWidth = options.linewidth;
            linestyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); //4294963200;
            var fillstyle = sector.FillStyle;
            fillstyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //4294963200;

            sector.EndUpdate();
            elements.push(sector);
            earth.AttachObject(sector);
            _createElement(options, sector);
            earth.ShapeCreator.Clear();
        } else if (flag == "createTexturePolygon") { //矢量面贴图
            if (geometry.Count) {
                options.type = 245;
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "TexturePolygon";
                }
                var guid = earth.Factory.CreateGuid();
                options.guid = guid;
                var polygon = earth.Factory.CreateElementTexturePolygon(options.guid, options.name);
                polygon.BeginUpdate();
                polygon.SetExteriorRing(geometry);

                polygon.FillStyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase());
                polygon.TextureImagePath = options.picture; //"E:\\Users\\Administrator\\Desktop\\trunk\\trunk\\css\\zTree\\img\\diy\\1.png";
                polygon.TextureMode = parseInt(options.textture); //  0 无纹理        1 平铺纹理   2 拉伸（必须四个顶点）
                polygon.TextureTiltX = parseInt(options.expandX); //  横向平铺重复次shu
                polygon.TextureTiltY = parseInt(options.expandY); //  纵向平铺重复次shu

                polygon.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); //options.linecolor;
                polygon.LineStyle.LineWidth = parseInt(options.linewidth);
                polygon.AltitudeType = parseInt(options.shadow); // 0绝对 1贴地
                //alert(polygon.area);
                //新增属性
                polygon.Selectable = options.selectable;
                polygon.Editable = options.editable;
                //polygon.DrawOrder = options.drawOrder;

                polygon.EndUpdate();

                earth.AttachObject(polygon);
                _createElement(options, polygon);
                elements.push(polygon);
                earth.ShapeCreator.Clear();
            }
        } else if (flag == "createrectangle") { //矩形贴图
            if (geometry.Count) {
                options.type = 245;
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "rectangle";
                }
                var guid = earth.Factory.CreateGuid();
                options.guid = guid;
                var rectangle = earth.Factory.CreateElementTexturePolygon(guid, options.name);
                rectangle.BeginUpdate();
                rectangle.SetExteriorRing(geometry);
                rectangle.name = options.name;
                rectangle.FillStyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase());
                rectangle.TextureImagePath = options.picture; //"E:\\Users\\Administrator\\Desktop\\trunk\\trunk\\css\\zTree\\img\\diy\\1.png";
                rectangle.TextureMode = parseInt(options.textture); //  0 无纹理        1 平铺纹理   2 拉伸（必须四个顶点）
                rectangle.TextureTiltX = parseInt(options.expandX); //  横向平铺重复次shu
                rectangle.TextureTiltY = parseInt(options.expandY); //  纵向平铺重复次shu
                //alert(rectangle.area);
                rectangle.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase()); //options.linecolor;
                rectangle.LineStyle.LineWidth = parseInt(options.linewidth);
                //alert(options.shadow);
                rectangle.AltitudeType = parseInt(options.shadow); // 0绝对 1贴地

                //新增属性
                rectangle.Selectable = options.selectable;
                rectangle.Editable = options.editable;

                rectangle.EndUpdate();

                earth.AttachObject(rectangle);
                _createElement(options, rectangle);
                elements.push(rectangle);
                earth.ShapeCreator.Clear();
            }
        } else if (flag === "parallelLines") { //平行线
            var coll = options.originalGeometry;
            var offset = options.offset;
            if (coll) {
                if (coll.Rtti === 241) {
                    options.type = 241;
                } else {
                    options.type = 220;
                }
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = coll.Name;
                }
                options.drawOrder = 0;
                options.action = "parallel";
                //属性先从Coll中获取到 增加到userdata中
                // options.linecolor = coll.LineStyle.LineColor;
                options.linecolor = "#" + parseInt(50).toString(16) + "0000ff";
                options.linewidth = coll.LineStyle.LineWidth;
                options.selectable = coll.AltitudeType;
                options.selectable = coll.Selectable;
                options.editable = coll.Editable;
                options.drawOrder = coll.DrawOrder;
                options.desc = "";
                options.shadow = 1;

                var guid = earth.Factory.CreateGuid();
                options.guid = guid;
                options.lineLength = coll.Length;
                var outPoly;
                var elementLine;
                if (options.type === 220) {
                    outPoly = earth.GeometryAlgorithm.CreateParallelLine(coll.GetPointArray(), offset, 1);
                    elementLine = earth.Factory.CreateElementLine(guid, options.name);
                    elementLine.BeginUpdate();
                    elementLine.SetPointArray(outPoly);
                } else {
                    outPoly = earth.GeometryAlgorithm.CreateParallelLine(coll.GetControlPointArray(), offset, 1);
                    elementLine = earth.Factory.CreateElementCurve(guid, options.name);
                    elementLine.BeginUpdate();
                    elementLine.SetControlPointArray(outPoly);
                }
                //todo:修改默认颜色为#0000ff
                elementLine.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase());
                //alert(coll.LineStyle.LineWidth);
                elementLine.LineStyle.LineWidth = options.linewidth;
                elementLine.AltitudeType = options.shadow;
                elementLine.IsAddArrow = coll.IsAddArrow;
                elementLine.Visibility = true;
                //options.shadow = elementLine.AltitudeType;
                //新增属性
                elementLine.Selectable = options.selectable;
                elementLine.Editable = options.editable;
                elementLine.DrawOrder = options.drawOrder;

                elementLine.EndUpdate();
                options.linecolor = "#" + elementLine.LineStyle.LineColor.toString(16);
                options.arrow = elementLine.IsAddArrow;
                //options.fillcolor= "#"+elementLine.LineStyle.LineColor.toString(16);
                earth.AttachObject(elementLine);
                _createElement(options, elementLine);
                elements.push(elementLine);

                //结束编辑状态
                earth.ToolManager.SphericalObjectEditTool.Browse();
            }
        } else if (flag === "parallelSurface") { //平行面
            var coll = options.originalGeometry;
            var offset = options.offset;
            if (coll) {
                if (coll.Rtti === 239) {
                    options.type = 239; //圆
                    options.radius = coll.Radius - offset;
                } else if (coll.Rtti === 243) {
                    options.type = 243; //椭圆
                    options.longRadius = coll.LongRadius - offset;
                    options.shortRadius = coll.ShortRadius - offset;
                } else {
                    options.type = 211;
                }
                options.name = coll.Name;
                options.drawOrder = 0;
                options.action = "parallel";
                //属性先从Coll中获取到 增加到userdata中
                options.shadow = coll.AltitudeType;
                options.drawOrder = coll.DrawOrder;
                //修改为自定义颜色
                //options.linecolor = parseInt(0xcc0000ff);
                options.linecolor = "#cc0000ff";
                //options.fillcolor = parseInt("0x" + parseInt(50).toString(16) + "0000ff");
                options.fillcolor = "#" + parseInt(50).toString(16) + "0000ff";
                options.linewidth = coll.LineStyle.LineWidth;
                //周长,面积
                options.perimeter = coll.perimeter;
                options.area = coll.area;

                options.selectable = true;
                options.editable = true;
                options.desc = "";

                var outPoly;
                var guid = earth.Factory.CreateGuid();
                options.guid = guid;
                var polygon;
                if (coll.Rtti === 239) {
                    //outPoly = earth.GeometryAlgorithm.CreateParallelPolygon(coll.SphericalTransform.GetLocation(), offset, 1);
                    polygon = earth.Factory.CreateElementCircle(guid, options.name); //圆
                    polygon.SphericalTransform.SetLocationEx(coll.SphericalTransform.GetLocation().X, coll.SphericalTransform.GetLocation().Y, coll.SphericalTransform.GetLocation().Z);
                    polygon.BeginUpdate();
                    polygon.Radius = options.radius;
                } else if (coll.Rtti === 243) {
                    //outPoly = earth.GeometryAlgorithm.CreateParallelPolygon(coll.SphericalTransform.GetLocation(), offset, 1);
                    polygon = earth.Factory.CreateElementEllipse(guid, options.name); //椭圆
                    polygon.SphericalTransform.SetLocationEx(coll.SphericalTransform.GetLocation().X, coll.SphericalTransform.GetLocation().Y, coll.SphericalTransform.GetLocation().Z);
                    polygon.BeginUpdate();
                    polygon.LongRadius = options.longRadius;
                    polygon.ShortRadius = options.shortRadius;
                } else {
                    outPoly = earth.GeometryAlgorithm.CreateParallelPolygon(coll.GetExteriorRing(), offset, 1);
                    polygon = earth.Factory.CreateElementPolygon(guid, options.name);
                    polygon.BeginUpdate();
                    polygon.SetExteriorRing(outPoly);
                    polygon.DrawOrder = options.drawOrder;
                }
                polygon.LineStyle.LineWidth = options.linewidth;
                polygon.LineStyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase());
                polygon.FillStyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase());
                polygon.AltitudeType = 1;
                //新增属性
                polygon.Selectable = options.selectable;
                polygon.Editable = options.editable;
                polygon.DrawOrder = options.drawOrder;
                polygon.EndUpdate();

                options.linecolor = "#" + polygon.LineStyle.LineColor.toString(16);
                options.fillcolor = "#" + polygon.FillStyle.FillColor.toString(16);

                options.perimeter = coll.perimeter;
                options.area = coll.area;

                earth.AttachObject(polygon);
                _createElement(options, polygon);
                elements.push(polygon);

                //结束编辑状态
                earth.ToolManager.SphericalObjectEditTool.Browse();
            } else {
                alert("您选择的对象不可画平行面！");
            }
        } else if (flag === "surfaceTosurface") { //面面求并, 面面求差, 面面求交
            options.type = 211;
            options.name = "polygon";
            earth.ToolManager.SphericalObjectEditTool.Select();
            var dataArr = [];
            earth.Event.OnSelectChanged = function(x) {
                var selectSet = earth.SelectSet;
                //alert(selectSet.GetCount());
                for (var i = 0; i < selectSet.GetCount(); i++) {
                    var element = selectSet.GetObject(i);
                    if (element.Rtti == 211) {
                        dataArr.push(element.GetPolygon());
                    } else {
                        alert("请选择面");
                        earth.ToolManager.SphericalObjectEditTool.Browse();
                    }
                }
                if (dataArr.length >= 2) {
                    earth.Event.OnSelectChanged = function() {};
                    var relationship = earth.PolygonAlgorithm.PolysRelationship(dataArr[0], dataArr[1]);
                    if (relationship != 3 && relationship != 1) {
                        alert("请选择两个相交的面");
                        return;
                    }
                    var a = earth.PolygonAlgorithm.PolysBoolOperation(dataArr[0], dataArr[1], flagArr[1]);
                    //alert(a.Count);
                    for (var m = 0; m < a.Count; m++) {
                        var guid = earth.Factory.createGuid();
                        options.name = "polygon2";
                        options.guid = guid;
                        var polygon = earth.Factory.CreateElementPolygon(guid, "polygon2");

                        polygon.BeginUpdate();
                        polygon.SetPolygon(a.Items(m));
                        polygon.LineStyle.LineWidth = 1;
                        polygon.LineStyle.LineColor = parseInt(0xff88ffff);
                        polygon.FillStyle.FillColor = parseInt(0xff88ffff);
                        // polygon.DrawOrder = 1;
                        polygon.AltitudeType = 1;
                        polygon.Selectable = true;
                        polygon.Editable = true;
                        polygon.DrawOrder = 0;
                        polygon.EndUpdate();
                        earth.AttachObject(polygon);
                        options.linecolor = "#" + polygon.LineStyle.LineColor.toString(16);
                        options.fillcolor = "#" + polygon.FillStyle.FillColor.toString(16);
                        options.shadow = 1;
                        options.selectable = true;
                        options.editable = true;
                        options.drawOrder = 0;
                        createElement(options, polygon);
                        updateTree(options);
                        userdataArr.push(polygon);
                        earth.Event.OnSelectChanged = function(x) {};
                    }
                    treePIDObject[guid] = 0;
                }
                //earth.Event.OnSelectChanged=function(x){};
            }

        } else if (flag == "createsphere") {
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "sphere";
            }
            options.type = 216;
            options.path = earth.Environment.RootPath;
            options.radius = geometry.Radius;
            var guid = earth.Factory.CreateGUID();
            options.guid = guid;
            var elementSphere = earth.Factory.CreateElementSphere(guid, options.name);
            elementSphere.BeginUpdate();
            //新增属性
            elementSphere.Selectable = options.selectable;
            elementSphere.Editable = options.editable;

            elementSphere.Underground = options.objectFlagType;

            elementSphere.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementSphere.Radius = options.radius;
            var materialStyles = elementSphere.MaterialStyles;
            var count = materialStyles.Count;
            //for(var i=0; i<count; i++){
            var materialStyle = materialStyles.Items(0);
            materialStyle.DiffuseTexture = options.texturePath[2];
            //}
            ////12632256
            ////parseInt("0x"+options.fillcolor.toString().substring(1).toLowerCase()  )
            elementSphere.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            elementSphere.EndUpdate();
            _createElement(options, elementSphere);
            earth.AttachObject(elementSphere);
            elements.push(elementSphere);
            earth.ShapeCreator.Clear();
        } else if (flag == "createbox") {
            options.type = 202;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "box";
            }
            polyCount = 6;
            options.path = earth.Environment.RootPath;
            //新增属性
            options.longValue = geometry.Length;
            options.widthValue = geometry.Width;
            options.heightValue = geometry.Height;

            var guid = earth.Factory.CreateGUID();
            options.guid = guid;
            var elementBox = earth.Factory.CreateElementBox(guid, options.name);
            elementBox.BeginUpdate();
            elementBox.Selectable = options.selectable;
            elementBox.Editable = options.editable;

            elementBox.Underground = options.objectFlagType;
            elementBox.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);

            var materialStyles = elementBox.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                if (options.texturePath.length != 0) {
                    materialStyle.DiffuseTexture = options.texturePath[i];
                }
            }
            elementBox.Width = options.widthValue;
            elementBox.Length = options.longValue;
            elementBox.Height = options.heightValue;

            elementBox.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            elementBox.EndUpdate();
            elements.push(elementBox);
            earth.AttachObject(elementBox);
            _createElement(options, elementBox);
            earth.ShapeCreator.Clear();
        } else if (flag == "createvolume") {
            options.type = 207;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "volume";
            }
            options.path = earth.Environment.RootPath;
            options.heightValue = geometry.Height;

            var guid = earth.Factory.CreateGUID();
            options.guid = guid;
            var elementVolume = earth.Factory.CreateElementVolume(guid, options.name);
            elementVolume.BeginUpdate();
            elementVolume.Selectable = options.selectable;
            elementVolume.Editable = options.editable;

            elementVolume.Underground = options.objectFlagType;
            elementVolume.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementVolume.Height = options.heightValue;
            elementVolume.Vectors = geometry.Vector3s;
            elementVolume.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            polyCount = geometry.Vector3s.Count + 2;
            var materialStyles = elementVolume.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                materialStyle.DiffuseTexture = options.texturePath[i];
            }
            elementVolume.EndUpdate();
            _createElement(options, elementVolume);
            earth.AttachObject(elementVolume);
            elements.push(elementVolume);
            earth.ShapeCreator.Clear();
        } else if (flag == "createcylinder") {
            polyCount = 3;
            options.type = 203;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "cylinder";
            }
            options.path = earth.Environment.RootPath;
            options.heightValue = geometry.Height;
            options.bottomRadius = geometry.Radius;

            var guid = earth.Factory.CreateGUID();
            options.guid = guid;
            var elementCylinder = earth.Factory.CreateElementCylinder(guid, options.name);
            elementCylinder.BeginUpdate();
            elementCylinder.Selectable = options.selectable;
            elementCylinder.Editable = options.editable;
            elementCylinder.Underground = options.objectFlagType;
            elementCylinder.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementCylinder.Radius = options.bottomRadius;
            elementCylinder.Height = options.heightValue;
            elementCylinder.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            var materialStyles = elementCylinder.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                materialStyle.DiffuseTexture = options.texturePath[i];
            }
            elementCylinder.EndUpdate();
            earth.AttachObject(elementCylinder);
            _createElement(options, elementCylinder);
            elements.push(elementCylinder);
            earth.ShapeCreator.Clear();
        } else if (flag == "createcone") { //创建圆锥
            polyCount = 3;
            options.type = 204;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "cone";
            }
            var guid = earth.Factory.CreateGUID();
            options.path = earth.Environment.RootPath;
            options.heightValue = geometry.Height;
            options.bottomRadius = geometry.BottomRadius;
            options.topRadius = geometry.TopRadius;
            options.guid = guid;
            var elementCone = earth.Factory.CreateElementCone(guid, options.name);
            elementCone.BeginUpdate();
            elementCone.Selectable = options.selectable;
            elementCone.Editable = options.editable;
            elementCone.Underground = options.objectFlagType;
            elementCone.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementCone.BottomRadius = options.bottomRadius;
            elementCone.TopRadius = options.topRadius;
            elementCone.Height = options.heightValue;
            elementCone.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); // 12632256;
            var materialStyles = elementCone.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                materialStyle.DiffuseTexture = options.texturePath[i];
            }
            elementCone.EndUpdate();
            earth.AttachObject(elementCone);
            _createElement(options, elementCone);
            elements.push(elementCone);
            earth.ShapeCreator.Clear();
        } else if (flag == "createprism") { //创建棱柱
            options.type = 205;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "prism";
            }
            options.sides = 5;
            var guid = earth.Factory.CreateGUID();
            options.path = earth.Environment.RootPath;
            options.heightValue = geometry.Height;
            options.bottomRadius = geometry.Radius;
            options.guid = guid;
            var elementPrism = earth.Factory.CreateElementPrism(guid, options.name);
            elementPrism.BeginUpdate();
            elementPrism.Selectable = options.selectable;
            elementPrism.Editable = options.editable;
            elementPrism.Underground = options.objectFlagType;
            elementPrism.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementPrism.Sides = geometry.Sides;
            elementPrism.Radius = options.bottomRadius;
            elementPrism.Height = options.heightValue;
            elementPrism.Sides = options.sides;
            elementPrism.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            var materialStyles = elementPrism.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                materialStyle.DiffuseTexture = options.texturePath[i];
            }

            elementPrism.EndUpdate();
            earth.AttachObject(elementPrism);
            _createElement(options, elementPrism);
            elements.push(elementPrism);
            earth.ShapeCreator.Clear();
        } else if (flag == "createpyramid") { //创建棱椎
            options.type = 206;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "pyramid";
            }
            options.sides = 5;
            options.path = earth.Environment.RootPath;
            options.heightValue = geometry.Height;
            options.bottomRadius = geometry.BottomRadius;
            options.topRadius = geometry.TopRadius;
            var guid = earth.Factory.CreateGUID();
            options.guid = guid;
            var elementPyramid = earth.Factory.CreateElementPyramid(guid, options.name);
            elementPyramid.BeginUpdate();
            elementPyramid.Selectable = options.selectable;
            elementPyramid.Editable = options.editable;
            elementPyramid.Underground = options.objectFlagType;
            elementPyramid.SphericalTransform.SetLocationEx(geometry.Longitude, geometry.Latitude, geometry.Altitude);
            elementPyramid.Sides = geometry.Sides;
            elementPyramid.BottomRadius = options.bottomRadius;
            elementPyramid.TopRadius = options.topRadius;
            elementPyramid.Height = options.heightValue;
            elementPyramid.Sides = options.sides;
            elementPyramid.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase()); //12632256;
            var materialStyles = elementPyramid.MaterialStyles;
            var count = materialStyles.Count;
            for (var i = 0; i < count; i++) {
                var materialStyle = materialStyles.Items(i);
                materialStyle.DiffuseTexture = options.texturePath[i];
            }
            elementPyramid.EndUpdate();
            earth.AttachObject(elementPyramid);
            _createElement(options, elementPyramid);
            elements.push(elementPyramid);
            earth.ShapeCreator.Clear();
        } else if (flag == "createdWater") {
            if (geometry.Count) {
                options.type = "dWater";
                if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                    options.name = "dWater";
                }
                var particleType = "dWater"
                options.path = earth.Environment.RootPath;
                var guid = earth.Factory.CreateGUID();
                options.guid = guid;
                var elementWater = earth.Factory.CreateElementWater(guid, options.name);
                elementWater.BeginUpdate();
                var polygon = earth.Factory.CreatePolygon();
                polygon.AddRing(geometry);
                elementWater.SetPolygon(polygon);
                elementWater.EndUpdate();
                earth.AttachObject(elementWater);

                _createElement(options, elementWater);
                elements.push(elementWater);
                earth.ShapeCreator.Clear();
            }
        }
        objects.push(options);
        return options;
    };

    /**
     * 创建军标图形
     * @param  {seEnums.seMiltiaryTagType} miltiaryTagType 军标类型
     * @param {object} geometry 军标的几何信息
     * @param {object} options 军标的属性信息
     * @return {object}                 返回创建的军标
     */
    function _createMilitaryTag(miltiaryTagType, geometry, options) {
        var flag = miltiaryTagType;
        options.action = "add";
        options.earth = earth;
        if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
            options.name = flag;
        }
        if (geometry.Count) {
            var sarrow;
            var guid = earth.Factory.CreateGuid();
            if (flag == "sArrow") {
                sarrow = earth.Factory.CreateElementPlotSArrow(guid, options.name);
                options.type = "250";
            } else if (flag == "customArrow") {
                sarrow = earth.Factory.CreateElementPlotCustomArrow(guid, options.name);
                options.type = "253";
            } else if (flag == "tailArrow") {
                sarrow = earth.Factory.CreateElementPlotTailSArrow(guid, options.name);
                options.type = "252";
            } else if (flag == "customTailArrow") {
                sarrow = earth.Factory.CreateElementPlotCustomTailArrow(guid, options.name);
                options.type = "254";
            } else if (flag == "equalSArrow") {
                sarrow = earth.Factory.CreateElementPlotEqualSArrow(guid, options.name);
                options.type = "251";
            } else if (flag == "doubleArrow") {
                sarrow = earth.Factory.CreateElementPlotDoubleArrow(guid, options.name);
                options.type = "255";
            } else if (flag == "xArrow") {
                sarrow = earth.Factory.CreateElementPlotXArrow(guid, options.name);
                options.type = "256";
            } else if (flag == "assemblyArea") {
                sarrow = earth.Factory.CreateElementPlotAssemblyArea(guid, options.name);
                options.type = "260";
            } else if (flag == "triangleFlag") {
                sarrow = earth.Factory.CreateElementPlotTriangleFlag(guid, options.name);
                options.type = "257";
            } else if (flag == "rectFlag") {
                sarrow = earth.Factory.CreateElementPlotRectFlag(guid, options.name);
                options.type = "258";
            } else if (flag == "curveFlag") {
                sarrow = earth.Factory.CreateElementPlotCurveFlag(guid, options.name);
                options.type = "259";
            }

            var militaryData;

            options.guid = sarrow.Guid;
            //obj.type=flag;
            sarrow.BeginUpdate();
            sarrow.name = options.name;
            //sarrow.AltitudeType = 1;
            sarrow.SetControlPointArray(geometry);
            var linestyle = sarrow.LineStyle;
            linestyle.LineWidth = options.linewidth;
            //alert(obj.linecolor);
            if (options.linecolor) {
                linestyle.LineColor = parseInt("0x" + options.linecolor.toString().substring(1).toLowerCase());
            }
            var fillstyle = sarrow.FillStyle;
            if (options.fillcolor) {
                fillstyle.FillColor = parseInt("0x" + options.fillcolor.toString().substring(1).toLowerCase());
            }
            //新增属性[DrawOrder, Selectable, Editable]
            //sarrow.DrawOrder = obj.drawOrder;
            sarrow.selectable = options.selectable;
            sarrow.editable = options.editable;

            if (flag != "triangleFlag" && flag != "rectFlag" && flag != "curveFlag") {
                sarrow.AltitudeType = options.Shadow;
            }

            sarrow.EndUpdate();
            earth.AttachObject(sarrow);
            _createElement(options, sarrow);
            elements.push(sarrow);
            objects.push(options);
            earth.ShapeCreator.Clear();
        }
        return options;
    };


    /**
     * 导入模型
     * @param  {seEnums.seImportModelType} importModelType 导入的模型的种类
     * @param  {object} geometry        导入的位置几何信息
     * @param  {object} options         导入的模型的一些其他信息
     * @return 导入的新模型              
     */
    function _importModelData(importModelType, geometry, options) {
        options.action = "add";
        options.earth = earth;
        earth.ShapeCreator.Clear();
        var flag = importModelType;
        if (typeof options == 'undefined' || options == null) {
            return null;
        }
        if (flag === "model" || flag === "tree" || flag === "match") {
            options.path = earth.Environment.RootPath;
            options.flag = flag;
            var tag = 1;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                if (flag === "model") {
                    options.name = "模型";
                    tag = 1;
                } else if (flag === "tree") {
                    options.name = "树";
                    tag = 2;
                } else if (flag === "match") {
                    options.name = "小品";
                    tag = 3;
                }
            }
            options.tag = tag;
            var guid = earth.Factory.CreateGuid();
            options.guid = guid;
            options.type = 229;
            options.longitude = geometry.Longitude;
            options.latitude = geometry.Latitude;
            options.altitude = geometry.Altitude;

            var model = earth.Factory.CreateEditModelByLocal(options.guid, options.name, options.link, tag);
            //model.SetIsCollapse(true);
            model.SphericalTransform.SetLocationEx(options.longitude, options.latitude, options.altitude);
            model.name = options.name;
            model.Selectable = true;
            model.Editable = true;
            earth.AttachObject(model);
            elements.push(model);
            _createElement(options, model);
        } else if (flag == "picture") {
            options.path = earth.Environment.RootPath;
            var guid = earth.Factory.CreateGuid();
            options.guid = guid;
            options.type = 217;
            options.longitude = geometry.Longitude;
            options.latitude = geometry.Latitude;
            options.altitude = geometry.Altitude;
            options.image = options.iconFileName;

            var billboard = earth.Factory.CreateElementSimpleBillboard(options.guid, options.name);
            billboard.SphericalTransform.SetLocationEx(options.longitude, options.latitude, options.altitude);
            billboard.BeginUpdate();
            billboard.name = options.name;
            billboard.Width = options.width;
            billboard.Height = options.height;
            billboard.Image = options.image; //"G:\\project\\UniBrowser_3.0\\root\icon\\icon.dds";//
            billboard.Visibility = true;
            billboard.EndUpdate();
            earth.AttachObject(billboard);
            elements.push(billboard);
            _createElement(options, billboard);
        } else if (flag == "simplebuilding") {
            options.path = earth.Environment.RootPath;
            options.floorsAllHeight = geometry.Height;
            options.earth = earth;
            options.guid = earth.Factory.CreateGUID();
            options.type = 280;
            options.vector3s = geometry.Vector3s;
            options.longitude = geometry.Longitude;
            options.latitude = geometry.Latitude;
            options.altitude = geometry.Altitude;
            if (typeof options.name == 'undefined' || options.name == null || options.name == '') {
                options.name = "simplebuilding";
            }

            var simpleBuilding = earth.factory.CreateSimpleBuilding(options.guid, options.name);
            simpleBuilding.SphericalTransform.SetLocationEx(options.longitude, options.latitude, options.altitude);
            simpleBuilding.BeginUpdate();
            var polygon = earth.factory.CreatePolygon();
            polygon.AddRing(options.vector3s);
            simpleBuilding.SetPolygon(0, polygon);
            var floorCount = parseInt(options.floorCount);
            var floorHeight = parseFloat(options.floorHeight);
            simpleBuilding.SetFloorsHeight(floorHeight * floorCount);
            simpleBuilding.SetFloorHeight(floorHeight);
            simpleBuilding.SetRoofType(options.roofTypeNode);
            var roofcolor = parseInt("0x" + options.roofColor.toString().substring(1).toLowerCase());
            var floorcolor = parseInt("0x" + options.floorColor.toString().substring(1).toLowerCase());
            simpleBuilding.FloorsColor = floorcolor;
            simpleBuilding.RoofColor = roofcolor;

            var floorMats = simpleBuilding.GetFloorsMaterialStyles();
            floorMats.Items(0).DiffuseTexture = options.roofTexture;
            floorMats.Items(1).DiffuseTexture = options.roofTexture;
            for (var i = 2; i < floorMats.Count; i++) {
                floorMats.Items(i).DiffuseTexture = options.floorTexture;
            }
            var roofMats = simpleBuilding.GetRoofMaterialStyles();
            for (var i = 0; i < roofMats.Count; i++) {
                roofMats.Items(i).DiffuseTexture = options.roofTexture;
            }

            simpleBuilding.EndUpdate();
            earth.ShapeCreator.Clear();
            earth.AttachObject(simpleBuilding);
            elements.push(simpleBuilding);
            _createElement(options, simpleBuilding);
        } else if (flag == "grid") { //晕渲图
            options.path = earth.Environment.RootPath;
            options.name = "grid";
            options.type = 244;
            if (EditLayer === null) {
                createLayer();
            }
            options.guid = earth.Factory.CreateGuid();
            options.longitude = geometry.Longitude;
            options.latitude = geometry.Latitude;
            options.altitude = geometry.Altitude;
            var grid = earth.Factory.CreateElementGrid(options.guid, options.name, options.path, options.height /*高度2千米*/ );
            grid.name = options.name;
            earth.AttachObject(grid);

            userdataArr.push(grid);
            _createElement(options, grid);
            elements.push(grid);
        }
        objects.push(options);
        return options;
    };

    function _createParallelElement(primitiveType, options) {

    };

    /**
     * 根据创建的用户数据类型生成对应的xml字符串
     * @param  {object} obj    模型参数
     * @param  {object} elment 模型
     * @return {string}        生成的xml字符串
     */
    function _createElementXml(obj, elment) {
        var xmlData = "";
        xmlData += "<Element id='" + obj.guid + "' name='" + obj.name + "' shadow_cast='1' type='" + obj.type + "' checked='1' >";
        xmlData += " <Visibility>true</Visibility>";
        if (obj.desc != undefined) {
            xmlData += " <Description>" + obj.desc + "</Description>";
        } else {
            xmlData += " <Description></Description>";
        }
        xmlData += " <RenderStyle>";

        //这里要针对动态对象单独处理 不用转类型
        if (!(obj.type === "fire" || obj.type === "mist" || obj.type === "fountain" || obj.type === "nozzle" || obj.type === "SprayNozzle")) {
            obj.type = Number(obj.type);
        }

        //二维xml格式
        if (obj.type === 220 || obj.type === 239 || obj.type === 243 || obj.type === 240 || obj.type === 241 || obj.type === 211 || obj.type === 245) {
            var lineColor = elment.LineStyle.LineColor;
            xmlData += "  <LineColor>" + obj.linecolor + "</LineColor>";
            xmlData += "  <LineWidth>" + elment.LineStyle.LineWidth + "</LineWidth>";
            if (elment.FillStyle) {
                xmlData += "  <FillColor>" + obj.fillcolor + "</FillColor>";
            }
            // else if(obj.fillcolor){
            //     xmlData += "  <FillColor>" +obj.fillcolor+ "</FillColor>";
            // }

            xmlData += "  <ShadowStyle>";
            //alert(obj.shadow);
            xmlData += "   <ShadowType>" + obj.shadow + "</ShadowType>";
            xmlData += "   <ShadowColor>" + obj.linecolor + "</ShadowColor>";
            xmlData += "  </ShadowStyle>";

            xmlData += "  <Selectable>" + obj.selectable + "</Selectable>";
            xmlData += "  <Editable>" + obj.editable + "</Editable>";
            //由于圆 椭圆与扇形还没有DrawOrder属性 因此这里暂不添加
            if (obj.type === 220 || obj.type === 211 || obj.type === 241 || obj.type === 240 || obj.type === 243) {
                xmlData += " <DrawOrder>" + obj.drawOrder + "</DrawOrder>";
            }
            var vector3s = "";
            if (obj.type === 220) {
                xmlData += "  <isShadowArrow>" + obj.arrow + "</isShadowArrow>";
                xmlData += "  <Length>" + obj.lineLength + "</Length>";
                vector3s = elment.GetPointArray();
            } else if (obj.type === 211) {
                vector3s = elment.GetExteriorRing();
            } else if (obj.type === 239) {
                vector3s = "";
                xmlData += "   <Radius>" + elment.Radius + "</Radius>";
            } else if (obj.type === 241) {
                xmlData += "   <isShadowArrow>" + obj.arrow + "</isShadowArrow>";
                vector3s = elment.GetControlPointArray();
                //vector3s=elment.GetControlPointArray();
            } else if (obj.type === 240) {
                vector3s = "";
                var ArcCenterStr = " " + elment.ArcCenter.x + "," + elment.ArcCenter.y + "," + elment.ArcCenter.z;
                xmlData += "   <ArcCenter>" + ArcCenterStr + "</ArcCenter>";
                xmlData += "   <Angle>" + elment.Angle + "</Angle>";
                xmlData += "   <Radius>" + obj.radius + "</Radius>";
            } else if (obj.type === 243) {
                vector3s = "";
                xmlData += "   <LongRadius>" + elment.LongRadius + "</LongRadius>";
                xmlData += "   <ShortRadius>" + elment.ShortRadius + "</ShortRadius>";
            } else if (obj.type === 245) {
                vector3s = elment.GetExteriorRing();
                xmlData += "   <TextureImagePath>" + obj.picture + "</TextureImagePath>";
                xmlData += "   <TextureTiltX>" + obj.expandX + "</TextureTiltX>";
                xmlData += "   <TextureTiltY>" + obj.expandY + "</TextureTiltY>";
                xmlData += "   <TextureMode>" + obj.textture + "</TextureMode>";
            }

            //新增接口 obj.type==="texturePolygon" || obj.type==="rectangle" 这两者不支持周长与面积
            if (obj.type === 211 || obj.type === 239 || obj.type === 240 || obj.type === 243) {
                xmlData += "  <Perimeter>" + elment.perimeter + "</Perimeter>";
                xmlData += "  <Area>" + elment.area + "</Area>";
            }

            var ptString = "";
            for (var i = 0; i < vector3s.Count; i++) {
                ptString += " " + vector3s.Items(i).x + "," + vector3s.Items(i).y + "," + vector3s.Items(i).z;
            }
            xmlData += " <Coordinates>" + ptString + "</Coordinates>";
        }

        xmlData += " </RenderStyle>";

        //军标xml
        if (obj.type == 250 || obj.type == 251 || obj.type == 252 || obj.type == 253 || obj.type == 254 || obj.type == 255 || obj.type == 256 || obj.type == 257 || obj.type == 258 || obj.type == 259 || obj.type == 260) {
            xmlData += "  <LineColor>" + obj.linecolor + "</LineColor>";
            xmlData += "  <LineWidth>" + elment.LineStyle.LineWidth + "</LineWidth>";
            xmlData += "  <FillColor>" + obj.fillcolor + "</FillColor>";
            // xmlData += "  <Transparency>" +obj.transparency+ "</Transparency>";
            xmlData += "   <Shadow>" + obj.Shadow + "</Shadow>";
            var vector3s = elment.GetControlPointArray();
            var ptString = "";
            for (var i = 0; i < vector3s.Count; i++) {
                ptString += " " + vector3s.Items(i).x + "," + vector3s.Items(i).y + "," + vector3s.Items(i).z;
            }
            xmlData += " <Coordinates>" + ptString + "</Coordinates>";

            //新增节点
            if (obj.type == 250 || obj.type == 251 || obj.type == 252 || obj.type == 253 || obj.type == 254 || obj.type == 255 || obj.type == 256 || obj.type == 260) {
                xmlData += " <DrawOrder>" + obj.drawOrder + "</DrawOrder>";
                xmlData += " <Selectable>" + obj.selectable + "</Selectable>";
                xmlData += " <Editable>" + obj.editable + "</Editable>";
            }
        }
        //三维对象
        if (obj.type === 216 || obj.type === 202 || obj.type === 207 || obj.type === 203 ||
            obj.type === 204 || obj.type === 205 || obj.type === 206) { //三维xml格式
            xmlData += "<TextureParams>";
            xmlData += "    <UniqueSideStyle>" + "" + "</UniqueSideStyle>";
            xmlData += "  <FillColor>" + obj.fillcolor + "</FillColor>";

            //新增属性
            xmlData += "  <Selectable>" + obj.selectable + "</Selectable>";
            xmlData += "  <Editable>" + obj.editable + "</Editable>";
            xmlData += "  <ObjectFlagType>" + obj.objectFlagType + "</ObjectFlagType>";

            var count = elment.MaterialStyles.Count;
            for (var i = 0; i < count; i++) {
                xmlData += "    <TextureParam>";
                xmlData += "        <TexturePath>" + obj.texturePath[i] + "</TexturePath>";
                xmlData += "    </TextureParam>";
            }
            xmlData += "</TextureParams>";
            xmlData += "  <SphericalTransform><Location>" + elment.SphericalTransform.Longitude + "," + elment.SphericalTransform.Latitude + "," + elment.SphericalTransform.Altitude + "</Location></SphericalTransform>";

            if (obj.type === 216) {
                xmlData += "   <Radius>" + elment.radius + "</Radius>";
                xmlData += "        <TexturePath>" + obj.texturePath[2] + "</TexturePath>";
                //xmlData += "  <Sphere><Radius>"+elment.Radius+"</Radius></Sphere>";
            } else if (obj.type === 202) {
                xmlData += "  <Box><Width>" + elment.Width + "</Width>";
                xmlData += "  <Length>" + elment.Length + "</Length>";
                xmlData += "  <Height>" + elment.Height + "</Height></Box>";
            } else if (obj.type === 207) {
                var pList = "";
                var vector3s = elment.Vectors;
                for (var i = 0; i < vector3s.Count; i++) {
                    var point = vector3s.Items(i);
                    pList += point.X + "," + point.Y + "," + point.Z + " ";
                }
                xmlData += "  <Volume><PointList>" + pList + " </PointList>";
                xmlData += " <Height>" + obj.heightValue + "</Height></Volume>";
            } else if (obj.type === 203) { //圆柱
                xmlData += "  <Cylinder><Radius>" + obj.bottomRadius + "</Radius>";
                xmlData += "  <Height>" + obj.heightValue + "</Height></Cylinder>";
            } else if (obj.type === 204) { //圆锥
                xmlData += "  <Cone><Radius>" + obj.bottomRadius + "</Radius>";
                xmlData += "  <TopRadius>" + obj.topRadius + "</TopRadius>";
                xmlData += "  <Height>" + obj.heightValue + "</Height></Cone>";
            } else if (obj.type === 205) { //棱柱
                xmlData += "  <Prism><Sides>" + obj.sides + "</Sides>";
                xmlData += "  <Radius>" + obj.bottomRadius + "</Radius>";
                xmlData += "  <Height>" + obj.heightValue + "</Height></Prism>";
            } else if (obj.type === 206) { //棱锥
                xmlData += "  <Pyramid><Sides>" + obj.sides + "</Sides>";
                xmlData += "  <BottomRadius>" + obj.bottomRadius + "</BottomRadius>";
                xmlData += "  <TopRadius>" + obj.topRadius + "</TopRadius>";
                xmlData += "  <Height>" + obj.heightValue + "</Height></Pyramid>";
            }
        }
        if (obj.type === 229) { //model
            xmlData += " <EditModel>";
            xmlData += "  <Link>" + obj.link + "</Link>";
            xmlData += "  <Pivot>0.00000000,0.00000000,0.00000000</Pivot>";
            xmlData += "  <BBox>";
            xmlData += "   <MinBoundary>-10,0.0,-10</MinBoundary>";
            xmlData += "   <MaxBoundary>10,10,10</MaxBoundary>";
            if (obj.tag) {
                xmlData += "   <Tag>" + obj.tag + "</Tag>";
            }
            xmlData += "  </BBox>";
            xmlData += " </EditModel>";
        } else if (obj.type === 217) { //picture 
            xmlData += " <Width>" + obj.width + "</Width>";
            xmlData += " <Height>" + obj.height + "</Height>";
            xmlData += " <RenderStyle>";
            xmlData += "  <Icon>" + obj.iconFileName + "</Icon>";
            xmlData += " </RenderStyle>";
            xmlData += "<Location>" + obj.longitude + "," + obj.latitude + "," + obj.altitude + "</Location>";

        } else if (obj.type === 209) { //icon
            xmlData += " <RenderStyle type='normal'>";
            xmlData += "  <Icon>" + obj.iconNormalFileName + "</Icon>";
            xmlData += " </RenderStyle>";
            xmlData += " <RenderStyle type='selected'>";
            xmlData += "  <Icon>" + obj.iconSelectedFileName + "</Icon>";
            xmlData += " </RenderStyle>";
            xmlData += " <Location>" + obj.longitude + "," + obj.latitude + "," + obj.altitude + "</Location>";
            //新增属性
            xmlData += "  <TextFormat>" + obj.textFormat + "</TextFormat>";
            xmlData += "  <TextColor>" + obj.textColor + "</TextColor>";
            xmlData += "  <TextHorizontalScale>" + obj.textHorizontalScale + "</TextHorizontalScale>";
            xmlData += "  <TextVerticalScale>" + obj.textVerticalScale + "</TextVerticalScale>";
            xmlData += "  <ShowHandle>" + obj.showHandle + "</ShowHandle>";
            xmlData += "  <HandleHeight>" + obj.handleHeight + "</HandleHeight>";
            xmlData += "  <HandleColor>" + obj.handleColor + "</HandleColor>";
            xmlData += "  <MinVisibleRange>" + obj.minVisibleRange + "</MinVisibleRange>";
            xmlData += "  <MaxVisibleRange>" + obj.maxVisibleRange + "</MaxVisibleRange>";
            xmlData += "  <Selectable>" + obj.selectable + "</Selectable>";
            xmlData += "  <Editable>" + obj.editable + "</Editable>";

        } else if (obj.type === 280) {
            xmlData += " <FloorHeight>" + obj.floorHeight + "</FloorHeight>";
            xmlData += " <FloorCount>" + obj.floorCount + "</FloorCount>";
            xmlData += " <RoofType>" + obj.roofTypeNode + "</RoofType>";
            xmlData += " <FloorTexture>" + obj.floorTexture + "</FloorTexture>";
            xmlData += " <RoofTexture>" + obj.roofTexture + "</RoofTexture>";
            xmlData += " <RoofColor>" + obj.roofColor + "</RoofColor>";
            xmlData += " <FloorColor>" + obj.floorColor + "</FloorColor>";

            xmlData += " <Location>" + obj.longitude + "," + obj.latitude + "," + obj.altitude + "</Location>";
            var pList = "";
            var vector3s = elment.GetPolygon(0).GetRingAt(0);
            for (var i = 0; i < vector3s.Count; i++) {
                var point = vector3s.Items(i);
                pList += point.X + "," + point.Y + "," + point.Z + " ";
            }
            xmlData += "  <SimpleBuilding><PointList>" + pList + " </PointList></SimpleBuilding>";
        } else if (obj.type === 244) {
            xmlData += " <Height>" + obj.height + "</Height>";
            xmlData += " <Grid>";
            xmlData += "  <Link>" + obj.path + "</Link>";
            xmlData += " </Grid>";
        }

        var Rotation = elment.SphericalTransform.GetRotation();
        var Scale = elment.SphericalTransform.GetScale();
        var Position = elment.SphericalTransform.GetLocation();

        xmlData += " <ControlParams>";
        xmlData += "    <Rotation>" + Rotation.X + "," + Rotation.Y + "," + Rotation.Z + "</Rotation>";
        xmlData += "    <Scale>" + Scale.X + "," + Scale.Y + "," + Scale.Z + "</Scale>";
        xmlData += "    <Position>" + Position.X + "," + Position.Y + "," + Position.Z + "</Position>";
        xmlData += " </ControlParams>";
        //alert(Position.X+","+Position.Y+","+Position.Z);
        var heading = earth.GlobeObserver.Pose.heading;
        var tilt = earth.GlobeObserver.Pose.tilt;
        var range = earth.GlobeObserver.PickRange();

        xmlData += " <LookAt>";
        if (obj.name == "grid") {
            xmlData += "  <Longitude>" + obj.longitude + "</Longitude>";
            xmlData += "  <Latitude>" + obj.latitude + "</Latitude>";
            xmlData += "  <Altitude>" + obj.altitude + "</Altitude>";
        } else {
            xmlData += "  <Longitude>" + elment.SphericalTransform.Longitude + "</Longitude>";
            xmlData += "  <Latitude>" + elment.SphericalTransform.Latitude + "</Latitude>";
            xmlData += "  <Altitude>" + elment.SphericalTransform.Altitude + "</Altitude>";
        }
        xmlData += "  <Heading>" + heading + "</Heading>";
        xmlData += "  <Tilt>" + tilt + "</Tilt>";
        xmlData += "  <Range>" + range + "</Range>";
        xmlData += " </LookAt>";

        xmlData += "</Element>";

        return xmlData;
    };

    /**
     * 通过guid获取对应的element节点
     * @param  {xml} xml  xml
     * @param  {string} guid guid
     * @return {xml}      符合的element节点
     */
    function _getElementByGUID(xml, guid) {
        //根节点
        var rootNode = xml.getElementsByTagName("ElementDocument")[0];
        var id = rootNode.getAttribute("id");
        //允许通过其guid或者pid(0)来获取根节点节点
        if (id === guid) {
            return rootNode;
        }
        var elements = xml.getElementsByTagName('Element');
        var elementFolder = xml.getElementsByTagName('ElementFolder');

        for (var i = 0; i < elements.length; i++) {
            var elementGUID = elements[i].getAttribute("id");
            if (elementGUID == guid) {
                return elements[i];
            }
        }

        for (var i = 0; i < elementFolder.length; i++) {
            var elementFolderGUID = elementFolder[i].getAttribute("id");
            if (elementFolderGUID == guid) {
                return elementFolder[i];
            }
        }
    };

    /**
     * 将创建的模型生成xml并保存到本地xml文件中·
     * @param  {object} obj     模型的相关参数
     * @param  {object} element earth创建的模型
     */
    function _createElement(obj, element) {
        var rootxml = _loadElementsXmlFromLocal();
        var elementXML = _getElementByGUID(rootxml, obj.guid);
        if (elementXML) {
            obj.desc = elementXML.getElementsByTagName("Description")[0].text;
        }
        var xmlData = _createElementXml(obj, element);
        var xmlDoc = globalScope.STAMP.seUtil.loadXMLStr("<xml>" + xmlData + "</xml>");
        var lookupNode = null;
        if (rootxml.childNodes.length > 1) {
            lookupNode = rootxml.childNodes[rootxml.childNodes.length - 1].firstChild;
        } else {
            lookupNode = rootxml.documentElement.firstChild;
        }
        lookupNode.appendChild(xmlDoc.documentElement.firstChild);
        var root = earth.Environment.RootPath + "userdata\\" + XML_FILE_NAME;
        earth.UserDocument.saveXmlFile(root, rootxml.xml);
    };


    /************************************
        定义SeUserData
    ************************************/
    SeUserDataFactory = function(options) {
        this.initialize.apply(this, arguments);
    };

    /************************************
        对SeUserData进行扩展，暴露可供调用函数
    ************************************/
    $.extend(SeUserDataFactory.prototype, {
        initialize: function(options) {
            if (typeof options == 'undefined' || options == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 初始化参数强制需要，请参阅API文档');
                }
                return;
            }
            if (typeof options.earth == 'undefined' || options.earth == null) {
                if (globalScope.debugMode == true) {
                    console.error('err3005, 位置: STAMP.SeLayerManager.initialize. 参数中"earth"属性不能为空');
                    return;
                }
            }
            earth = options.earth;
        },
        getElements: function() {
            return elements;
        },
        getCameras: function() {
            return cameras;
        },
        initializeAllElements: function() {
            _initializeAllElements();
        },
        removeAllElements: function() {},

        createPrimitives: function(primitiveType, geometry, options) {
            return _createPrimitives(primitiveType, geometry, options);
        },

        createMilitaryTag: function(miltiaryTagType, geometry, options) {
            return _createMilitaryTag(miltiaryTagType, geometry, options);
        },

        importModelData:function(importModelType, geometry, options) {
        	return _importModelData(importModelType, geometry, options);
        },

        createParticle: function(particleTypeEnum, point, particleOptions) {
            var guid = earth.Factory.CreateGuid();
            var obj = {};
            obj.action = 'add';
            obj.earth = earth;
            obj.guid = guid;
            var particleType = '';
            if (typeof particleOptions == 'undefined' || particleOptions == null) {
                particleOptions = {
                    name: ''
                };
            }
            if (typeof particleTypeEnum === 'undefined' || particleTypeEnum === null) {
                particleTypeEnum = globalScope.STAMP.seEnums.seParticleType.fire;
            } else {
                switch (particleTypeEnum) {
                    case globalScope.STAMP.seEnums.seParticleType.fire:
                        particleType = obj.name = obj.type = 'fire';
                        break;
                    case globalScope.STAMP.seEnums.seParticleType.mist:
                        particleType = obj.name = obj.type = 'mist';
                        break;
                    case globalScope.STAMP.seEnums.seParticleType.fountain:
                        particleType = obj.name = obj.type = 'fountain';
                        break;
                    case globalScope.STAMP.seEnums.seParticleType.nozzle:
                        particleType = obj.name = obj.type = 'nozzle';
                        break;
                    case globalScope.STAMP.seEnums.seParticleType.SprayNozzle:
                        particleType = obj.name = obj.type = 'SprayNozzle';
                        break;
                    default:
                        particleType = obj.name = obj.type = 'fire';
                        break;
                }
            }
            if (particleOptions.name != '') {
                obj.name = particleOptions.name;
            }
            var particleObj = earth.factory.CreateElementParticle(guid, obj.name);
            particleObj.SphericalTransform.SetLocationEx(point.Longitude, point.Latitude, point.Altitude);
            //参数更新开始
            particleObj.BeginUpdate();
            particleObj.Type = particleTypeEnum;
            particleObj.EndUpdate();
            elements.push(particleObj);
            objects.push(obj);
            earth.AttachObject(particleObj);
            //生成该模型的xml字符串
            _createElement(obj, particleObj);
            return obj;
        }
    });

    if (typeof globalScope == 'undefined') {
        globalScope = window;
    }
    if (typeof globalScope.STAMP == 'undefined') {
        globalScope.STAMP = {};
    }
    globalScope.STAMP.SeUserDataFactory = SeUserDataFactory;

    //check for CommonJS
    if (typeof module !== 'undefined' && module.exports) {
        module.exports = SeUserDataFactory;
    } else if (typeof define === 'function' && define.amd) {
        define([], function() {
            return SeUserDataFactory;
        });
    } else {
        globalScope.STAMP.SeUserDataFactory = SeUserDataFactory;
    }

}).call(this, window);
